@@ -7,7 +7,7 @@ kiara = Kiara.instance()
 
 d = {"col1": [1, 2], "col2": [3, 4]}
 
-df = pd.DataFrame(data=d)
-table_value = kiara.data_registry.register_data(data=df, schema="table")
-print(table_value.dict())
+example_df = pd.DataFrame(data=d)
+table_value = kiara.data_registry.register_data(data=example_df, schema="table")
+print(table_value.model_dump())
 print(table_value.data.arrow_table)
