@@ -9,14 +9,6 @@
 
 __all__ = [
     "get_version",
-    "Kiara",
-    "KiaraAPI",
-    "KiaraModule",
-    "KiaraModuleConfig",
-    "Value",
-    "ValueMap",
-    "ValueMapSchema",
-    "ValueSchema",
 ]
 import logging
 import os
@@ -25,17 +17,12 @@ import sys
 import structlog
 import typing
 
+from .utils import is_develop, is_debug
 from .utils.class_loading import (
     KiaraEntryPointItem,
     find_kiara_model_classes_under,
     find_kiara_renderers_under,
 )
-from .context import Kiara
-from .interfaces.python_api import KiaraAPI
-from .modules import KiaraModule, ValueMapSchema, KiaraModuleConfig
-from .models.values.value import Value, ValueMap
-from .models.values.value_schema import ValueSchema
-
 
 try:
     builtins = __import__("__builtin__")
@@ -69,8 +56,9 @@ try:
         file: typing.Union[typing.IO[str], None] = None,
         flush: bool = False,
     ):
-
         for obj in objects:
+            if hasattr(obj, "create_renderable"):
+                obj = obj.create_renderable()
             try:
                 rich_print(obj, sep=sep, end=end, file=file, flush=flush)
             except Exception:
@@ -87,7 +75,6 @@ try:
         file: typing.Union[typing.IO[str], None] = None,
         flush: bool = False,
     ):
-
         objs = (
             ["[green]----------------------------------------------[/green]"]  # noqa
             + list(objects)
@@ -100,6 +87,13 @@ try:
 except ImportError:  # Graceful fallback if IceCream isn't installed.
     pass
 
+if is_develop() or is_debug():
+    try:
+        from icecream import install
+
+        install()
+    except ImportError:  # Graceful fallback if IceCream isn't installed.
+        pass
 
 """Top-level package for kiara."""
 
@@ -150,7 +144,6 @@ def get_version() -> str:
         dist_name = __name__
         __version__ = get_distribution(dist_name).version
     except DistributionNotFound:
-
         try:
             version_file = os.path.join(os.path.dirname(__file__), "version.txt")
 
@@ -160,10 +153,10 @@ def get_version() -> str:
             else:
                 __version__ = "unknown"
 
-        except (Exception):
+        except Exception:
             __version__ = "unknown"
 
         if __version__ is None:
             __version__ = "unknown"
 
-    return __version__
+    return __version__  # type: ignore
