@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
+import atexit
 import os
 import uuid
+from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Set, Type, Union
 
 import structlog
@@ -8,8 +10,10 @@ import structlog
 # from alembic import command  # type: ignore
 from pydantic import Field
 
-from kiara.context.config import KiaraConfig, KiaraContextConfig, KiaraRuntimeConfig
+from kiara.context.config import KiaraArchiveConfig, KiaraConfig, KiaraContextConfig
+from kiara.context.runtime_config import KiaraRuntimeConfig
 from kiara.data_types import DataType
+from kiara.exceptions import KiaraContextException
 from kiara.interfaces import get_console
 from kiara.interfaces.python_api.models.info import (
     DataTypeClassesInfo,
@@ -27,24 +31,25 @@ from kiara.models.context import ContextInfo
 from kiara.models.module.manifest import Manifest
 from kiara.models.runtime_environment import RuntimeEnvironment
 from kiara.models.values.value import ValueMap
-from kiara.registries import KiaraArchive
+from kiara.registries import KiaraArchive, SqliteArchiveConfig
 from kiara.registries.aliases import AliasRegistry
 from kiara.registries.data import DataRegistry
-from kiara.registries.destinies.registry import DestinyRegistry
 from kiara.registries.environment import EnvironmentRegistry
 from kiara.registries.events.metadata import CreateMetadataDestinies
 from kiara.registries.events.registry import EventRegistry
 from kiara.registries.ids import ID_REGISTRY
 from kiara.registries.jobs import JobRegistry
+from kiara.registries.metadata import MetadataRegistry
 from kiara.registries.models import ModelRegistry
 from kiara.registries.modules import ModuleRegistry
 from kiara.registries.operations import OperationRegistry
 from kiara.registries.rendering import RenderRegistry
 from kiara.registries.types import TypeRegistry
 from kiara.registries.workflows import WorkflowRegistry
-from kiara.utils import log_exception
+from kiara.utils import log_exception, log_message
 from kiara.utils.class_loading import find_all_archive_types
 from kiara.utils.operations import filter_operations
+from kiara.utils.stores import check_external_archive
 
 #  Copyright (c) 2021, University of Luxembourg / DHARPA project
 #  Copyright (c) 2021, Markus Binsteiner
@@ -61,7 +66,6 @@ logger = structlog.getLogger()
 
 def explain(item: Any, kiara: Union[None, "Kiara"] = None):
     """Pretty print information about an item on the terminal."""
-
     if isinstance(item, type):
         from kiara.modules import KiaraModule
 
@@ -75,7 +79,8 @@ def explain(item: Any, kiara: Union[None, "Kiara"] = None):
 
 
 class Kiara(object):
-    """The core context of a kiara session.
+    """
+    The core context of a kiara session.
 
     The `Kiara` object holds all information related to the current environment the user does works in. This includes:
 
@@ -100,16 +105,14 @@ class Kiara(object):
     def instance(cls) -> "Kiara":
         """The default *kiara* context. In most cases, it's recommended you create and manage your own, though."""
 
-        from kiara.interfaces.python_api import KiaraAPI
-
-        return KiaraAPI.instance().context
+        raise NotImplementedError("Kiara.instance() is not implemented yet.")
+        # return BaseAPI.instance().context
 
     def __init__(
         self,
         config: Union[KiaraContextConfig, None] = None,
         runtime_config: Union[KiaraRuntimeConfig, None] = None,
     ) -> None:
-
         kc: Union[KiaraConfig, None] = None
         if not config:
             kc = KiaraConfig()
@@ -127,9 +130,12 @@ class Kiara(object):
         self._config: KiaraContextConfig = config
         self._runtime_config: KiaraRuntimeConfig = runtime_config
 
+        self._env_mgmt: EnvironmentRegistry = EnvironmentRegistry()
+
         self._event_registry: EventRegistry = EventRegistry(kiara=self)
         self._type_registry: TypeRegistry = TypeRegistry(self)
         self._data_registry: DataRegistry = DataRegistry(kiara=self)
+        self._metadata_registry: MetadataRegistry = MetadataRegistry(kiara=self)
         self._job_registry: JobRegistry = JobRegistry(kiara=self)
         self._module_registry: ModuleRegistry = ModuleRegistry(kiara=self)
         self._operation_registry: OperationRegistry = OperationRegistry(kiara=self)
@@ -137,14 +143,12 @@ class Kiara(object):
         self._kiara_model_registry: ModelRegistry = ModelRegistry.instance()
 
         self._alias_registry: AliasRegistry = AliasRegistry(kiara=self)
-        self._destiny_registry: DestinyRegistry = DestinyRegistry(kiara=self)
+        # self._destiny_registry: DestinyRegistry = DestinyRegistry(kiara=self)
 
         self._workflow_registry: WorkflowRegistry = WorkflowRegistry(kiara=self)
 
         self._render_registry = RenderRegistry(kiara=self)
 
-        self._env_mgmt: Union[EnvironmentRegistry, None] = None
-
         metadata_augmenter = CreateMetadataDestinies(kiara=self)
         self._event_registry.add_listener(
             metadata_augmenter, *metadata_augmenter.supported_event_types()
@@ -157,7 +161,31 @@ class Kiara(object):
         self._archives: Dict[str, KiaraArchive] = {}
 
         for archive_alias, archive in self._config.archives.items():
+            # TODO: this is just to make old context that still had that not error out
+            if "_destiny_" in archive.archive_type:
+                continue
+
+            if (
+                archive_alias == "default_job_store"
+                and archive.archive_type == "filesystem_job_store"
+            ):
+                # this is a temporary solution for contexts that still have the old filesystem job store
+                # TODO: remove this at some stage
+
+                archive_path = Path(archive.config["archive_path"])
+                file_name = f"{archive_path.name}.kiarchive"
+
+                js_config = SqliteArchiveConfig.create_new_store_config(
+                    store_base_path=archive.config["archive_path"],
+                    file_name=file_name,
+                    use_wal_mode=True,
+                )
+                archive = KiaraArchiveConfig(
+                    archive_type="sqlite_job_store", config=js_config.model_dump()
+                )
+
             archive_cls = self._archive_types.get(archive.archive_type, None)
+
             if archive_cls is None:
                 raise Exception(
                     f"Can't create context: no archive type '{archive.archive_type}' available. Available types: {', '.join(self._archive_types.keys())}"
@@ -165,34 +193,45 @@ class Kiara(object):
 
             config_cls = archive_cls._config_cls
             archive_config = config_cls(**archive.config)
-            archive_obj = archive_cls(archive_id=archive.archive_uuid, config=archive_config)  # type: ignore
+            archive_obj = archive_cls(
+                archive_name=archive_alias, archive_config=archive_config
+            )  # type: ignore
             for supported_type in archive_obj.supported_item_types():
+                if supported_type == "metadata":
+                    self.metadata_registry.register_metadata_archive(archive_obj)  # type: ignore
                 if supported_type == "data":
                     self.data_registry.register_data_archive(
-                        archive_obj, alias=archive_alias  # type: ignore
+                        archive_obj,  # type: ignore
                     )
                 if supported_type == "job_record":
-                    self.job_registry.register_job_archive(archive_obj, alias=archive_alias)  # type: ignore
+                    self.job_registry.register_job_archive(archive_obj)  # type: ignore
 
                 if supported_type == "alias":
-                    self.alias_registry.register_archive(archive_obj, alias=archive_alias)  # type: ignore
+                    self.alias_registry.register_archive(archive_obj)  # type: ignore
 
-                if supported_type == "destiny":
-                    self.destiny_registry.register_destiny_archive(archive_obj, alias=archive_alias)  # type: ignore
+                # if supported_type == "destiny":
+                #     self.destiny_registry.register_destiny_archive(archive_obj)  # type: ignore
 
                 if supported_type == "workflow":
-                    self.workflow_registry.register_archive(archive_obj, alias=archive_alias)  # type: ignore
+                    self.workflow_registry.register_archive(archive_obj)  # type: ignore
 
-    # def _run_alembic_migrations(self):
-    #     script_location = os.path.abspath(KIARA_DB_MIGRATIONS_FOLDER)
-    #     dsn = self._config.db_url
-    #     log_message("running migration script", script=script_location, db_url=dsn)
-    #     from alembic.config import Config
-    #
-    #     alembic_cfg = Config(KIARA_DB_MIGRATIONS_CONFIG)
-    #     alembic_cfg.set_main_option("script_location", script_location)
-    #     alembic_cfg.set_main_option("sqlalchemy.url", dsn)
-    #     command.upgrade(alembic_cfg, "head")
+        if self._runtime_config.lock_context:
+            self.lock_context()
+
+    def lock_context(self):
+        """Lock the context, so that it can't be used by other processes."""
+        aquired = ID_REGISTRY.lock_context(self.id)
+
+        if not aquired:
+            raise KiaraContextException(
+                "Can't lock context: already locked by another process.",
+                context_id=self.id,
+            )
+
+        atexit.register(self.unlock_context)
+
+    def unlock_context(self):
+        ID_REGISTRY.unlock_context(self.id)
 
     @property
     def id(self) -> uuid.UUID:
@@ -207,7 +246,6 @@ class Kiara(object):
         return self._runtime_config
 
     def update_runtime_config(self, **settings) -> KiaraRuntimeConfig:
-
         for k, v in settings.items():
             setattr(self.runtime_config, k, v)
 
@@ -215,7 +253,6 @@ class Kiara(object):
 
     @property
     def context_info(self) -> "KiaraContextInfo":
-
         if self._context_info is None:
             self._context_info = KiaraContextInfo.create_from_kiara_instance(kiara=self)
         return self._context_info
@@ -225,10 +262,6 @@ class Kiara(object):
 
     @property
     def environment_registry(self) -> EnvironmentRegistry:
-        if self._env_mgmt is not None:
-            return self._env_mgmt
-
-        self._env_mgmt = EnvironmentRegistry.instance()
         return self._env_mgmt
 
     @property
@@ -247,9 +280,9 @@ class Kiara(object):
     def alias_registry(self) -> AliasRegistry:
         return self._alias_registry
 
-    @property
-    def destiny_registry(self) -> DestinyRegistry:
-        return self._destiny_registry
+    # @property
+    # def destiny_registry(self) -> DestinyRegistry:
+    #     return self._destiny_registry
 
     @property
     def job_registry(self) -> JobRegistry:
@@ -264,6 +297,10 @@ class Kiara(object):
     def data_registry(self) -> DataRegistry:
         return self._data_registry
 
+    @property
+    def metadata_registry(self) -> MetadataRegistry:
+        return self._metadata_registry
+
     @property
     def workflow_registry(self) -> WorkflowRegistry:
         return self._workflow_registry
@@ -289,7 +326,7 @@ class Kiara(object):
 
     @property
     def data_type_names(self) -> List[str]:
-        return self.type_registry.data_type_names
+        return self.type_registry.get_data_type_names(include_profiles=True)
 
     @property
     def module_type_classes(self) -> Mapping[str, Type["KiaraModule"]]:
@@ -302,21 +339,79 @@ class Kiara(object):
     # ===================================================================================================
     # kiara session API methods
 
+    def register_external_archive(
+        self,
+        archive: Union[str, KiaraArchive, Iterable[Union[KiaraArchive, str]]],
+        allow_write_access: bool = False,
+    ) -> Dict[str, str]:
+        """Register one or several external archives with the context.
+
+        In case you provide KiaraArchive instances, they will be modified in case the provided 'allow_write_access' is different from the 'is_force_read_only' attribute of the archive.
+        """
+
+        archive_instances = check_external_archive(
+            archive=archive, allow_write_access=allow_write_access
+        )
+
+        result = {}
+        for archive_type, _archive_inst in archive_instances.items():
+            log_message(
+                "register.external.archive",
+                archive=_archive_inst.archive_name,
+                allow_write_access=allow_write_access,
+            )
+
+            _archive_inst.set_force_read_only(not allow_write_access)
+
+            if archive_type == "data":
+                result["data"] = self.data_registry.register_data_archive(_archive_inst)  # type: ignore
+                log_message(
+                    "archive.registered",
+                    archive=_archive_inst.archive_name,
+                    archive_type="data",
+                )
+            elif archive_type == "metadata":
+                result["metadata"] = self.metadata_registry.register_metadata_archive(
+                    _archive_inst  # type: ignore
+                )  # type: ignore
+                log_message(
+                    "archive.registered",
+                    archive=_archive_inst.archive_name,
+                    archive_type="metadata",
+                )
+            elif archive_type == "alias":
+                result["alias"] = self.alias_registry.register_archive(_archive_inst)  # type: ignore
+                log_message(
+                    "archive.registered",
+                    archive=_archive_inst.archive_name,
+                    archive_type="alias",
+                )
+            elif archive_type == "job_record":
+                result["job_record"] = self.job_registry.register_job_archive(
+                    _archive_inst  # type: ignore
+                )  # type: ignore
+                log_message(
+                    "archive.registered",
+                    archive=_archive_inst.archive_name,
+                    archive_type="job_record",
+                )
+            else:
+                raise Exception(f"Can't register archive of type '{archive_type}'.")
+
+        return result
+
     def create_manifest(
         self, module_or_operation: str, config: Union[Mapping[str, Any], None] = None
     ) -> Manifest:
-
         if config is None:
             config = {}
 
         if module_or_operation in self.module_type_names:
-
             manifest: Manifest = Manifest(
                 module_type=module_or_operation, module_config=config
             )
 
         elif module_or_operation in self.operation_registry.operation_ids:
-
             if config:
                 raise Exception(
                     f"Specified run target '{module_or_operation}' is an operation, additional module configuration is not allowed (yet)."
@@ -345,36 +440,39 @@ class Kiara(object):
     def queue(
         self, manifest: Manifest, inputs: Mapping[str, Any], wait: bool = False
     ) -> uuid.UUID:
-        """Queue a job with the specified manifest and inputs.
+        """
+        Queue a job with the specified manifest and inputs.
 
         Arguments:
+        ---------
            manifest: the job manifest
            inputs: the job inputs
            wait: whether to wait for the job to be finished before returning
 
         Returns:
+        -------
             the job id that can be used to look up job status & results
         """
-
         return self.job_registry.execute(manifest=manifest, inputs=inputs, wait=wait)
 
     def process(self, manifest: Manifest, inputs: Mapping[str, Any]) -> ValueMap:
-        """Queue a job with the specified manifest and inputs.
+        """
+        Queue a job with the specified manifest and inputs.
 
         Arguments:
+        ---------
            manifest: the job manifest
            inputs: the job inputs
            wait: whether to wait for the job to be finished before returning
 
-        Returns
+        Returns:
+        -------
         """
-
         return self.job_registry.execute_and_retrieve(manifest=manifest, inputs=inputs)
 
     def save_values(
         self, values: ValueMap, alias_map: Mapping[str, Iterable[str]]
     ) -> StoreValuesResult:
-
         _values = {}
         for field_name in values.field_names:
             value = values.get_value_obj(field_name)
@@ -382,11 +480,12 @@ class Kiara(object):
             self.data_registry.store_value(value=value)
         stored = {}
         for field_name, field_aliases in alias_map.items():
-
             value = _values[field_name]
             try:
                 if field_aliases:
-                    self.alias_registry.register_aliases(value.value_id, *field_aliases)
+                    self.alias_registry.register_aliases(
+                        value_id=value.value_id, aliases=field_aliases
+                    )
 
                 stored[field_name] = StoreValueResult(
                     value=value,
@@ -404,22 +503,23 @@ class Kiara(object):
                     persisted_data=None,
                 )
 
-        return StoreValuesResult.construct(__root__=stored)
+        return StoreValuesResult(root=stored)
 
     def create_context_summary(self) -> ContextInfo:
         return ContextInfo.create_from_context(kiara=self)
 
     def get_all_archives(self) -> Dict[KiaraArchive, Set[str]]:
-
         result: Dict[KiaraArchive, Set[str]] = {}
 
         archive: KiaraArchive
+        for alias, archive in self.metadata_registry.metadata_archives.items():
+            result.setdefault(archive, set()).add(alias)
         for alias, archive in self.data_registry.data_archives.items():
             result.setdefault(archive, set()).add(alias)
         for alias, archive in self.alias_registry.alias_archives.items():
             result.setdefault(archive, set()).add(alias)
-        for alias, archive in self.destiny_registry.destiny_archives.items():
-            result.setdefault(archive, set()).add(alias)
+        # for alias, archive in self.destiny_registry.destiny_archives.items():
+        #     result.setdefault(archive, set()).add(alias)
         for alias, archive in self.job_registry.job_archives.items():
             result.setdefault(archive, set()).add(alias)
         for alias, archive in self.workflow_registry.workflow_archives.items():
@@ -433,7 +533,6 @@ class KiaraContextInfo(KiaraModel):
     def create_from_kiara_instance(
         cls, kiara: "Kiara", package_filter: Union[str, None] = None
     ):
-
         data_types = kiara.type_registry.get_context_metadata(
             only_for_package=package_filter
         )
@@ -457,7 +556,7 @@ class KiaraContextInfo(KiaraModel):
 
         # metadata_types = find_metadata_models(only_for_package=package_filter)
 
-        return KiaraContextInfo.construct(
+        return KiaraContextInfo(
             kiara_id=kiara.id,
             package_filter=package_filter,
             data_types=data_types,
@@ -497,8 +596,7 @@ class KiaraContextInfo(KiaraModel):
         return {"kiara_id": self.kiara_id, "package": self.package_filter}
 
     def get_info(self, item_type: str, item_id: str) -> ItemInfo:
-
-        if item_type == "data_type" or item_type == "data_types":
+        if item_type in ("data_type", "data_types"):
             group_info: InfoItemGroup = self.data_types
         elif "module" in item_type:
             group_info = self.module_types
@@ -521,10 +619,10 @@ class KiaraContextInfo(KiaraModel):
             raise Exception(
                 f"Can't determine item type '{item_type}', use one of: {', '.join(item_types)}"
             )
-        return group_info[item_id]
+        result: ItemInfo = group_info.item_infos[item_id]
+        return result
 
     def get_all_info(self, skip_empty_types: bool = True) -> Dict[str, InfoItemGroup]:
-
         result: Dict[str, InfoItemGroup] = {}
         if self.data_types or not skip_empty_types:
             result["data_types"] = self.data_types
