@@ -5,7 +5,8 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
-"""This is the base module that contains everything data type-related in *kiara*.
+"""
+This is the base module that contains everything data type-related in *kiara*.
 
 I'm still not 100% sure how to best implement the *kiara* type system, there are several ways it could be done, for example
 based on Python type-hints, using JSON-schema, Avro (which is my 2nd favourite option), as well as by implementing a
@@ -23,14 +24,14 @@ using their own serializing functions), or will have to implement custom seriali
 be discouraged, since this might not be trivial and there are quite a few things to consider).
 
 """
+
 import abc
 import uuid
 from typing import TYPE_CHECKING, Any, Generic, Mapping, Tuple, Type, TypeVar, Union
 
-import orjson
 import structlog
 from deepdiff import DeepHash
-from pydantic import BaseModel, Extra, PrivateAttr, ValidationError
+from pydantic import BaseModel, ConfigDict, PrivateAttr, ValidationError
 from rich import box
 from rich.console import Console, ConsoleOptions, RenderResult
 from rich.rule import Rule
@@ -40,7 +41,6 @@ from rich.table import Table
 from kiara.defaults import (
     INVALID_HASH_MARKER,
     INVALID_SIZE_MARKER,
-    KIARA_HASH_FUNCTION,
     NO_SERIALIZATION_MARKER,
     SpecialValue,
 )
@@ -48,13 +48,13 @@ from kiara.exceptions import KiaraValueException, ValueTypeConfigException
 from kiara.models.python_class import PythonClass
 from kiara.models.values import DataTypeCharacteristics, ValueStatus
 from kiara.models.values.value_schema import ValueSchema
+from kiara.utils.hashing import KIARA_HASH_FUNCTION
 
 #
 #     if obj.__class__.__module__ == "builtins":
 #         return obj.__class__.__name__
 #     else:
 #         return f"{obj.__class__.__module__}.{obj.__class__.__name__}"
-from kiara.utils.json import orjson_dumps
 
 if TYPE_CHECKING:
     from kiara.models.values.value import (
@@ -71,23 +71,20 @@ logger = structlog.getLogger()
 
 
 class DataTypeConfig(BaseModel):
-    """Base class that describes the configuration a [``DataType``][kiara.data.data_types.DataType] class accepts.
+    """
+    Base class that describes the configuration a [``DataType``][kiara.data.data_types.DataType] class accepts.
 
     This is stored in the ``_config_cls`` class attribute in each ``DataType`` class. By default,
     a ``DataType`` is not configurable, unless the ``_config_cls`` class attribute points to a sub-class of this class.
     """
 
-    class Config:
-        json_loads = orjson.loads
-        json_dumps = orjson_dumps
-        extra = Extra.forbid
+    model_config = ConfigDict(extra="forbid")
 
     @classmethod
     def requires_config(cls) -> bool:
         """Return whether this class can be used as-is, or requires configuration before an instance can be created."""
-
-        for field_name, field in cls.__fields__.items():
-            if field.required and field.default is None:
+        for field_name, field in cls.model_fields.items():
+            if field.is_required() and field.default is None:
                 return True
         return False
 
@@ -95,8 +92,7 @@ class DataTypeConfig(BaseModel):
 
     def get(self, key: str) -> Any:
         """Get the value for the specified configuation key."""
-
-        if key not in self.__fields__:
+        if key not in self.model_fields.keys():
             raise Exception(
                 f"No config value '{key}' in module config class '{self.__class__.__name__}'."
             )
@@ -105,32 +101,28 @@ class DataTypeConfig(BaseModel):
 
     @property
     def config_hash(self) -> int:
-
         if self._config_hash is None:
-            _d = self.dict()
+            _d = self.model_dump()
             hashes = DeepHash(_d)
             self._config_hash = hashes[_d]
         return self._config_hash
 
     def __eq__(self, other):
-
         if self.__class__ != other.__class__:
             return False
 
-        return self.dict() == other.dict()
+        return self.model_dump() == other.model_dump()
 
     def __hash__(self):
-
         return self.config_hash
 
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
-
         my_table = Table(box=box.MINIMAL, show_header=False)
         my_table.add_column("Field name", style="i")
         my_table.add_column("Value")
-        for field in self.__fields__:
+        for field in self.model_fields.keys():
             my_table.add_row(field, getattr(self, field))
 
         yield my_table
@@ -141,7 +133,8 @@ TYPE_CONFIG_CLS = TypeVar("TYPE_CONFIG_CLS", bound=DataTypeConfig)
 
 
 class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
-    """Base class that all *kiara* data_types must inherit from.
+    """
+    Base class that all *kiara* data_types must inherit from.
 
     *kiara* data_types have 3 main responsibilities:
 
@@ -165,17 +158,17 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
     @classmethod
     @abc.abstractmethod
     def python_class(cls) -> Type[TYPE_PYTHON_CLS]:
-        pass
+        """The Python class that the internal 'data' attribute of a value has."""
 
     @classmethod
     def data_type_config_class(cls) -> Type[TYPE_CONFIG_CLS]:
+        """The Python class that holds the (optional) configuration for a data type instance."""
         return DataTypeConfig  # type: ignore
 
     @classmethod
     def _calculate_data_type_hash(
         cls, data_type_config: Union[Mapping[str, Any], DataTypeConfig]
     ) -> int:
-
         if isinstance(data_type_config, Mapping):
             data_type_config = cls.data_type_config_class()(**data_type_config)  # type: ignore
 
@@ -184,12 +177,14 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
             "type_config": data_type_config.config_hash,
         }
         h = DeepHash(obj, hasher=KIARA_HASH_FUNCTION)
-        return h[obj]
+        result: int = h[obj]
+        return result
 
     def __init__(self, **type_config: Any):
-
         try:
-            self._type_config: TYPE_CONFIG_CLS = self.__class__.data_type_config_class()(**type_config)  # type: ignore  # TODO: double-check this is only a mypy issue
+            self._type_config: TYPE_CONFIG_CLS = (
+                self.__class__.data_type_config_class()(**type_config)
+            )
         except ValidationError as ve:
             raise ValueTypeConfigException(
                 f"Error creating object for type: {ve}",
@@ -216,7 +211,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
     @property
     def info(self) -> "DataTypeInfo":
-
         if self._info is not None:
             return self._info
 
@@ -224,7 +218,7 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
         self._info = DataTypeInfo(
             data_type_name=self.data_type_name,
-            data_type_config=self.type_config.dict(),
+            data_type_config=self.type_config.model_dump(),
             characteristics=self.characteristics,
             data_type_class=PythonClass.from_class(self.__class__),
         )
@@ -248,21 +242,18 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
     def calculate_hash(self, data: "SerializedData") -> str:
         """Calculate the hash of the value."""
-
         return data.instance_id
 
     def calculate_size(self, data: "SerializedData") -> int:
         """Calculate the size of the value."""
-
         return data.data_size
 
     def serialize_as_json(self, data: Any) -> "SerializedData":
-
         _data = {"data": {"type": "inline-json", "inline_data": data, "codec": "json"}}
 
         serialized_data = {
             "data_type": self.data_type_name,
-            "data_type_config": self.type_config.dict(),
+            "data_type_config": self.type_config.model_dump(),
             "data": _data,
             "serialization_profile": "json",
             "metadata": {
@@ -281,7 +272,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
         return serialized
 
     def serialize(self, data: TYPE_PYTHON_CLS) -> Union[None, str, "SerializedData"]:
-
         logger.debug(
             "ignore.serialize_request",
             data_type=self.data_type_name,
@@ -330,7 +320,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
     def _pre_examine_data(
         self, data: Any, schema: ValueSchema
     ) -> Tuple[Any, Union[str, "SerializedData"], ValueStatus, str, int]:
-
         assert data is not None
 
         if data is SpecialValue.NOT_SET:
@@ -366,7 +355,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
             value_hash = INVALID_HASH_MARKER
             serialized: Union[None, str, "SerializedData"] = NO_SERIALIZATION_MARKER
         else:
-
             from kiara.models.values.value import SerializedData
 
             if isinstance(data, SerializedData):
@@ -377,6 +365,7 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
                 not_serialized: bool = False
             else:
                 data = self.parse_python_obj(data)
+
                 if data is None:
                     raise Exception(
                         f"Invalid data, can't parse into a value of type '{schema.type}'."
@@ -384,6 +373,7 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
                 self._validate(data)
 
                 serialized = self.serialize(data)
+
                 if serialized is None:
                     serialized = NO_SERIALIZATION_MARKER
 
@@ -417,7 +407,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
         kiara_id: uuid.UUID,
         pedigree_output_name: str,
     ) -> Tuple["Value", Any]:
-
         from kiara.models.values.value import Value
 
         if isinstance(status, str):
@@ -425,7 +414,6 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
         if status in [ValueStatus.SET, ValueStatus.DEFAULT]:
             try:
-
                 value = Value(
                     value_id=value_id,
                     kiara_id=kiara_id,
@@ -462,40 +450,41 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
         return value, data
 
     def parse_python_obj(self, data: Any) -> TYPE_PYTHON_CLS:
-        """Parse a value into a supported python type.
+        """
+        Parse a value into a supported python type.
 
         This exists to make it easier to do trivial conversions (e.g. from a date string to a datetime object).
         If you choose to overwrite this method, make 100% sure that you don't change the meaning of the value, and try to
         avoid adding or removing information from the data (e.g. by changing the resolution of a date).
 
         Arguments:
+        ---------
             v: the value
 
         Returns:
+        -------
             'None', if no parsing was done and the original value should be used, otherwise return the parsed Python object
         """
 
-        return data
+        # this would in most cases be overwritten by an implementing class
+        # if not, then the _validate method should catch the issue
+        return data  # type: ignore
 
     def _validate(self, value: TYPE_PYTHON_CLS) -> None:
         """Validate the value. This expects an instance of the defined Python class (from 'backing_python_type)."""
-
         if not isinstance(value, self.__class__.python_class()):
             raise ValueError(
                 f"Invalid python type '{type(value)}', must be: {self.__class__.python_class()}"
             )
 
     def create_renderable(self, **config):
-
         show_type_info = config.get("show_type_info", False)
 
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("key")
         table.add_column("value", style="i")
         table.add_row("type_name", self.data_type_name)
-        config_json = self.type_config.json(
-            exclude_unset=True, option=orjson.OPT_INDENT_2
-        )
+        config_json = self.type_config.model_dump_json(exclude_unset=True, indent=2)
         config = Syntax(config_json, "json", background_color="default")
         table.add_row("type_config", config)
 
