@@ -7,8 +7,7 @@
 import uuid
 from typing import TYPE_CHECKING, Any, Iterable, List, Mapping, Type, Union
 
-from orjson import orjson
-from rich.markdown import Markdown
+import orjson
 
 from kiara.defaults import NOT_AVAILBLE_MARKER
 from kiara.utils.json import orjson_dumps
@@ -27,23 +26,31 @@ if TYPE_CHECKING:
 
 class KiaraException(Exception):
     @classmethod
-    def get_root_details(cls, e: Exception) -> Union[str, None]:
-
+    def get_root_details(
+        cls, e: Exception, default: Union[None, str] = None
+    ) -> Union[str, None]:
         if isinstance(e, KiaraException):
             return e.root_details()
         else:
-            return str(e)
+            if default is None:
+                return str(e)
+            else:
+                return default
 
     def __init__(self, msg: str, parent: Union[Exception, None] = None, **kwargs):
-
+        self._msg = msg
         self._parent: Union[Exception, None] = parent
         self._properties = kwargs
         super().__init__(msg)
 
     @property
-    def details(self) -> Union[str, None]:
+    def msg(self):
+        return self._msg
 
-        return self._properties.get("details", None)
+    @property
+    def details(self) -> Union[str, None]:
+        result: Union[None, str] = self._properties.get("details", None)
+        return result
 
     @property
     def parent(self) -> Union[Exception, None]:
@@ -51,7 +58,6 @@ class KiaraException(Exception):
 
     @property
     def root_cause(self) -> Exception:
-
         current: Exception = self
         while hasattr(current, "parent") and current.parent is not None:  # type: ignore
             current = current.parent  # type: ignore
@@ -59,7 +65,6 @@ class KiaraException(Exception):
         return current
 
     def root_details(self) -> Union[str, None]:
-
         current: Exception = self
         if hasattr(self, "details"):
             current_details = self.details  # type: ignore
@@ -76,17 +81,44 @@ class KiaraException(Exception):
         return current_details
 
     def create_renderable(self, **config) -> "RenderableType":
-
         from rich.console import Group
 
-        rows: List[RenderableType] = [f"[red]Error[/red]: {str(self)}"]
+        rows: List[RenderableType] = [f"[red]Error[/red]: {self._msg}"]
         root_details = self.root_details()
         if root_details:
+            from rich.markdown import Markdown
+
             rows.append("")
             rows.append(Markdown(root_details))
 
         return Group(*rows)
 
+    def __str__(self) -> str:
+        msg = super().__str__()
+        root_details = self.root_details()
+        if root_details:
+            msg += f"\n\n{root_details}"
+        return msg
+
+
+class InvalidCommandLineInvocation(KiaraException):
+    def __init__(
+        self,
+        msg: str,
+        parent: Union[Exception, None] = None,
+        error_code: int = 0,
+        **kwargs,
+    ):
+        self.error_code: int = error_code
+
+        super().__init__(msg, parent=parent, **kwargs)
+
+
+class KiaraContextException(KiaraException):
+    def __init__(self, msg: str, context_id: uuid.UUID):
+        self._context_id: uuid.UUID = context_id
+        super().__init__(msg)
+
 
 class KiaraModuleConfigException(KiaraException):
     def __init__(
@@ -96,7 +128,6 @@ class KiaraModuleConfigException(KiaraException):
         config: Mapping[str, Any],
         parent: Union[Exception, None] = None,
     ):
-
         self._module_cls = module_cls
         self._config = config
 
@@ -117,7 +148,6 @@ class InvalidManifestException(KiaraException):
         available_module_types: Union[None, Iterable[str]] = None,
         parent: Union[Exception, None] = None,
     ):
-
         self._module_type = module_type
         self._module_config = module_config
         self._available_module_types = available_module_types
@@ -125,7 +155,6 @@ class InvalidManifestException(KiaraException):
 
     @property
     def details(self) -> Union[str, None]:
-
         if not self._available_module_types:
             return None
 
@@ -144,7 +173,6 @@ class ValueTypeConfigException(KiaraException):
         config: Mapping[str, Any],
         parent: Union[Exception, None] = None,
     ):
-
         self._type_cls = type_cls
         self._config = config
 
@@ -163,7 +191,6 @@ class DataTypeUnknownException(KiaraException):
         msg: Union[str, None] = None,
         value: Union[None, "Value"] = None,
     ):
-
         self._data_type = data_type
         if msg is None:
             msg = f"Data type '{data_type}' not registered in current context."
@@ -181,7 +208,6 @@ class DataTypeUnknownException(KiaraException):
         return self._value
 
     def create_renderable(self, **config: Any) -> "Table":
-
         from rich import box
         from rich.table import Table
 
@@ -216,7 +242,10 @@ class KiaraValueException(KiaraException):
         if not exc_msg:
             exc_msg = "no details available"
 
-        super().__init__(f"Invalid value of type '{data_type._data_type_name}': {exc_msg}", parent=parent)  # type: ignore
+        super().__init__(
+            f"Invalid value of type '{data_type._data_type_name}': {exc_msg}",  # type: ignore
+            parent=parent,
+        )  # type: ignore
 
 
 class NoSuchExecutionTargetException(KiaraException):
@@ -226,7 +255,6 @@ class NoSuchExecutionTargetException(KiaraException):
         available_targets: Iterable[str],
         msg: Union[str, None] = None,
     ):
-
         if msg is None:
             msg = f"Specified run target '{selected_target}' is an operation, additional module configuration is not allowed."
 
@@ -243,13 +271,22 @@ class KiaraProcessingException(KiaraException):
     ):
         self._module: Union["KiaraModule", None] = module
         self._inputs: Union[Mapping[str, Value], None] = inputs
-        if isinstance(msg, Exception):
-            self._parent: Union[Exception, None] = msg
+        _properties = None
+
+        if isinstance(msg, KiaraException):
+            _parent: Union[Exception, None] = msg.parent
+            _msg = msg.msg
+            _properties = msg._properties
+        elif isinstance(msg, Exception):
+            _parent = msg
             _msg = str(msg)
         else:
-            self._parent = None
+            _parent = None
             _msg = msg
-        super().__init__(_msg)
+        if _properties:
+            super().__init__(msg=_msg, parent=_parent, **_properties)
+        else:
+            super().__init__(_msg)
 
     @property
     def module(self) -> "KiaraModule":
@@ -270,7 +307,6 @@ class InvalidValuesException(KiaraException):
         msg: Union[None, str, Exception] = None,
         invalid_values: Union[Mapping[str, str], None] = None,
     ):
-
         if invalid_values is None:
             invalid_values = {}
 
@@ -302,7 +338,6 @@ class InvalidValuesException(KiaraException):
         return result
 
     def create_renderable(self, **config: Any) -> "Table":
-
         from rich import box
         from rich.console import RenderableType
         from rich.table import Table
@@ -313,7 +348,6 @@ class InvalidValuesException(KiaraException):
         table.add_column("[red]error[/red]")
 
         for field_name, error in self.invalid_inputs.items():
-
             row: List[RenderableType] = [field_name]
             row.append(error)
             table.add_row(*row)
@@ -328,7 +362,6 @@ class JobConfigException(KiaraException):
         manifest: "Manifest",
         inputs: Mapping[str, Any],
     ):
-
         self._manifest: Manifest = manifest
         self._inputs: Mapping[str, Any] = inputs
 
@@ -357,19 +390,20 @@ class FailedJobException(KiaraException):
         msg: Union[str, None] = None,
         parent: Union[Exception, None] = None,
     ):
-
         self.job: ActiveJob = job
-        if msg is None:
-            msg = "Job failed."
-        self.msg = msg
-        super().__init__(msg, parent=parent)
 
-    @property
-    def details(self) -> Union[str, None]:
-        return None
+        if isinstance(parent, KiaraException):
+            super().__init__(msg=parent.msg, parent=parent.parent, **parent._properties)
+        else:
+            if msg is None:
+                msg = "Job failed."
+            super().__init__(msg=msg, parent=parent)
+
+    # @property
+    # def details(self) -> Union[str, None]:
+    #     return None
 
     def create_renderable(self, **config: Any):
-
         from rich import box
         from rich.console import Group
         from rich.panel import Panel
@@ -389,7 +423,6 @@ class FailedJobException(KiaraException):
 
 
 class NoSuchValueException(KiaraException):
-
     pass
 
 
@@ -418,7 +451,6 @@ class NoSuchWorkflowException(KiaraException):
 
     @property
     def alias_requested(self) -> bool:
-
         if isinstance(self._workflow, str):
             try:
                 uuid.UUID(self._workflow)
@@ -436,7 +468,6 @@ class NoSuchOperationException(KiaraException):
         available_operations: Iterable[str],
         msg: Union[None, str] = None,
     ):
-
         self._operation_id: str = operation_id
         self._available_operations: Iterable[str] = available_operations
 
@@ -456,7 +487,6 @@ class NoSuchOperationException(KiaraException):
 
 class InvalidOperationException(KiaraException):
     def __init__(self, operation_details: Mapping[str, Any]):
-
         self._all_details: Mapping[str, Any] = operation_details
         msg = operation_details.get("operation_id", None)
         if msg is None:
@@ -485,7 +515,6 @@ class InvalidOperationException(KiaraException):
 
 class InvalidPipelineStepConfig(KiaraException):
     def __init__(self, msg: str, step_config: Mapping[str, Any]):
-
         self._step_config: Mapping[str, Any] = step_config
         super().__init__(msg)
 
@@ -499,7 +528,6 @@ class InvalidPipelineStepConfig(KiaraException):
 
 class InvalidPipelineConfig(KiaraException):
     def __init__(self, msg: str, config: "PipelineConfig", details: str):
-
         self._config = config
         self._details = details
         super().__init__(msg)
@@ -510,5 +538,4 @@ class InvalidPipelineConfig(KiaraException):
 
     @property
     def details(self) -> Union[str, None]:
-
         return self._details
