@@ -6,26 +6,28 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 """Data-related sub-commands for the cli."""
+
+import typing
 from typing import Any, Dict, Tuple, Union
 
 import rich_click as click
 import structlog
 
-from kiara import KiaraAPI
-from kiara.context import Kiara
-from kiara.interfaces.python_api.workflow import Workflow
 from kiara.utils.cli import dict_from_cli_args, terminal_print, terminal_print_model
-from kiara.utils.yaml import StringYAML
+
+if typing.TYPE_CHECKING:
+    from kiara.interfaces.python_api.base_api import BaseAPI, Kiara
 
 logger = structlog.getLogger()
 
-yaml = StringYAML()
-
 
 @click.group()
 @click.pass_context
 def workflow(ctx):
-    """Workflow-related sub-commands."""
+    """Workflow-related sub-commands.
+
+    All of those are experiments at the moment, so don't expect them to be available in the future.
+    """
 
 
 @workflow.command()
@@ -35,8 +37,7 @@ def workflow(ctx):
 @click.pass_context
 def list(ctx, all) -> None:
     """List existing workflows."""
-
-    kiara_api: KiaraAPI = ctx.obj["kiara_api"]
+    kiara_api: BaseAPI = ctx.obj.base_api
 
     if all:
         workflows = kiara_api.retrieve_workflows_info()
@@ -69,8 +70,7 @@ def create(
     force_alias: bool = False,
 ):
     """Create a new workflow."""
-
-    kiara_api: KiaraAPI = ctx.obj["kiara_api"]
+    kiara_api: BaseAPI = ctx.obj.base_api
 
     inputs_dict: Union[None, Dict[str, Any]] = None
     if inputs:
@@ -100,8 +100,7 @@ def create(
 @click.pass_context
 def explain(ctx, workflow: str):
     """Explain the workflow with the specified id/alias."""
-
-    kiara_api: KiaraAPI = ctx.obj["kiara_api"]
+    kiara_api: BaseAPI = ctx.obj.base_api
     workflow_info = kiara_api.retrieve_workflow_info(workflow=workflow)
     terminal_print(
         workflow_info.create_renderable(),
@@ -123,7 +122,9 @@ def explain(ctx, workflow: str):
 def set_input(ctx, workflow: str, inputs: Tuple[str], process: bool):
     """Set one or several inputs on the specified workflow."""
 
-    kiara: Kiara = ctx.obj["kiara"]
+    from kiara.interfaces.python_api.workflow import Workflow
+
+    kiara: Kiara = ctx.obj.kiara
 
     workflow_details = kiara.workflow_registry.get_workflow_metadata(workflow=workflow)
     workflow_obj = Workflow(kiara=kiara, workflow=workflow_details.workflow_id)
