@@ -3,11 +3,14 @@ import uuid
 from datetime import datetime
 from typing import TYPE_CHECKING, Any, Dict, Hashable, List, Mapping, Set, Tuple, Union
 
-import pytz
 import structlog
 from boltons.strutils import slugify
 
-from kiara.defaults import NONE_VALUE_ID, NOT_SET_VALUE_ID
+from kiara.defaults import (
+    KIARA_DEFAULT_STAGES_EXTRACTION_TYPE,
+    NONE_VALUE_ID,
+    NOT_SET_VALUE_ID,
+)
 from kiara.exceptions import NoSuchWorkflowException
 from kiara.models import KiaraModel
 from kiara.models.documentation import DocumentationMetadataModel
@@ -29,6 +32,7 @@ from kiara.models.values.value_schema import ValueSchema
 from kiara.models.workflow import WorkflowInfo, WorkflowMetadata, WorkflowState
 from kiara.registries.ids import ID_REGISTRY
 from kiara.utils import find_free_id, log_exception
+from kiara.utils.dates import get_current_time_incl_timezone
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -37,7 +41,8 @@ logger = structlog.getLogger()
 
 
 class WorkflowPipelineController(SinglePipelineController):
-    """A [PipelineController][kiara.models.modules.pipeline.controller.PipelineController] that executes all pipeline steps non-interactively.
+    """
+    A [PipelineController][kiara.models.modules.pipeline.controller.PipelineController] that executes all pipeline steps non-interactively.
 
     This is the default implementation of a ``PipelineController``, and probably the most simple implementation of one.
     It waits until all inputs are set, after which it executes all pipeline steps in the required order.
@@ -48,12 +53,10 @@ class WorkflowPipelineController(SinglePipelineController):
         self,
         kiara: "Kiara",
     ):
-
         self._is_running: bool = False
         super().__init__(job_registry=kiara.job_registry)
 
     def _pipeline_event_occurred(self, event: PipelineEvent):
-
         if event.pipeline_id != self.pipeline.pipeline_id:
             return
 
@@ -62,7 +65,6 @@ class WorkflowPipelineController(SinglePipelineController):
     def process_pipeline(
         self,
     ) -> Tuple[Mapping[uuid.UUID, uuid.UUID], Mapping[str, ActiveJob]]:
-
         log = logger.bind(pipeline_id=self.pipeline.pipeline_id)
         if self._is_running:
             log.debug(
@@ -78,10 +80,10 @@ class WorkflowPipelineController(SinglePipelineController):
         errors: Dict[str, ActiveJob] = {}
 
         try:
-            for idx, stage in enumerate(
-                self.pipeline.structure.processing_stages, start=1
-            ):
-
+            stages = self.pipeline.structure.extract_processing_stages(
+                stages_extraction_type=KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
+            )
+            for idx, stage in enumerate(stages, start=1):
                 log.debug(
                     "execute.pipeline.stage",
                     stage=idx,
@@ -90,7 +92,6 @@ class WorkflowPipelineController(SinglePipelineController):
                 job_ids = {}
                 stage_failed = False
                 for step_id in stage:
-
                     log.debug(
                         "execute.pipeline.step",
                         step_id=step_id,
@@ -141,7 +142,8 @@ class WorkflowStatus(KiaraModel):
 
 
 class Workflow(object):
-    """A wrapper object to make working with workflows easier for frontend code.
+    """
+    A wrapper object to make working with workflows easier for frontend code.
 
     This is the type of class everyone advises you against creating in Object-Oriented code, all it contains is basically
     internal state. In this case, I believe it's warranted because otherwise frontend code would spin out of control,
@@ -156,6 +158,7 @@ class Workflow(object):
      - if you want to create the wrapper object from an existing workflow: provide its id or alias string
 
     Arguments:
+    ---------
         kiara: the kiara context in which this workflow lives
         workflow: the workflow metadata (or reference to it)
         load_existing: if set to 'False', a workflow with the provided id/alias can't already exist, if 'True', it must, if set to 'None', kiara tries to be smart and loads if exists, otherwise creates a new one
@@ -169,12 +172,10 @@ class Workflow(object):
         create: bool = False,
     ):
         """Load an existing workflow using a workflow id or alias."""
-
         try:
             workflow_obj = Workflow(workflow=workflow, kiara=kiara, load_existing=True)
         except NoSuchWorkflowException as nswe:
             if create:
-
                 if isinstance(workflow, uuid.UUID):
                     raise nswe
                 temp = None
@@ -206,7 +207,6 @@ class Workflow(object):
         kiara: Union["Kiara", None] = None,
     ):
         """Create a new workflow object."""
-
         if replace_existing_alias and alias is not None:
             if kiara is None:
                 from kiara.context import Kiara
@@ -226,7 +226,6 @@ class Workflow(object):
         kiara: Union["Kiara", None] = None,
         load_existing: Union[bool, None] = None,
     ):
-
         if kiara is None:
             from kiara.context import Kiara
 
@@ -349,9 +348,9 @@ class Workflow(object):
         self._steps: Dict[str, PipelineStep] = {}
 
         self._current_workflow_inputs_schema: Union[Dict[str, ValueSchema], None] = None
-        self._current_workflow_outputs_schema: Union[
-            Dict[str, ValueSchema], None
-        ] = None
+        self._current_workflow_outputs_schema: Union[Dict[str, ValueSchema], None] = (
+            None
+        )
         self._workflow_input_aliases: Dict[str, str] = dict(
             _workflow_metadata.input_aliases
         )
@@ -381,7 +380,6 @@ class Workflow(object):
 
     def _sync_workflow_metadata(self):
         """Store/update the metadata of this workflow."""
-
         self._workflow_metadata = self._kiara.workflow_registry.register_workflow(
             workflow_metadata=self._workflow_metadata,
             workflow_aliases=self._pending_aliases,
@@ -406,14 +404,12 @@ class Workflow(object):
 
     @documentation.setter
     def documentation(self, documentation: Any):
-
         doc = DocumentationMetadataModel.create(documentation)
         self.workflow_metadata.documentation = doc
 
     @property
     def is_persisted(self) -> bool:
         """Check whether this workflow is persisted in it's current state."""
-
         return self.workflow_metadata.is_persisted
 
     @property
@@ -422,7 +418,6 @@ class Workflow(object):
 
     @property
     def current_pipeline_inputs(self) -> Mapping[str, uuid.UUID]:
-
         if self._current_pipeline_inputs is None:
             self._apply_inputs()
         assert self._current_pipeline_inputs is not None
@@ -440,7 +435,6 @@ class Workflow(object):
 
     @property
     def current_pipeline_outputs(self) -> Mapping[str, uuid.UUID]:
-
         if self._current_pipeline_outputs is None:
             try:
                 self.process_steps()
@@ -467,14 +461,13 @@ class Workflow(object):
         return self._workflow_output_aliases
 
     def clear_current_inputs_for_step(self, step_id):
-
         fields = self.get_current_inputs_schema_for_step(step_id)
-        for field in fields.keys():
-            self.set_inputs(**{k: None for k in fields.keys()})
+        vals = dict.fromkeys(fields, None)
+        self.set_inputs(**vals)
+        # self.set_inputs(**{k: None for k in fields.keys()})
 
     @property
     def current_inputs_schema(self) -> Mapping[str, ValueSchema]:
-
         if self._current_workflow_inputs_schema is not None:
             return self._current_workflow_inputs_schema
 
@@ -508,7 +501,6 @@ class Workflow(object):
 
     @property
     def current_inputs(self) -> Mapping[str, uuid.UUID]:
-
         if self._current_workflow_inputs is not None:
             return self._current_workflow_inputs
 
@@ -527,7 +519,6 @@ class Workflow(object):
 
     @property
     def current_outputs_schema(self) -> Mapping[str, ValueSchema]:
-
         if self._current_workflow_outputs_schema is not None:
             return self._current_workflow_outputs_schema
 
@@ -549,7 +540,6 @@ class Workflow(object):
 
     @property
     def current_outputs(self) -> Mapping[str, uuid.UUID]:
-
         if self._current_workflow_outputs is not None:
             return self._current_workflow_outputs
 
@@ -569,7 +559,6 @@ class Workflow(object):
 
     @property
     def current_state(self) -> WorkflowState:
-
         if self._current_state is not None:
             return self._current_state
 
@@ -579,7 +568,6 @@ class Workflow(object):
 
     @property
     def pipeline(self) -> Pipeline:
-
         if self._pipeline is not None:
             return self._pipeline
 
@@ -614,7 +602,6 @@ class Workflow(object):
         return self._pipeline
 
     def _apply_inputs(self) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
-
         pipeline = self.pipeline
 
         inputs_to_set = {}
@@ -630,9 +617,9 @@ class Workflow(object):
             keys=", ".join(inputs_to_set.keys()),
         )
 
-        changed: Mapping[
-            str, Mapping[str, Mapping[str, ChangedValue]]
-        ] = pipeline.set_pipeline_inputs(inputs=inputs_to_set)
+        changed: Mapping[str, Mapping[str, Mapping[str, ChangedValue]]] = (
+            pipeline.set_pipeline_inputs(inputs=inputs_to_set)
+        )
 
         self._current_pipeline_inputs = pipeline.get_current_pipeline_inputs()
 
@@ -650,11 +637,12 @@ class Workflow(object):
                     break
                 elif step_details.status == StepStatus.INPUTS_READY:
                     job_config = JobConfig(
-                        module_type=step_details.step.module_type,
-                        module_config=step_details.step.module.config.dict(),
+                        module_type=step_details.step.module.manifest.module_type,
+                        module_config=step_details.step.module.manifest.module_config,
+                        is_resolved=step_details.step.module.manifest.is_resolved,
                         inputs=step_details.inputs,
                     )
-                    match = self._kiara.job_registry.find_matching_job_record(
+                    match = self._kiara.job_registry.find_job_record_for_manifest(
                         inputs_manifest=job_config
                     )
                     if match:
@@ -672,7 +660,6 @@ class Workflow(object):
     def process_steps(
         self, *step_ids: str
     ) -> Tuple[Mapping[uuid.UUID, uuid.UUID], Mapping[str, ActiveJob]]:
-
         self.pipeline
 
         if not step_ids:
@@ -705,7 +692,6 @@ class Workflow(object):
         return output_job_map, errors
 
     def _invalidate_pipeline(self):
-
         self._pipeline_controller.pipeline = None
         self._pipeline = None
         self._pipeline_info = None
@@ -717,30 +703,34 @@ class Workflow(object):
         self._current_pipeline_outputs = None
 
     def set_input(self, field_name: str, value: Any) -> Union[None, uuid.UUID]:
-        """Set a single pipeline input.
+        """
+        Set a single pipeline input.
 
         Arguments:
+        ---------
             field_name: The name of the input field.
             value: The value to set.
 
         Returns:
+        -------
             None if the value for that field in the pipeline didn't change, otherwise the value_id of the new (registered) value.
 
         """
-
         diff = self.set_inputs(**{field_name: value})
         return diff.get(field_name, None)
 
     def set_inputs(self, **inputs: Any) -> Dict[str, Union[uuid.UUID, None]]:
-        """Set multiple pipeline inputs, at once.
+        """
+        Set multiple pipeline inputs, at once.
 
         Arguments:
+        ---------
             inputs: The inputs to set.
 
         Returns:
+        -------
             a dict containing only the newly set or changed values with field name as keys, and value_id (or None) as values.
         """
-
         _inputs = {}
         for k, v in inputs.items():
             # translate aliases
@@ -766,7 +756,6 @@ class Workflow(object):
 
         diff: Dict[str, Union[None, uuid.UUID]] = {}
         for k, val_new in inputs.items():
-
             val_old = self._all_inputs.get(k, None)
             if val_old is None and val_new is None:
                 continue
@@ -855,7 +844,6 @@ class Workflow(object):
         replace_existing: bool = False,
         clear_existing: bool = False,
     ):
-
         if clear_existing:
             self.clear_steps()
 
@@ -896,7 +884,6 @@ class Workflow(object):
         self._invalidate_pipeline()
 
     def clear_steps(self, *step_ids: str):
-
         if not step_ids:
             self._steps.clear()
         else:
@@ -906,7 +893,6 @@ class Workflow(object):
         self._invalidate_pipeline()
 
     def set_input_alias(self, input_field: str, alias: str):
-
         if "." in input_field:
             tokens = input_field.split(".")
             if len(tokens) != 2:
@@ -922,7 +908,6 @@ class Workflow(object):
         self._metadata_is_synced = False
 
     def set_output_alias(self, output_field: str, alias: str):
-
         if "." in output_field:
             tokens = output_field.split(".")
             if len(tokens) != 2:
@@ -960,18 +945,19 @@ class Workflow(object):
         doc: Union[str, DocumentationMetadataModel, None] = None,
         replace_existing: bool = False,
     ) -> PipelineStep:
-        """Add a step to the workflows current pipeline structure.
+        """
+        Add a step to the workflows current pipeline structure.
 
         If no 'step_id' is provided, a unque one will automatically be generated based on the 'module_type' argument.
 
         Arguments:
+        ---------
             operation: the module or operation name
             step_id: the id of the new step
             module_config: (optional) configuration for the kiara module this step uses
             input_connections: a map with this steps input field name(s) as keys and output field links (format: <step_id>.<output_field_name>) as value(s).
             replace_existing: if set to 'True', this replaces a step with the same id that already exists, otherwise an exception will be thrown
         """
-
         if step_id is None:
             step_id = find_free_id(
                 slugify(operation, delim="_"), current_ids=self._steps.keys()
@@ -997,7 +983,7 @@ class Workflow(object):
         step = PipelineStep(
             step_id=step_id,
             module_type=module.module_type_name,
-            module_config=module.config.dict(),
+            module_config=module.config.model_dump(),
             module_details=KiaraModuleInstance.from_module(module=module),
             doc=doc,
             manifest_src=manifest_src,
@@ -1014,7 +1000,6 @@ class Workflow(object):
         return step
 
     def connect_fields(self, *fields: Union[Tuple[str, str], str]):
-
         pairs = []
         current_pair = None
         for field in fields:
@@ -1063,7 +1048,6 @@ class Workflow(object):
         target_step: Union[PipelineStep, str],
         target_field: str,
     ):
-
         if isinstance(source_step, str):
             source_step_obj = self.get_step(source_step)
         else:
@@ -1117,7 +1101,6 @@ class Workflow(object):
         self._invalidate_pipeline()
 
     def connect_to_inputs(self, source_field: str, *input_fields: str):
-
         source_tokens = source_field.split(".")
         if len(source_tokens) != 2:
             raise Exception(
@@ -1169,7 +1152,6 @@ class Workflow(object):
         self._invalidate_pipeline()
 
     def get_step(self, step_id: str) -> PipelineStep:
-
         step = self._steps.get(step_id, None)
         if step is None:
             if self._steps:
@@ -1182,14 +1164,15 @@ class Workflow(object):
     def load_state(
         self, workflow_state_id: Union[str, None] = None
     ) -> Union[None, WorkflowState]:
-        """Load a past state.
+        """
+        Load a past state.
 
         If no state id is specified, the latest one that was saved will be used.
 
         Returns:
+        -------
             'None' if no state was loaded, otherwise the relevant 'WorkflowState' instance
         """
-
         if workflow_state_id is None:
             if not self._workflow_metadata.workflow_history:
                 return None
@@ -1222,7 +1205,15 @@ class Workflow(object):
         # self._workflow_output_aliases = dict(state.output_aliases)
 
         self.set_inputs(**state.inputs)
-        assert {k: v for k, v in self._current_pipeline_inputs.items() if v not in [NONE_VALUE_ID, NOT_SET_VALUE_ID]} == {k: v for k, v in state.inputs.items() if v not in [NONE_VALUE_ID, NOT_SET_VALUE_ID]}  # type: ignore
+        assert {
+            k: v
+            for k, v in self._current_pipeline_inputs.items()  # type: ignore
+            if v not in [NONE_VALUE_ID, NOT_SET_VALUE_ID]
+        } == {
+            k: v
+            for k, v in state.inputs.items()
+            if v not in [NONE_VALUE_ID, NOT_SET_VALUE_ID]
+        }  # type: ignore
         self._current_pipeline_outputs = (
             state.pipeline_info.pipeline_state.pipeline_outputs
         )
@@ -1234,14 +1225,12 @@ class Workflow(object):
 
     @property
     def all_state_ids(self) -> List[str]:
-
         hashes = set(self._workflow_metadata.workflow_history.values())
         return sorted(hashes)
 
     @property
     def all_states(self) -> Mapping[str, WorkflowState]:
         """Return a list of all states this workflow had in the past, indexed by the hash of each state."""
-
         missing = []
         for state_id in self.workflow_metadata.workflow_history.values():
             if state_id not in self._state_cache.keys():
@@ -1258,7 +1247,6 @@ class Workflow(object):
 
     @property
     def info(self) -> WorkflowInfo:
-
         if self._current_info is not None:
             return self._current_info
 
@@ -1267,7 +1255,6 @@ class Workflow(object):
 
     @property
     def pipeline_info(self) -> PipelineInfo:
-
         if self._pipeline_info is not None:
             return self._pipeline_info
 
@@ -1277,7 +1264,6 @@ class Workflow(object):
         return self._pipeline_info
 
     def save(self, *aliases: str):
-
         self._pending_aliases.update(aliases)
 
         self._workflow_metadata = self._kiara.workflow_registry.register_workflow(
@@ -1289,13 +1275,12 @@ class Workflow(object):
         self._metadata_is_synced = True
 
     def snapshot(self, save: bool = False) -> WorkflowState:
-
         state = self.current_state
 
         if state.instance_id not in self._state_cache.keys():
             self._state_cache[state.instance_id] = state
 
-        now = datetime.now(pytz.utc)
+        now = get_current_time_incl_timezone()
 
         for field_name, value in self.current_pipeline_outputs.items():
             if value in [NOT_SET_VALUE_ID, NONE_VALUE_ID]:
@@ -1314,7 +1299,6 @@ class Workflow(object):
         return state
 
     def register_snapshot(self, snapshot: Union[datetime, str]):
-
         timestamps: List[datetime]
         if isinstance(snapshot, str):
             if snapshot not in self._state_cache.keys():
@@ -1372,7 +1356,6 @@ class Workflow(object):
         return state
 
     def create_renderable(self, **config: Any):
-
         if not self._steps:
             return "Invalid workflow: no steps set yet."
 
