@@ -9,21 +9,23 @@ from abc import ABC
 from typing import Any, ClassVar, Dict, Iterable, List, Mapping, Union
 
 import networkx as nx
-import orjson
-from dag_cbor import Kind
-from deepdiff import DeepHash
+from dag_cbor import IPLDKind
 from multiformats import CID
-from pydantic import Extra
+from pydantic import ConfigDict
 from pydantic.fields import PrivateAttr
 from pydantic.main import BaseModel
 from rich import box
-from rich.console import Console, ConsoleOptions, RenderableType, RenderResult
+from rich.console import Console, ConsoleOptions, Group, RenderableType, RenderResult
 from rich.jupyter import JupyterMixin
 from rich.panel import Panel
 from rich.table import Table
 from rich.tree import Tree
 
-from kiara.defaults import KIARA_HASH_FUNCTION
+from kiara.defaults import (
+    KIARA_MODEL_DATA_KEY,
+    KIARA_MODEL_ID_KEY,
+    KIARA_MODEL_SCHEMA_KEY,
+)
 from kiara.registries.templates import TemplateRegistry
 from kiara.utils.class_loading import _default_id_func
 from kiara.utils.develop import log_dev_message
@@ -38,7 +40,8 @@ from kiara.utils.models import (
 
 
 class KiaraModel(ABC, BaseModel, JupyterMixin):
-    """Base class that all models in kiara inherit from.
+    """
+    Base class that all models in kiara inherit from.
 
     This class provides utility functions for things like rendering the model on terminal or as html, integration into
     a tree hierarchy of the overall kiara context, hashing, etc.
@@ -46,10 +49,7 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
 
     __slots__ = ["__weakref__"]
 
-    class Config(object):
-        json_loads = orjson.loads
-        json_dumps = orjson_dumps
-        extra = Extra.forbid
+    model_config = ConfigDict(extra="forbid")
 
     # @classmethod
     # def get_model_title(cls):
@@ -57,13 +57,23 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     #     return to_camel_case(cls._kiara_model_name)
 
     @classmethod
-    def get_schema_hash(cls) -> int:
+    def get_schema_cid(cls) -> CID:
         if cls._schema_hash_cache is not None:
             return cls._schema_hash_cache
 
-        obj = cls.schema_json()
-        h = DeepHash(obj, hasher=KIARA_HASH_FUNCTION)
-        cls._schema_hash_cache = h[obj]
+        model_schema = cls.model_json_schema()
+        try:
+            _, cid = compute_cid(data=model_schema)
+        except Exception as e:
+            from kiara.utils.output import extract_renderable
+
+            msg = "Failed to compute cid for model schema instance."
+            item = extract_renderable(model_schema)
+            renderable = Group(msg, item, extract_renderable(e))
+            log_dev_message(renderable, title="cid computation error")
+            raise e
+
+        cls._schema_hash_cache = cid
         return cls._schema_hash_cache
 
     _graph_cache: Union[nx.DiGraph, None] = PrivateAttr(default=None)
@@ -71,24 +81,23 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     _dynamic_subcomponents: Dict[str, "KiaraModel"] = PrivateAttr(default_factory=dict)
     _id_cache: Union[str, None] = PrivateAttr(default=None)
     _category_id_cache: Union[str, None] = PrivateAttr(default=None)
-    _schema_hash_cache: ClassVar = None
+    _schema_hash_cache: ClassVar[Union[None, CID]] = None
     _cid_cache: Union[CID, None] = PrivateAttr(default=None)
     _dag_cache: Union[bytes, None] = PrivateAttr(default=None)
     _size_cache: Union[int, None] = PrivateAttr(default=None)
 
-    def _retrieve_data_to_hash(self) -> Kind:
-        """Return data important for hashing this model instance. Implemented by sub-classes.
+    def _retrieve_data_to_hash(self) -> IPLDKind:
+        """
+        Return data important for hashing this model instance. Implemented by sub-classes.
 
         This returns the relevant data that makes this model unique, excluding any secondary metadata that is not
         necessary for this model to be used functionally. Like for example documentation.
         """
-
-        return self.dict()
+        return self.model_dump()
 
     @property
     def instance_id(self) -> str:
         """The unique id of this model, within its category."""
-
         if self._id_cache is not None:
             return self._id_cache
 
@@ -103,14 +112,12 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
 
     @property
     def instance_dag(self) -> bytes:
-
         if self._dag_cache is None:
             self._compute_cid()
         return self._dag_cache  # type: ignore
 
     @property
     def instance_size(self) -> int:
-
         if self._size_cache is None:
             self._compute_cid()
         return self._size_cache  # type: ignore
@@ -118,7 +125,6 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     @property
     def model_type_id(self) -> str:
         """The id of the category of this model."""
-
         if hasattr(self.__class__, "_kiara_model_id"):
             return self._kiara_model_id  # type: ignore
         else:
@@ -133,7 +139,16 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
             return
 
         obj = self._retrieve_data_to_hash()
-        dag, cid = compute_cid(data=obj)
+        try:
+            dag, cid = compute_cid(data=obj)
+        except Exception as e:
+            from kiara.utils.output import extract_renderable
+
+            msg = "Failed to compute cid for model instance."
+            item = extract_renderable(obj)
+            renderable = Group(msg, item, extract_renderable(e))
+            log_dev_message(renderable, title="cid computation error")
+            raise e
 
         self._cid_cache = cid
         self._dag_cache = dag
@@ -144,7 +159,6 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     @property
     def subcomponent_keys(self) -> Iterable[str]:
         """The keys of available sub-components of this model."""
-
         if self._subcomponent_names_cache is None:
             self._subcomponent_names_cache = sorted(self._retrieve_subcomponent_keys())
         return self._subcomponent_names_cache
@@ -161,7 +175,6 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
 
     def get_subcomponent(self, path: str) -> "KiaraModel":
         """Retrieve the subcomponent identified by the specified path."""
-
         if path not in self._dynamic_subcomponents.keys():
             self._dynamic_subcomponents[path] = self._retrieve_subcomponent(path=path)
         return self._dynamic_subcomponents[path]
@@ -185,31 +198,29 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         return result
 
     def _retrieve_subcomponent_keys(self) -> Iterable[str]:
-        """Retrieve the keys of all subcomponents of this model.
+        """
+        Retrieve the keys of all subcomponents of this model.
 
         Can be overwritten in sub-classes, by default it tries to automatically determine the subcomponents.
         """
-
         return retrieve_data_subcomponent_keys(self)
 
     def _retrieve_subcomponent(self, path: str) -> "KiaraModel":
-        """Retrieve the subcomponent under the specified path.
+        """
+        Retrieve the subcomponent under the specified path.
 
         Can be overwritten in sub-classes, by default it tries to automatically determine the subcomponents.
         """
-
         m = get_subcomponent_from_model(self, path=path)
         return m
 
     # ==========================================================================================
     # model rendering related methods
     def create_panel(self, title: Union[str, None] = None, **config: Any) -> Panel:
-
         rend = self.create_renderable(**config)
         return Panel(rend, box=box.ROUNDED, title=title, title_align="left")
 
     def create_html(self, **config) -> str:
-
         template_registry = TemplateRegistry.instance()
         template = template_registry.get_template_for_model_type(
             model_type=self.model_type_id, template_format="html"
@@ -217,7 +228,7 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
 
         if template:
             try:
-                result = template.render(instance=self)
+                result: str = template.render(instance=self)
                 return result
             except Exception as e:
                 log_dev_message(
@@ -228,7 +239,7 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         try:
             from kiara.utils.html import generate_html
 
-            html = generate_html(item=self, add_header=False)
+            html: str = generate_html(item=self, add_header=False)  # type: ignore
             return html
         except Exception as e:
             log_dev_message(
@@ -237,11 +248,10 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
             )
 
         r = self.create_renderable(**config)
-        mime_bundle = r._repr_mimebundle_(include=[], exclude=[])  # type: ignore
+        mime_bundle: Mapping[str, str] = r._repr_mimebundle_(include=[], exclude=[])  # type: ignore
         return mime_bundle["text/html"]
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         from kiara.utils.output import extract_renderable
 
         include = config.get("include", None)
@@ -249,7 +259,7 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("Key", style="i")
         table.add_column("Value")
-        for k in self.__fields__.keys():
+        for k in self.__class__.model_fields.keys():
             if include is not None and k not in include:
                 continue
             attr = getattr(self, k)
@@ -258,17 +268,15 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         return table
 
     def create_renderable_tree(self, **config: Any) -> Tree:
-
         show_data = config.get("show_data", False)
         tree = create_subcomponent_tree_renderable(data=self, show_data=show_data)
         return tree
 
     def create_info_data(self, **config) -> Mapping[str, Any]:
-
         include = config.get("include", None)
 
         result = {}
-        for k in self.__fields__.keys():
+        for k in self.model_fields.keys():
             if include is not None and k not in include:
                 continue
             attr = getattr(self, k)
@@ -277,19 +285,45 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         return result
 
     def as_dict_with_schema(self) -> Dict[str, Dict[str, Any]]:
-        return {"data": self.dict(), "schema": self.schema()}
+        return {"data": self.model_dump(), "schema": self.model_json_schema()}
 
-    def as_json_with_schema(self) -> str:
-
-        data_json = self.json()
-        schema_json = self.schema_json()
-        return '{"data": ' + data_json + ', "schema": ' + schema_json + "}"
+    def as_json_with_schema(self, incl_model_id: bool = False) -> str:
+        data_json = self.model_dump_json()
+        schema_json = self.model_json_schema()
+        schema_json_str = orjson_dumps(schema_json)
+        if not incl_model_id:
+            return (
+                '{"'
+                + KIARA_MODEL_DATA_KEY
+                + '": '
+                + data_json
+                + ', "'
+                + KIARA_MODEL_SCHEMA_KEY
+                + '": '
+                + schema_json_str
+                + "}"
+            )
+        else:
+            return (
+                '{"'
+                + KIARA_MODEL_DATA_KEY
+                + '": '
+                + data_json
+                + ', "'
+                + KIARA_MODEL_SCHEMA_KEY
+                + '": '
+                + schema_json_str
+                + ', "'
+                + KIARA_MODEL_ID_KEY
+                + '": "'
+                + self.model_type_id
+                + '"}'
+            )
 
     def __hash__(self):
         return int.from_bytes(self.instance_cid.digest, "big")
 
     def __eq__(self, other):
-
         if self.__class__ != other.__class__:
             return False
         else:
@@ -299,13 +333,12 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
             )
 
     def __repr__(self):
-
         try:
             model_id = self.instance_id
         except Exception:
             model_id = "-- n/a --"
 
-        return f"{self.__class__.__name__}(model_id={model_id}, category={self.model_type_id}, fields=[{', '.join(self.__fields__.keys())}])"
+        return f"{self.__class__.__name__}(model_id={model_id}, category={self.model_type_id}, fields=[{', '.join(self.model_fields.keys())}])"
 
     def __str__(self):
         return self.__repr__()
@@ -316,23 +349,4 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
-
         yield self.create_renderable()
-
-    # def _repr_mimebundle_(
-    #     self: "ConsoleRenderable",
-    #     include: Sequence[str],
-    #     exclude: Sequence[str],
-    #     **kwargs: Any,
-    # ) -> Dict[str, str]:
-    #
-    #     console = get_console()
-    #     segments = list(console.render(self, console.options))
-    #     html = _render_segments(segments)
-    #     text = console._render_buffer(segments)
-    #     data = {"text/plain": text, "text/html": html}
-    #     if include:
-    #         data = {k: v for (k, v) in data.items() if k in include}
-    #     if exclude:
-    #         data = {k: v for (k, v) in data.items() if k not in exclude}
-    #     return data
