@@ -6,7 +6,7 @@
 import copy
 import datetime
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Union
 
 import structlog
 from pydantic import Field, PrivateAttr
@@ -29,7 +29,7 @@ VALUE_ALIAS_SEPARATOR = "."
 class AliasValueMap(ValueMap):
     """A model class that holds a tree of values and their schemas."""
 
-    _kiara_model_id = "instance.value_map.aliases"
+    _kiara_model_id: ClassVar = "instance.value_map.aliases"
 
     alias: Union[str, None] = Field(description="This maps own (full) alias.")
     version: int = Field(description="The version of this map (in this maps parent).")
@@ -47,7 +47,7 @@ class AliasValueMap(ValueMap):
         description="The values contained in this set.", default_factory=dict
     )
 
-    _data_registry: "DataRegistry" = PrivateAttr(default=None)
+    _data_registry: Union["DataRegistry", None] = PrivateAttr(default=None)
     _schema_locked: bool = PrivateAttr(default=False)
     _auto_schema: bool = PrivateAttr(default=True)
     _is_stored: bool = PrivateAttr(default=False)
@@ -62,16 +62,15 @@ class AliasValueMap(ValueMap):
     def get_child_map(
         self, field_name: str, version: Union[str, None] = None
     ) -> Union["AliasValueMap", None]:
-        """Get the child map for the specified field / version combination.
+        """
+        Get the child map for the specified field / version combination.
 
         Raises an error if the child field does not exist. Returns 'None' if not value is set yet (but schema is).
         """
-
         if version is not None:
             raise NotImplementedError()
 
         if VALUE_ALIAS_SEPARATOR not in field_name:
-
             if self.values_schema.get(field_name, None) is None:
                 if not self.values_schema:
                     msg = "No available fields"
@@ -99,7 +98,7 @@ class AliasValueMap(ValueMap):
             return child_map.get_child_map(rest)
 
     def get_value_obj(self, field_name: str) -> Value:
-
+        assert self._data_registry is not None
         item = self.get_child_map(field_name=field_name)
         if item is None:
             return self._data_registry.NONE_VALUE
@@ -109,7 +108,6 @@ class AliasValueMap(ValueMap):
         return self._data_registry.get_value(value=item.assoc_value)
 
     def get_value_id(self, field_name: str) -> uuid.UUID:
-
         item = self.get_child_map(field_name=field_name)
         if item is None:
             result = NONE_VALUE_ID
@@ -121,7 +119,6 @@ class AliasValueMap(ValueMap):
     def get_all_value_ids(
         self,
     ) -> Dict[str, uuid.UUID]:
-
         result: Dict[str, uuid.UUID] = {}
         for k in self.values_schema.keys():
             v_id = self.get_value_id(field_name=k)
@@ -131,12 +128,10 @@ class AliasValueMap(ValueMap):
         return result
 
     def set_alias_schema(self, alias: str, schema: ValueSchema):
-
         if self._schema_locked:
             raise Exception(f"Can't add schema for alias '{alias}': schema locked.")
 
         if VALUE_ALIAS_SEPARATOR not in alias:
-
             self._set_local_field_schema(field_name=alias, schema=schema)
         else:
             child, rest = alias.split(VALUE_ALIAS_SEPARATOR, maxsplit=1)
@@ -154,7 +149,6 @@ class AliasValueMap(ValueMap):
             child_map.set_alias_schema(alias=rest, schema=schema)
 
     def _set_local_field_schema(self, field_name: str, schema: ValueSchema):
-
         assert field_name is not None
         if VALUE_ALIAS_SEPARATOR in field_name:
             raise Exception(
@@ -179,7 +173,6 @@ class AliasValueMap(ValueMap):
         self.value_items[field_name] = {}
 
     def get_alias(self, alias: str) -> Union["AliasValueMap", None]:
-
         if VALUE_ALIAS_SEPARATOR not in alias:
             if "@" in alias:
                 raise NotImplementedError()
@@ -203,13 +196,11 @@ class AliasValueMap(ValueMap):
             return child_map.get_alias(rest)
 
     def set_value(self, field_name: str, data: Any) -> None:
-
         assert VALUE_ALIAS_SEPARATOR not in field_name
 
         self._set_alias(alias=field_name, data=data)
 
     def _set_aliases(self, **aliases: Any) -> Mapping[str, "AliasValueMap"]:
-
         result = {}
         for k, v in aliases.items():
             r = self._set_alias(alias=k, data=v)
@@ -218,7 +209,6 @@ class AliasValueMap(ValueMap):
         return result
 
     def _set_alias(self, alias: str, data: Any) -> "AliasValueMap":
-
         if VALUE_ALIAS_SEPARATOR not in alias:
             field_name: Union[str, None] = alias
 
@@ -230,7 +220,6 @@ class AliasValueMap(ValueMap):
                 assert data is None
                 value_id = None
             else:
-
                 if data in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
                     if vs.default:
                         if callable(vs.default):
@@ -238,6 +227,7 @@ class AliasValueMap(ValueMap):
                         else:
                             data = copy.deepcopy(vs.default)
 
+                assert self._data_registry is not None
                 value = self._data_registry.register_data(data=data, schema=vs)
                 value_id = value.value_id
 
@@ -301,8 +291,8 @@ class AliasValueMap(ValueMap):
     def _set_local_value_item(
         self, field_name: str, value_id: Union[uuid.UUID, None] = None
     ) -> "AliasValueMap":
-
         assert VALUE_ALIAS_SEPARATOR not in field_name
+        assert self._data_registry is not None
 
         value: Union[Value, None] = None
         if value_id is not None:
@@ -359,12 +349,10 @@ class AliasValueMap(ValueMap):
         return new_map
 
     def print_tree(self):
-
         t = self.get_tree("base")
         terminal_print(t)
 
     def get_tree(self, base_name: str) -> Tree:
-
         if self.assoc_schema:
             type_name = self.assoc_schema.type
         else:
@@ -378,11 +366,11 @@ class AliasValueMap(ValueMap):
         tree = Tree(f"{base_name}{type_str}")
         if self.assoc_value:
             data = tree.add("__data__")
+            assert self._data_registry is not None
             value = self._data_registry.get_value(self.assoc_value)
             data.add(str(value.data))
 
         for field_name, schema in self.values_schema.items():
-
             alias = self.get_alias(alias=field_name)
             if alias is not None:
                 tree.add(alias.get_tree(base_name=field_name))
@@ -397,7 +385,6 @@ class AliasValueMap(ValueMap):
         return tree
 
     def __repr__(self):
-
         return f"AliasMap(assoc_value={self.assoc_value}, field_names={self.value_items.keys()})"
 
     def __str__(self):
