@@ -6,20 +6,13 @@
 
 import uuid
 
-import orjson
 from pydantic import BaseModel, Field
 
 from kiara.utils import camel_case_to_snake_case
-from kiara.utils.json import orjson_dumps
 
 
 class KiaraEvent(BaseModel):
-    class Config:
-        json_loads = orjson.loads
-        json_dumps = orjson_dumps
-
     def get_event_type(self) -> str:
-
         if hasattr(self, "event_type"):
             return self.event_type  # type: ignore
 
@@ -28,7 +21,6 @@ class KiaraEvent(BaseModel):
 
 
 class RegistryEvent(KiaraEvent):
-
     kiara_id: uuid.UUID = Field(
         description="The id of the kiara context the value was created in."
     )
