@@ -6,7 +6,7 @@
 
 import copy
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Union
 
 from pydantic import Field, PrivateAttr
 
@@ -18,17 +18,19 @@ from kiara.registries.ids import ID_REGISTRY
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
+    from kiara.models.values.value import Value
     from kiara.modules import KiaraModule
 
 
 class Destiny(Manifest):
-    """A destiny is basically a link to a potential future transformation result involving one or several values as input.
+    """
+    A destiny is basically a link to a potential future transformation result involving one or several values as input.
 
     It is immutable, once executed, each of the input values can only have one destiny with a specific alias.
     This is similar to what is usually called a 'future' in programming languages, but more deterministic, sorta.
     """
 
-    _kiara_model_id = "instance.destiny"
+    _kiara_model_id: ClassVar = "instance.destiny"
 
     @classmethod
     def create_from_values(
@@ -38,8 +40,7 @@ class Destiny(Manifest):
         values: Mapping[str, uuid.UUID],
         manifest: Manifest,
         result_field_name: Union[str, None] = None,
-    ):
-
+    ) -> "Destiny":
         module = kiara.module_registry.create_module(manifest=manifest)
 
         if result_field_name is None:
@@ -127,7 +128,6 @@ class Destiny(Manifest):
 
     @property
     def merged_inputs(self) -> Mapping[str, uuid.UUID]:
-
         if self._merged_inputs is not None:
             return self._merged_inputs
 
@@ -162,6 +162,18 @@ class Destiny(Manifest):
             self._module = m_cls(module_config=self.module_config)
         return self._module
 
+    def execute(self, kiara: "Kiara") -> "Value":
+        if self.result_value_id is not None:
+            raise Exception("Destiny already resolved.")
+
+        results = kiara.job_registry.execute_and_retrieve(
+            manifest=self, inputs=self.merged_inputs
+        )
+        value = results.get_value_obj(field_name=self.result_field_name)
+
+        self.result_value_id = value.value_id
+        return value
+
     # def _retrieve_job_config_hash(self) -> int:
     #     obj = {"module_config": self.manifest_data, "inputs": self.merged_inputs}
     #     return compute_cid(obj)
