@@ -10,18 +10,19 @@ import os
 import uuid
 from datetime import datetime
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, List, Mapping, Union
 
-from pydantic import validator
+from pydantic import field_validator
 from pydantic.fields import Field, PrivateAttr
 from pydantic.main import BaseModel
 from rich import box
 from rich.console import RenderableType
 from rich.table import Table
 
-from kiara.exceptions import InvalidValuesException
+from kiara.exceptions import InvalidValuesException, KiaraException
 from kiara.models import KiaraModel
 from kiara.models.module.manifest import InputsManifest
+from kiara.utils.dates import get_current_time_incl_timezone
 
 if TYPE_CHECKING:
     from kiara.context import DataRegistry, Kiara
@@ -29,8 +30,7 @@ if TYPE_CHECKING:
 
 
 class ExecutionContext(KiaraModel):
-
-    _kiara_model_id = "instance.execution_context"
+    _kiara_model_id: ClassVar = "instance.execution_context"
 
     working_dir: str = Field(
         description="The path of the working directory.", default_factory=os.getcwd
@@ -42,7 +42,6 @@ class ExecutionContext(KiaraModel):
 
 
 class JobStatus(Enum):
-
     CREATED = "__job_created__"
     STARTED = "__job_started__"
     SUCCESS = "__job_success__"
@@ -50,7 +49,6 @@ class JobStatus(Enum):
 
 
 class LogMessage(BaseModel):
-
     timestamp: datetime = Field(
         description="The time the message was logged.", default_factory=datetime.now
     )
@@ -59,7 +57,6 @@ class LogMessage(BaseModel):
 
 
 class JobLog(BaseModel):
-
     log: List[LogMessage] = Field(
         description="The logs for this job.", default_factory=list
     )
@@ -69,14 +66,17 @@ class JobLog(BaseModel):
     )
 
     def add_log(self, msg: str, log_level: int = logging.DEBUG):
-
         _msg = LogMessage(msg=msg, log_level=log_level)
         self.log.append(_msg)
 
 
-class JobConfig(InputsManifest):
+class PipelineMetadata(BaseModel):
+    pipeline_id: uuid.UUID = Field(description="The id of the pipeline.")
+    step_id: str = Field(description="The id of the step in the pipeline.")
 
-    _kiara_model_id = "instance.job_config"
+
+class JobConfig(InputsManifest):
+    _kiara_model_id: ClassVar = "instance.job_config"
 
     @classmethod
     def create_from_module(
@@ -84,31 +84,44 @@ class JobConfig(InputsManifest):
         data_registry: "DataRegistry",
         module: "KiaraModule",
         inputs: Mapping[str, Any],
-    ):
-
+    ) -> "JobConfig":
         augmented = module.augment_module_inputs(inputs=inputs)
 
         values = data_registry.create_valuemap(
             data=augmented, schema=module.full_inputs_schema
         )
+
         invalid = values.check_invalid()
         if invalid:
             raise InvalidValuesException(invalid_values=invalid)
 
         value_ids = values.get_all_value_ids()
-        return JobConfig.construct(
-            module_type=module.module_type_name,
-            module_config=module.config.dict(),
+
+        if not module.manifest.is_resolved:
+            raise KiaraException(
+                msg="Cannot create job config from unresolved manifest."
+            )
+
+        return JobConfig(
+            module_type=module.manifest.module_type,
+            module_config=module.manifest.module_config,
+            is_resolved=module.manifest.is_resolved,
             inputs=value_ids,
         )
 
     def _retrieve_data_to_hash(self) -> Any:
         return {"manifest": self.manifest_cid, "inputs": self.inputs_cid}
 
+    pipeline_metadata: Union[PipelineMetadata, None] = Field(
+        description="Metadata for the pipeline this job is part of.", default=None
+    )
+    # job_metadata: Mapping[str, Any] = Field(
+    #     description="Optional metadata for this job.", default_factory=dict
+    # )
+
 
 class ActiveJob(KiaraModel):
-
-    _kiara_model_id = "instance.active_job"
+    _kiara_model_id: ClassVar = "instance.active_job"
 
     job_id: uuid.UUID = Field(description="The job id.")
 
@@ -118,7 +131,8 @@ class ActiveJob(KiaraModel):
     )
     job_log: JobLog = Field(description="The lob jog.")
     submitted: datetime = Field(
-        description="When the job was submitted.", default_factory=datetime.now
+        description="When the job was submitted.",
+        default_factory=get_current_time_incl_timezone,
     )
     started: Union[datetime, None] = Field(
         description="When the job was started.", default=None
@@ -134,6 +148,9 @@ class ActiveJob(KiaraModel):
     )
     _exception: Union[Exception, None] = PrivateAttr(default=None)
 
+    def is_finished(self) -> bool:
+        return self.finished is not None
+
     def _retrieve_id(self) -> str:
         return str(self.job_id)
 
@@ -146,7 +163,6 @@ class ActiveJob(KiaraModel):
 
     @property
     def runtime(self) -> Union[float, None]:
-
         if self.started is None or self.finished is None:
             return None
 
@@ -155,7 +171,6 @@ class ActiveJob(KiaraModel):
 
 
 class JobRuntimeDetails(BaseModel):
-
     # @classmethod
     # def from_manifest(
     #     cls,
@@ -175,20 +190,38 @@ class JobRuntimeDetails(BaseModel):
     submitted: datetime = Field(description="When the job was submitted.")
     started: datetime = Field(description="When the job was started.")
     finished: datetime = Field(description="When the job was finished.")
-    runtime: float = Field(description="The duration of the job.")
+    runtime: float = Field(description="The duration of the job (in seconds).")
+
+    def create_renderable(self, **config: Any) -> RenderableType:
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("Key", style="i")
+        table.add_column("Value")
+
+        table.add_row("submitted", str(self.submitted))
+        table.add_row("started", str(self.started))
+        table.add_row("finished", str(self.finished))
+        table.add_row("runtime", f"{self.runtime} seconds")
+
+        job_log_table = Table(show_header=False, box=box.SIMPLE)
+        job_log_table.add_column("timestamp", style="i")
+        job_log_table.add_column("message")
+        for log in self.job_log.log:
+            job_log_table.add_row(str(log.timestamp), log.msg)
+
+        table.add_row("job log", job_log_table)
+
+        return table
 
 
 class JobRecord(JobConfig):
-
-    _kiara_model_id = "instance.job_record"
+    _kiara_model_id: ClassVar = "instance.job_record"
 
     @classmethod
     def from_active_job(self, kiara: "Kiara", active_job: ActiveJob):
-
         assert active_job.status == JobStatus.SUCCESS
         assert active_job.results is not None
 
-        job_details = JobRuntimeDetails.construct(
+        job_details = JobRuntimeDetails(
             job_log=active_job.job_log,
             submitted=active_job.submitted,
             started=active_job.started,  # type: ignore
@@ -196,53 +229,77 @@ class JobRecord(JobConfig):
             runtime=active_job.runtime,  # type: ignore
         )
 
-        inputs_data_cid = active_job.job_config.calculate_inputs_data_cid(
+        (
+            inputs_data_cid,
+            contains_invalid,
+        ) = active_job.job_config.calculate_inputs_data_cid(
             data_registry=kiara.data_registry
         )
+        inputs_data_hash = str(inputs_data_cid)
+
+        module = kiara.module_registry.create_module(active_job.job_config)
+        is_internal = module.characteristics.is_internal
+
+        env_hashes = {
+            env.model_type_id: str(env.instance_cid)
+            for env in kiara.current_environments.values()
+        }
 
         job_record = JobRecord(
             job_id=active_job.job_id,
+            job_submitted=active_job.submitted,
+            is_internal=is_internal,
             module_type=active_job.job_config.module_type,
             module_config=active_job.job_config.module_config,
+            is_resolved=active_job.job_config.is_resolved,
             inputs=active_job.job_config.inputs,
             outputs=active_job.results,
             runtime_details=job_details,
-            environment_hashes=kiara.environment_registry.environment_hashes,
-            inputs_data_hash=str(inputs_data_cid)
-            if inputs_data_cid is not None
-            else None,
+            environment_hashes=env_hashes,
+            # input_ids_hash=active_job.job_config.input_ids_hash,
+            inputs_data_hash=inputs_data_hash,
         )
+        job_record._manifest_cid = active_job.job_config.manifest_cid
+        job_record._manifest_data = active_job.job_config.manifest_data
+        job_record._jobs_cid = active_job.job_config.job_cid
+        job_record._inputs_cid = active_job.job_config.inputs_cid
         return job_record
 
     job_id: uuid.UUID = Field(description="The globally unique id for this job.")
-    environment_hashes: Mapping[str, Mapping[str, str]] = Field(
+    job_submitted: datetime = Field(description="When the job was submitted.")
+    environment_hashes: Mapping[str, str] = Field(
         description="Hashes for the environments this value was created in."
     )
-    enviroments: Union[Mapping[str, Mapping[str, Any]], None] = Field(
-        description="Information about the environments this value was created in.",
-        default=None,
-    )
-    inputs_data_hash: Union[str, None] = Field(
-        description="A map of the hashes of this jobs inputs."
+    # enviroments: Union[Mapping[str, Mapping[str, Any]], None] = Field(
+    #     description="Information about the environments this value was created in.",
+    #     default=None,
+    # )
+    is_internal: bool = Field(description="Whether this job was created by the system.")
+    # job_hash: str = Field(description="The hash of the job. Calculated from manifest & input_ids hashes.")
+    # manifest_hash: str = Field(description="The hash of the manifest.")
+    # input_ids_hash: str = Field(description="The hash of the field names and input ids (the value_ids/uuids).")
+    inputs_data_hash: str = Field(
+        description="A map of the hashes of this jobs inputs (the hashes of field names and the actual bytes)."
     )
 
     outputs: Dict[str, uuid.UUID] = Field(description="References to the job outputs.")
     runtime_details: Union[JobRuntimeDetails, None] = Field(
         description="Runtime details for the job."
     )
-    job_metadata: Mapping[str, Any] = Field(
-        description="Optional metadata for this job.", default_factory=dict
-    )
+    # job_metadata: Mapping[str, Any] = Field(
+    #     description="Optional metadata for this job.", default_factory=dict
+    # )
 
-    _is_stored: bool = PrivateAttr(default=None)
-    _outputs_hash: Union[int, None] = PrivateAttr(default=None)
+    _is_stored: bool = PrivateAttr(default=None)  # type: ignore
+    _outputs_hash: Union[int, None] = PrivateAttr(default=None)  # type: ignore
 
-    @validator("job_metadata", pre=True)
-    def validate_metadata(cls, value):
-
-        if value is None:
-            value = {}
-        return value
+    # @field_validator("job_metadata", mode="before")
+    # @classmethod
+    # def validate_metadata(cls, value):
+    #
+    #     if value is None:
+    #         value = {}
+    #     return value
 
     def _retrieve_data_to_hash(self) -> Any:
         return {
@@ -252,7 +309,6 @@ class JobRecord(JobConfig):
         }
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         from kiara.utils.output import extract_renderable
 
         include = config.get("include", None)
@@ -260,14 +316,14 @@ class JobRecord(JobConfig):
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("Key", style="i")
         table.add_column("Value")
-        for k in self.__fields__.keys():
+        for k in self.model_fields.keys():
             if include is not None and k not in include:
                 continue
             attr = getattr(self, k)
             v = extract_renderable(attr)
             table.add_row(k, v)
         table.add_row("job hash", self.job_hash)
-        table.add_row("inputs hash", self.inputs_hash)
+        table.add_row("inputs hash", self.input_ids_hash)
         return table
 
     # @property
@@ -280,3 +336,42 @@ class JobRecord(JobConfig):
     #     h = DeepHash(obj, hasher=KIARA_HASH_FUNCTION)
     #     self._outputs_hash = h[obj]
     #     return self._outputs_hash
+
+
+class JobMatcher(KiaraModel):
+    @classmethod
+    def create_matcher(self, **match_options: Any):
+        m = JobMatcher(**match_options)
+        return m
+
+    job_ids: List[uuid.UUID] = Field(
+        description="A list of job ids, if specified, only jobs with one of these ids will be included.",
+        default_factory=list,
+    )
+    allow_internal: bool = Field(description="Allow internal jobs.", default=False)
+    earliest: Union[None, datetime] = Field(
+        description="The earliest time when the job was created.", default=None
+    )
+    latest: Union[None, datetime] = Field(
+        description="The latest time when the job was created.", default=None
+    )
+    operation_inputs: List[uuid.UUID] = Field(
+        description="A list of value ids, if specified, only jobs that use one of them will be included.",
+        default_factory=list,
+    )
+    produced_outputs: List[uuid.UUID] = Field(
+        description="A list of value ids, if specified, only jobs that produced one of them will be included.",
+        default_factory=list,
+    )
+
+    @field_validator("job_ids", mode="before")
+    @classmethod
+    def validate_job_ids(cls, v):
+        if v is None:
+            return []
+        elif isinstance(v, uuid.UUID):
+            return [v]
+        elif isinstance(v, str):
+            return [uuid.UUID(v)]
+        else:
+            return [x if isinstance(x, uuid.UUID) else uuid.UUID(x) for x in v]
