@@ -6,8 +6,9 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 import importlib
+import inspect
 from types import ModuleType
-from typing import TYPE_CHECKING, Any, Dict, Type
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Type, Union
 
 from pydantic.fields import Field, PrivateAttr
 
@@ -23,11 +24,10 @@ if TYPE_CHECKING:
 class PythonClass(KiaraModel):
     """Python class and module information."""
 
-    _kiara_model_id = "instance.wrapped_python_class"
+    _kiara_model_id: ClassVar = "instance.wrapped_python_class"
 
     @classmethod
     def from_class(cls, item_cls: Type, attach_context_metadata: bool = False):
-
         cls_name = item_cls.__name__
         module_name = item_cls.__module__
 
@@ -47,7 +47,7 @@ class PythonClass(KiaraModel):
             ctx_md = ContextMetadataModel.from_class(item_cls=item_cls)
             conf["items"] = ctx_md
 
-        result = PythonClass.construct(**conf)
+        result = PythonClass(**conf)
         result._cls_cache = item_cls
         return result
 
@@ -61,8 +61,9 @@ class PythonClass(KiaraModel):
     #     description="Context metadata for the class.", default=None
     # )
 
-    _module_cache: ModuleType = PrivateAttr(default=None)
-    _cls_cache: Type = PrivateAttr(default=None)
+    _module_cache: Union[ModuleType, None] = PrivateAttr(default=None)
+    _cls_cache: Union[Type, None] = PrivateAttr(default=None)
+    _src_cache: Union[str, None] = PrivateAttr(default=None)
 
     def _retrieve_id(self) -> str:
         return self.full_name
@@ -71,12 +72,16 @@ class PythonClass(KiaraModel):
         return self.full_name
 
     def get_class(self) -> Type:
-
         if self._cls_cache is None:
             m = self.get_python_module()
             self._cls_cache = getattr(m, self.python_class_name)
         return self._cls_cache
 
+    def get_source_code(self) -> str:
+        if self._src_cache is None:
+            self._src_cache = inspect.getsource(self.get_class())
+        return self._src_cache
+
     def get_python_module(self) -> ModuleType:
         if self._module_cache is None:
             self._module_cache = importlib.import_module(self.python_module_name)
@@ -84,12 +89,10 @@ class PythonClass(KiaraModel):
 
 
 class KiaraModuleInstance(PythonClass):
-
-    _kiara_model_id: str = "metadata.kiara_module_class"
+    _kiara_model_id: ClassVar[str] = "metadata.kiara_module_class"
 
     @classmethod
     def from_module(cls, module: "KiaraModule"):
-
         item_cls = module.__class__
 
         cls_name = item_cls.__name__
@@ -105,11 +108,11 @@ class KiaraModuleInstance(PythonClass):
             "full_name": full_name,
         }
 
-        conf["module_config"] = module.config
+        conf["module_config"] = module.config.model_dump()
         conf["inputs_schema"] = module.inputs_schema
         conf["outputs_schema"] = module.outputs_schema
 
-        result = KiaraModuleInstance.construct(**conf)
+        result = KiaraModuleInstance(**conf)
         result._cls_cache = item_cls
         result._module_instance_cache = module
         return result
@@ -122,10 +125,9 @@ class KiaraModuleInstance(PythonClass):
         description="The schema for the module output(s)."
     )
 
-    _module_instance_cache: "KiaraModule" = PrivateAttr(default=None)
+    _module_instance_cache: Union["KiaraModule", None] = PrivateAttr(default=None)
 
     def get_kiara_module_instance(self) -> "KiaraModule":
-
         if self._module_instance_cache is not None:
             return self._module_instance_cache
 
