@@ -12,10 +12,13 @@ import logging
 import os
 import tempfile
 import uuid
+from datetime import datetime
 from typing import (
     TYPE_CHECKING,
     Any,
+    ClassVar,
     Dict,
+    Generator,
     Iterable,
     List,
     Literal,
@@ -31,7 +34,7 @@ from humanfriendly import format_size
 from multiformats import CID, multihash
 from multiformats.multihash import Multihash
 from multiformats.varint import BytesLike
-from pydantic import BaseModel, Extra, PrivateAttr, root_validator
+from pydantic import BaseModel, ConfigDict, PrivateAttr, model_validator
 from pydantic.fields import Field
 from rich import box
 from rich.console import Group, RenderableType
@@ -53,6 +56,7 @@ from kiara.models.python_class import PythonClass
 from kiara.models.values import DataTypeCharacteristics, ValueStatus
 from kiara.models.values.value_schema import ValueSchema
 from kiara.utils import is_jupyter, log_exception
+from kiara.utils.dates import get_current_time_incl_timezone
 from kiara.utils.hashing import create_cid_digest
 from kiara.utils.json import orjson_dumps
 from kiara.utils.yaml import StringYAML
@@ -69,19 +73,17 @@ if TYPE_CHECKING:
 
 
 class SerializedChunks(BaseModel, abc.ABC):
-    class Config:
-        json_loads = orjson.loads
-        json_dumps = orjson_dumps
-        extra = Extra.forbid
+    model_config = ConfigDict(extra="forbid")
 
     _size_cache: Union[int, None] = PrivateAttr(default=None)
     _hashes_cache: Dict[str, Sequence[CID]] = PrivateAttr(default_factory=dict)
 
     @abc.abstractmethod
     def get_chunks(
-        self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
-        """Retrieve the chunks belonging to this data instance.
+        self, as_files: bool = True, symlink_ok: bool = True
+    ) -> Generator[Union[str, "BytesLike"], None, None]:
+        """
+        Retrieve the chunks belonging to this data instance.
 
         If 'as_file' is False, return the data as bytes. If set to 'True' store it to an arbitrary location (or use
         an existing one), and return the path to that file. If 'as_file' is a string, write the data (bytes) into
@@ -102,13 +104,11 @@ class SerializedChunks(BaseModel, abc.ABC):
         pass
 
     def get_size(self) -> int:
-
         if self._size_cache is None:
             self._size_cache = self._get_size()
         return self._size_cache
 
     def get_cids(self, hash_codec: str) -> Sequence[CID]:
-
         if self._hashes_cache.get(hash_codec, None) is None:
             self._hashes_cache[hash_codec] = self._create_cids(hash_codec=hash_codec)
         return self._hashes_cache[hash_codec]
@@ -117,7 +117,6 @@ class SerializedChunks(BaseModel, abc.ABC):
         self, chunks: Iterable[bytes], file: Union[str, None] = None
     ) -> str:
         "Utility method to store bytes to a file."
-
         if file is None:
             file_desc, file = tempfile.mkstemp()
 
@@ -138,7 +137,6 @@ class SerializedChunks(BaseModel, abc.ABC):
         return file
 
     def _read_bytes_from_file(self, file: str) -> bytes:
-
         with open(file, "rb") as f:
             content = f.read()
 
@@ -146,19 +144,16 @@ class SerializedChunks(BaseModel, abc.ABC):
 
 
 class SerializedPreStoreChunks(SerializedChunks):
-
     codec: str = Field(
         description="The codec used to encode the chunks in this model. Using the [multicodecs](https://github.com/multiformats/multicodec) codec table."
     )
 
     def _create_cid_from_chunk(self, chunk: bytes, hash_codec: str) -> CID:
-
         multihash = Multihash(codec=hash_codec)
         hash = multihash.digest(chunk)
         return create_cid_digest(digest=hash, codec=self.codec)
 
     def _create_cid_from_file(self, file: str, hash_codec: str) -> CID:
-
         assert hash_codec == "sha2-256"
 
         hash_func = hashlib.sha256
@@ -176,16 +171,14 @@ class SerializedPreStoreChunks(SerializedChunks):
 
 
 class SerializedBytes(SerializedPreStoreChunks):
-
     type: Literal["chunk"] = "chunk"
     chunk: bytes = Field(description="A byte-array")
 
     def get_chunks(
         self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
-
+    ) -> Generator[Union[str, BytesLike], None, None]:
         if as_files is False:
-            return [self.chunk]
+            yield self.chunk
         else:
             if as_files is True:
                 file = None
@@ -195,7 +188,7 @@ class SerializedBytes(SerializedPreStoreChunks):
                 assert len(as_files) == 1
                 file = as_files[0]
             path = self._store_bytes_to_file([self.chunk], file=file)
-            return path
+            yield path
 
     def get_number_of_chunks(self) -> int:
         return 1
@@ -208,29 +201,27 @@ class SerializedBytes(SerializedPreStoreChunks):
 
 
 class SerializedListOfBytes(SerializedPreStoreChunks):
-
     type: Literal["chunks"] = "chunks"
     chunks: List[bytes] = Field(description="A list of byte arrays.")
 
     def get_chunks(
         self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
+    ) -> Generator[Union[str, BytesLike], None, None]:
         if as_files is False:
-            return self.chunks
+            for chunk in self.chunks:
+                yield chunk
         else:
             if as_files is None or as_files is True or isinstance(as_files, str):
                 # means we write all the chunks into one file
                 file = None if as_files is True else as_files
                 path = self._store_bytes_to_file(self.chunks, file=file)
-                return [path]
+                yield path
             else:
                 assert len(as_files) == self.get_number_of_chunks()
-                result = []
                 for idx, chunk in enumerate(self.chunks):
                     _file = as_files[idx]
                     path = self._store_bytes_to_file([chunk], file=_file)
-                    result.append(path)
-                return result
+                    yield path
 
     def get_number_of_chunks(self) -> int:
         return len(self.chunks)
@@ -249,20 +240,18 @@ class SerializedListOfBytes(SerializedPreStoreChunks):
 
 
 class SerializedFile(SerializedPreStoreChunks):
-
     type: Literal["file"] = "file"
     file: str = Field(description="A path to a file containing the serialized data.")
 
     def get_chunks(
         self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
-
+    ) -> Generator[Union[str, BytesLike], None, None]:
         if as_files is False:
             chunk = self._read_bytes_from_file(self.file)
-            return [chunk]
+            yield chunk
         else:
             if as_files is True:
-                return [self.file]
+                yield self.file
             else:
                 if isinstance(as_files, str):
                     file = as_files
@@ -273,7 +262,7 @@ class SerializedFile(SerializedPreStoreChunks):
                     raise Exception(f"Can't write to file '{file}': file exists.")
                 if symlink_ok:
                     os.symlink(self.file, file)
-                    return [file]
+                    yield file
                 else:
                     raise NotImplementedError()
 
@@ -288,7 +277,6 @@ class SerializedFile(SerializedPreStoreChunks):
 
 
 class SerializedFiles(SerializedPreStoreChunks):
-
     type: Literal["files"] = "files"
     files: List[str] = Field(
         description="A list of strings, pointing to files containing parts of the serialized data."
@@ -296,14 +284,13 @@ class SerializedFiles(SerializedPreStoreChunks):
 
     def get_chunks(
         self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
+    ) -> Generator[Union[str, BytesLike], None, None]:
         raise NotImplementedError()
 
     def get_number_of_chunks(self) -> int:
         return len(self.files)
 
     def _get_size(self) -> int:
-
         size = 0
         for file in self.files:
             size = size + os.path.getsize(os.path.realpath(file))
@@ -317,10 +304,10 @@ class SerializedFiles(SerializedPreStoreChunks):
 
 
 class SerializedInlineJson(SerializedPreStoreChunks):
-
     type: Literal["inline-json"] = "inline-json"
     inline_data: Any = Field(
-        description="Data that will not be stored externally, but inline in the containing model. This should only contain data types that can be serialized reliably using json (scalars, etc.)."
+        None,
+        description="Data that will not be stored externally, but inline in the containing model. This should only contain data types that can be serialized reliably using json (scalars, etc.).",
     )
     _json_cache: Union[bytes, None] = PrivateAttr(default=None)
 
@@ -335,10 +322,9 @@ class SerializedInlineJson(SerializedPreStoreChunks):
 
     def get_chunks(
         self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
-
+    ) -> Generator[Union[str, BytesLike], None, None]:
         if as_files is False:
-            return [self.as_json()]
+            yield self.as_json()
         else:
             raise NotImplementedError()
 
@@ -353,43 +339,45 @@ class SerializedInlineJson(SerializedPreStoreChunks):
 
 
 class SerializedChunkIDs(SerializedChunks):
-
     type: Literal["chunk-ids"] = "chunk-ids"
     chunk_id_list: List[str] = Field(
         description="A list of chunk ids, which will be resolved via the attached data registry."
     )
     archive_id: Union[uuid.UUID, None] = Field(
-        description="The preferred data archive to get the chunks from."
+        None, description="The preferred data archive to get the chunks from."
     )
     size: int = Field(description="The size of all chunks combined.")
-    _data_registry: "DataRegistry" = PrivateAttr(default=None)
+    _data_registry: Union["DataRegistry", None] = PrivateAttr(default=None)
 
     def get_chunks(
-        self, as_files: Union[bool, str, Sequence[str]] = True, symlink_ok: bool = True
-    ) -> Iterable[Union[str, BytesLike]]:
+        self, as_files: bool = True, symlink_ok: bool = True
+    ) -> Generator[Union[str, BytesLike], None, None]:
+        """Retrieve the chunks of this value data.
 
-        if isinstance(as_files, (bool, str)):
-            return (
-                self._data_registry.retrieve_chunk(
-                    chunk_id=chunk,
-                    archive_id=self.archive_id,
-                    as_file=as_files,
-                    symlink_ok=symlink_ok,
-                )
-                for chunk in self.chunk_id_list
-            )
-        else:
-            result = []
-            for idx, chunk_id in enumerate(self.chunk_id_list):
-                file = as_files[idx]
-                self._data_registry.retrieve_chunk(
-                    chunk_id=chunk_id,
-                    archive_id=self.archive_id,
-                    as_file=file,
-                    symlink_ok=symlink_ok,
-                )
-                result.append(file)
-            return result
+        If 'as_files' is 'True', it will return strings representing paths to files containing the chunk data. If symlink_ok is also set to 'True', the returning Path could potentially be a symlink, which means the underlying function might not need to copy the file. In this case, you are responsible to not change the contents of the path, ever.
+
+        If 'as_files' is 'False', BytesLike objects will be returned, containing the chunk data bytes directly.
+
+        """
+
+        chunk_ids = self.chunk_id_list
+        assert self._data_registry is not None
+        return self._data_registry.retrieve_chunks(
+            chunk_ids=chunk_ids,
+            as_files=as_files,
+            symlink_ok=symlink_ok,
+            archive_id=self.archive_id,
+        )
+
+        # return (
+        #     self._data_registry.retrieve_chunk(
+        #         chunk_id=chunk,
+        #         archive_id=self.archive_id,
+        #         as_file=as_files,
+        #         symlink_ok=symlink_ok,
+        #     )
+        #     for chunk in self.chunk_id_list
+        # )
 
     def get_number_of_chunks(self) -> int:
         return len(self.chunk_id_list)
@@ -398,7 +386,6 @@ class SerializedChunkIDs(SerializedChunks):
         return self.size
 
     def _create_cids(self, hash_codec: str) -> Sequence[CID]:
-
         result = []
         for chunk_id in self.chunk_id_list:
             cid = CID.decode(chunk_id)
@@ -418,8 +405,7 @@ SERIALIZE_TYPES = {
 
 
 class SerializationMetadata(KiaraModel):
-
-    _kiara_model_id = "metadata.serialized_data"
+    _kiara_model_id: ClassVar = "metadata.serialized_data"
 
     environment: Mapping[str, int] = Field(
         description="Hash(es) for the environments the value was created/serialized.",
@@ -432,7 +418,6 @@ class SerializationMetadata(KiaraModel):
 
 
 class SerializedData(KiaraModel):
-
     data_type: str = Field(
         description="The name of the data type for this serialized value."
     )
@@ -458,7 +443,6 @@ class SerializedData(KiaraModel):
     # _cached_cid: Optional[CID] = PrivateAttr(default=None)
 
     def _retrieve_data_to_hash(self) -> Any:
-
         return self.dag
 
     @property
@@ -482,7 +466,6 @@ class SerializedData(KiaraModel):
         pass
 
     def get_cids_for_key(self, key) -> Sequence[CID]:
-
         if key in self._cids_cache.keys():
             return self._cids_cache[key]
 
@@ -492,7 +475,6 @@ class SerializedData(KiaraModel):
 
     @property
     def dag(self) -> Mapping[str, Sequence[CID]]:
-
         if self._cached_dag is not None:
             return self._cached_dag
 
@@ -505,8 +487,7 @@ class SerializedData(KiaraModel):
 
 
 class SerializationResult(SerializedData):
-
-    _kiara_model_id = "instance.serialization_result"
+    _kiara_model_id: ClassVar[str] = "instance.serialization_result"
 
     data: Dict[
         str,
@@ -527,9 +508,9 @@ class SerializationResult(SerializedData):
     def get_serialized_data(self, key: str) -> SerializedChunks:
         return self.data[key]
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def validate_data(cls, values):
-
         codec = values.get("codec", None)
         if codec is None:
             codec = "sha2-256"
@@ -555,14 +536,13 @@ class SerializationResult(SerializedData):
                     )
 
                 assert s_type != "chunk-ids"
-                cls = SERIALIZE_TYPES[s_type]
-                result[field_name] = cls(**data)
+                _cls = SERIALIZE_TYPES[s_type]
+                result[field_name] = _cls(**data)
 
         values["data"] = result
         return values
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("key")
         table.add_column("value")
@@ -585,7 +565,6 @@ class SerializationResult(SerializedData):
         return table
 
     def __repr__(self):
-
         return f"{self.__class__.__name__}(type={self.data_type} size={self.data_size})"
 
     def __str__(self):
@@ -593,8 +572,7 @@ class SerializationResult(SerializedData):
 
 
 class PersistedData(SerializedData):
-
-    _kiara_model_id = "instance.persisted_data"
+    _kiara_model_id: ClassVar = "instance.persisted_data"
 
     archive_id: uuid.UUID = Field(
         description="The id of the store that persisted the data."
@@ -611,8 +589,7 @@ class PersistedData(SerializedData):
 
 
 class ValuePedigree(InputsManifest):
-
-    _kiara_model_id = "instance.value_pedigree"
+    _kiara_model_id: ClassVar = "instance.value_pedigree"
 
     kiara_id: uuid.UUID = Field(
         description="The id of the kiara context a value was created in."
@@ -636,8 +613,7 @@ class ValuePedigree(InputsManifest):
 
 
 class DataTypeInfo(KiaraModel):
-
-    _kiara_model_id = "info.data_type_instance"
+    _kiara_model_id: ClassVar = "info.data_type_instance"
 
     data_type_name: str = Field(description="The registered name of this data type.")
     data_type_config: Mapping[str, Any] = Field(
@@ -650,11 +626,10 @@ class DataTypeInfo(KiaraModel):
     data_type_class: PythonClass = Field(
         description="The python class that is associated with this model."
     )
-    _data_type_instance: "DataType" = PrivateAttr(default=None)
+    _data_type_instance: Union["DataType", None] = PrivateAttr(default=None)
 
     @property
     def data_type_instance(self) -> "DataType":
-
         if self._data_type_instance is not None:
             return self._data_type_instance
 
@@ -667,7 +642,7 @@ class DataTypeInfo(KiaraModel):
 class ValueDetails(KiaraModel):
     """A wrapper class that manages and retieves value data and its details."""
 
-    _kiara_model_id = "instance.value_details"
+    _kiara_model_id: ClassVar = "instance.value_details"
 
     value_id: uuid.UUID = Field(description="The id of the value.")
 
@@ -679,6 +654,10 @@ class ValueDetails(KiaraModel):
         description="The schema that was used for this Value."
     )
 
+    value_created: datetime = Field(
+        description="The time when this value was created.",
+        default_factory=get_current_time_incl_timezone,
+    )
     value_status: ValueStatus = Field(description="The set/unset status of this value.")
     value_size: int = Field(description="The size of this value, in bytes.")
     value_hash: str = Field(description="The hash of this value.")
@@ -716,8 +695,7 @@ class ValueDetails(KiaraModel):
 
     @property
     def is_valid(self) -> bool:
-        """Check whether the current value is valid"""
-
+        """Check whether the current value is valid."""
         if self.is_optional:
             return True
         else:
@@ -730,7 +708,6 @@ class ValueDetails(KiaraModel):
     @property
     def value_status_string(self) -> str:
         """Print a human readable short description of this values status."""
-
         if self.value_status == ValueStatus.DEFAULT:
             return "set (default)"
         elif self.value_status == ValueStatus.SET:
@@ -749,22 +726,19 @@ class ValueDetails(KiaraModel):
         return result
 
     def __repr__(self):
-
         return f"{self.__class__.__name__}(id={self.value_id}, type={self.data_type_name}, status={self.value_status.value})"
 
     def __str__(self):
-
         return self.__repr__()
 
 
 class Value(ValueDetails):
-
-    _kiara_model_id = "instance.value"
+    _kiara_model_id: ClassVar = "instance.value"
 
     _value_data: Any = PrivateAttr(default=SpecialValue.NOT_SET)
     _serialized_data: Union[None, str, SerializedData] = PrivateAttr(default=None)
     _data_retrieved: bool = PrivateAttr(default=False)
-    _data_registry: "DataRegistry" = PrivateAttr(default=None)
+    _data_registry: Union["DataRegistry", None] = PrivateAttr(default=None)
     # _data_type: "DataType" = PrivateAttr(default=None)
     _is_stored: bool = PrivateAttr(default=False)
     _cached_properties: Union["ValueMap", None] = PrivateAttr(default=None)
@@ -773,10 +747,10 @@ class Value(ValueDetails):
     environment_hashes: Mapping[str, Mapping[str, str]] = Field(
         description="Hashes for the environments this value was created in."
     )
-    enviroments: Union[Mapping[str, Mapping[str, Any]], None] = Field(
-        description="Information about the environments this value was created in.",
-        default=None,
-    )
+    # enviroments: Union[Mapping[str, Mapping[str, Any]], None] = Field(
+    #     description="Information about the environments this value was created in.",
+    #     default=None,
+    # )
     property_links: Mapping[str, uuid.UUID] = Field(
         description="Links to values that are properties of this value.",
         default_factory=dict,
@@ -785,6 +759,10 @@ class Value(ValueDetails):
         description="Backlinks to values that this value acts as destiny/or property for.",
         default_factory=dict,
     )
+    job_id: Union[uuid.UUID, None] = Field(
+        description="The id of the job that created this value (if applicable).",
+        default=None,
+    )
 
     def add_property(
         self,
@@ -792,7 +770,6 @@ class Value(ValueDetails):
         property_path: str,
         add_origin_to_property_value: bool = True,
     ):
-
         value = None
         try:
             value_temp = value
@@ -835,7 +812,6 @@ class Value(ValueDetails):
         self._cached_properties = None
 
     def add_destiny_details(self, value_id: uuid.UUID, destiny_alias: str):
-
         if self._is_stored:
             raise Exception(
                 f"Can't set destiny_refs to value '{self.value_id}': value already locked."
@@ -845,7 +821,6 @@ class Value(ValueDetails):
 
     @property
     def is_serializable(self) -> bool:
-
         try:
             if self._serialized_data == NO_SERIALIZATION_MARKER:
                 return False
@@ -863,7 +838,6 @@ class Value(ValueDetails):
 
     @property
     def serialized_data(self) -> SerializedData:
-
         # if not self.is_set:
         #     raise Exception(f"Can't retrieve serialized data: value not set.")
 
@@ -875,6 +849,7 @@ class Value(ValueDetails):
 
             return self._serialized_data
 
+        assert self._data_registry is not None
         self._serialized_data = self._data_registry.retrieve_persisted_value_details(
             self.value_id
         )
@@ -893,7 +868,6 @@ class Value(ValueDetails):
             raise dtue
 
     def _retrieve_data(self) -> Any:
-
         if self._value_data is not SpecialValue.NOT_SET:
             return self._value_data
 
@@ -903,6 +877,7 @@ class Value(ValueDetails):
         elif self.value_status not in [ValueStatus.SET, ValueStatus.DEFAULT]:
             raise Exception(f"Invalid internal state of value '{self.value_id}'.")
 
+        assert self._data_registry is not None
         retrieved = self._data_registry.retrieve_value_data(value=self)
 
         if retrieved is None or isinstance(retrieved, SpecialValue):
@@ -920,7 +895,6 @@ class Value(ValueDetails):
     #     )
 
     def __repr__(self):
-
         return f"{self.__class__.__name__}(id={self.value_id}, type={self.data_type_name}, status={self.value_status.value}, initialized={self.is_initialized} optional={self.value_schema.optional})"
 
     def _set_registry(self, data_registry: "DataRegistry") -> None:
@@ -937,7 +911,6 @@ class Value(ValueDetails):
 
     @property
     def data_type(self) -> "DataType":
-
         return self.data_type_info.data_type_instance
 
     @property
@@ -947,15 +920,18 @@ class Value(ValueDetails):
 
         from kiara.models.values.lineage import ValueLineage
 
+        assert self._data_registry is not None
         self._lineage = ValueLineage(kiara=self._data_registry._kiara, value=self)
         return self._lineage
 
     @property
     def property_values(self) -> "ValueMap":
+        """Return a dictionary of all of this values properties."""
 
         if self._cached_properties is not None:
             return self._cached_properties
 
+        assert self._data_registry is not None
         self._cached_properties = self._data_registry.load_values(self.property_links)
         return self._cached_properties
 
@@ -964,28 +940,37 @@ class Value(ValueDetails):
         return self.property_links.keys()
 
     def get_property_value(self, property_key) -> "Value":
-
         if property_key not in self.property_links.keys():
             raise Exception(
                 f"Value '{self.value_id}' has no property with key '{property_key}."
             )
 
+        assert self._data_registry is not None
         return self._data_registry.get_value(self.property_links[property_key])
 
     def get_property_data(self, property_key: str) -> Any:
-
         try:
             return self.get_property_value(property_key=property_key).data
         except Exception as e:
             log_exception(e)
             return None
 
-    def get_all_property_data(self) -> Mapping[str, Any]:
+    def get_all_property_data(self, flatten_models: bool = False) -> Mapping[str, Any]:
+        result = {k: self.get_property_data(k) for k in self.property_names}
+        if not flatten_models:
+            return result
 
-        return {k: self.get_property_data(k) for k in self.property_names}
+        flat = {}
+        for k, v in result.items():
+            if hasattr(v, "model_dump"):
+                flat[k] = v.model_dump()
+            elif hasattr(v, "dict"):
+                flat[k] = v.dict()
+            else:
+                flat[k] = v
+        return flat
 
     def lookup_self_aliases(self) -> Set[str]:
-
         if not self._data_registry:
             raise Exception(
                 f"Can't lookup aliases for value '{self.value_id}': data registry not set (yet)."
@@ -994,7 +979,6 @@ class Value(ValueDetails):
         return self._data_registry.lookup_aliases(self)
 
     def create_info(self) -> "ValueInfo":
-
         if not self._data_registry:
             raise Exception(
                 f"Can't create info object for value '{self.value_id}': data registry not set (yet)."
@@ -1003,7 +987,6 @@ class Value(ValueDetails):
         return self._data_registry.create_value_info(value=self.value_id)
 
     def create_info_data(self, **config: Any) -> Mapping[str, Any]:
-
         show_pedigree = config.get("show_pedigree", False)
         show_lineage = config.get("show_lineage", False)
         show_properties = config.get("show_properties", False)
@@ -1027,8 +1010,7 @@ class Value(ValueDetails):
         if "kiara_id" not in ignore_fields:
             table["kiara_id"] = self.kiara_id
 
-        for k in sorted(self.__fields__.keys()):
-
+        for k in sorted(self.__class__.model_fields.keys()):
             if (
                 k
                 in [
@@ -1066,6 +1048,7 @@ class Value(ValueDetails):
         if show_lineage:
             table["lineage"] = self.lineage
 
+        assert self._data_registry is not None
         if show_serialized:
             serialized = self._data_registry.retrieve_persisted_value_details(
                 self.value_id
@@ -1121,7 +1104,6 @@ class Value(ValueDetails):
         return table
 
     def create_renderable(self, **render_config: Any) -> RenderableType:
-
         from kiara.utils.output import extract_renderable
 
         show_pedigree = render_config.get("show_pedigree", False)
@@ -1158,7 +1140,6 @@ class Value(ValueDetails):
         table.add_row("", "")
         table.add_row("", Rule())
         for k in sorted(info_data.keys()):
-
             if (
                 k
                 in [
@@ -1167,6 +1148,7 @@ class Value(ValueDetails):
                     "aliases",
                     "kiara_id",
                     "lineage",
+                    "properties",
                     "environments",
                     "environment_hashes",
                 ]
@@ -1255,6 +1237,7 @@ class Value(ValueDetails):
                 dr = destinies.create_renderable(show_header=False)
                 table.add_row("destinies", dr)
 
+        assert self._data_registry is not None
         if show_destiny_backlinks:
             if not self.destiny_backlinks:
                 table.add_row("destiny backlinks", "{}")
@@ -1283,11 +1266,13 @@ class Value(ValueDetails):
 
 
 class UnloadableData(KiaraModel):
-    """A special 'marker' model, indicating that the data of value can't be loaded.
+    """
+    A special 'marker' model, indicating that the data of value can't be loaded.
 
-    In most cases, the reason this happens is because the current kiara context is missing some value types and/or modules."""
+    In most cases, the reason this happens is because the current kiara context is missing some value types and/or modules.
+    """
 
-    _kiara_model_id = "instance.unloadable_data"
+    _kiara_model_id: ClassVar = "instance.unloadable_data"
 
     value: Value = Field(description="A reference to the value.")
 
@@ -1299,7 +1284,6 @@ class UnloadableData(KiaraModel):
 
 
 class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
-
     values_schema: Dict[str, ValueSchema] = Field(
         description="The schemas for all the values in this set."
     )
@@ -1327,19 +1311,15 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
 
     def check_invalid(self) -> Dict[str, str]:
         """Check whether the value set is invalid, if it is, return a description of what's wrong."""
-
         invalid: Dict[str, str] = {}
         for field_name in self.values_schema.keys():
-
             item = self.get_value_obj(field_name)
             field_schema = self.values_schema[field_name]
             if not field_schema.optional:
                 msg: Union[str, None] = None
                 if not item.value_status == ValueStatus.SET:
-
                     item_schema = self.values_schema[field_name]
                     if item_schema.is_required():
-
                         if not item.is_set:
                             msg = "not set"
                         elif item.value_status == ValueStatus.NONE:
@@ -1352,12 +1332,12 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
     def get_value_data_for_fields(
         self, *field_names: str, raise_exception_when_unset: bool = False
     ) -> Dict[str, Any]:
-        """Return the data for a one or several fields of this ValueMap.
+        """
+        Return the data for a one or several fields of this ValueMap.
 
         If a value is unset, by default 'None' is returned for it. Unless 'raise_exception_when_unset' is set to 'True',
         in which case an Exception will be raised (obviously).
         """
-
         if raise_exception_when_unset:
             unset: List[str] = []
             for k in field_names:
@@ -1398,7 +1378,6 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
         )
 
     def set_values(self, **values) -> None:
-
         for k, v in values.items():
             self.set_value(k, v)
 
@@ -1408,16 +1387,13 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
         )
 
     def __getitem__(self, item: str) -> Value:
-
         return self.get_value_obj(item)
 
     def __setitem__(self, key: str, value):
-
         raise NotImplementedError()
         # self.set_value(key, value)
 
     def __delitem__(self, key: str):
-
         raise Exception(f"Removing items not supported: {key}")
 
     def __iter__(self):
@@ -1433,7 +1409,6 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
         return self.__repr__()
 
     def create_invalid_renderable(self, **config) -> Union[RenderableType, None]:
-
         inv = self.check_invalid()
         if not inv:
             return None
@@ -1448,7 +1423,6 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
         return table
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         in_panel = config.get("in_panel", None)
         if in_panel is None:
             if is_jupyter():
@@ -1469,9 +1443,9 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
         table.add_column(value_title, style="i")
 
         for field_name in self.field_names:
-
             value = self.get_value_obj(field_name=field_name)
             if render_value_data:
+                assert value._data_registry is not None
                 rendered = value._data_registry.pretty_print_data(
                     value_id=value.value_id, target_type="terminal_renderable", **config
                 )
@@ -1490,24 +1464,24 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
 
 
 class ValueMapReadOnly(ValueMap):  # type: ignore
-
-    _kiara_model_id = "instance.value_map.readonly"
+    _kiara_model_id: ClassVar = "instance.value_map.readonly"
 
     @classmethod
     def create_from_ids(cls, data_registry: "DataRegistry", **value_ids: uuid.UUID):
-
         values = {k: data_registry.get_value(v) for k, v in value_ids.items()}
         values_schema = {k: v.value_schema for k, v in values.items()}
-        return ValueMapReadOnly.construct(
-            value_items=values, values_schema=values_schema
-        )
+        return ValueMapReadOnly(value_items=values, values_schema=values_schema)
+
+    @classmethod
+    def create_from_values(cls, **values: Value) -> "ValueMapReadOnly":
+        values_schema = {k: v.value_schema for k, v in values.items()}
+        return ValueMapReadOnly(value_items=values, values_schema=values_schema)
 
     value_items: Dict[str, Value] = Field(
         description="The values contained in this set."
     )
 
     def get_value_obj(self, field_name: str) -> Value:
-
         if field_name not in self.value_items.keys():
             raise KeyError(
                 f"Field '{field_name}' not available in value set. Available fields: {', '.join(self.field_names)}"
@@ -1516,8 +1490,7 @@ class ValueMapReadOnly(ValueMap):  # type: ignore
 
 
 class ValueMapWritable(ValueMap):  # type: ignore
-
-    _kiara_model_id = "instance.value_map.writeable"
+    _kiara_model_id: ClassVar = "instance.value_map.writeable"
 
     @classmethod
     def create_from_schema(
@@ -1527,7 +1500,6 @@ class ValueMapWritable(ValueMap):  # type: ignore
         pedigree: ValuePedigree,
         unique_value_ids: bool = False,
     ) -> "ValueMapWritable":
-
         v = ValueMapWritable(
             values_schema=dict(schema),
             pedigree=pedigree,
@@ -1549,17 +1521,20 @@ class ValueMapWritable(ValueMap):  # type: ignore
     )
 
     _values_uncommitted: Dict[str, Any] = PrivateAttr(default_factory=dict)
-    _kiara: "Kiara" = PrivateAttr(default=None)
-    _data_registry: "DataRegistry" = PrivateAttr(default=None)
+    _kiara: Union["Kiara", None] = PrivateAttr(default=None)
+    _data_registry: Union["DataRegistry", None] = PrivateAttr(default=None)
     _auto_commit: bool = PrivateAttr(default=True)
 
     def get_value_obj(self, field_name: str) -> Value:
-        """Retrieve the value object for the specified field.
+        """
+        Retrieve the value object for the specified field.
 
         This class only creates the actual value object the first time it is requested, because there is a potential
         cost to assembling it, and it might not be needed ever.
         """
 
+        assert self._data_registry is not None
+
         if field_name not in self.values_schema.keys():
             raise Exception(
                 f"Can't set data for field '{field_name}': field not valid, valid field names: {', '.join(self.field_names)}."
@@ -1592,7 +1567,6 @@ class ValueMapWritable(ValueMap):  # type: ignore
         return self.value_items[field_name]
 
     def sync_values(self):
-
         for field_name in self.field_names:
             self.get_value_obj(field_name)
 
@@ -1608,7 +1582,6 @@ class ValueMapWritable(ValueMap):  # type: ignore
 
     def set_value(self, field_name: str, data: Any) -> None:
         """Set the value for the specified field."""
-
         if field_name not in self.field_names:
             raise Exception(
                 f"Can't set data for field '{field_name}': field not valid, valid field names: {', '.join(self.field_names)}."
@@ -1627,8 +1600,12 @@ class ValueMapWritable(ValueMap):  # type: ignore
             self.get_value_obj(field_name=field_name)
 
 
-ValuePedigree.update_forward_refs()
+ValuePedigree.model_rebuild()
 ORPHAN = ValuePedigree(
-    kiara_id=VOID_KIARA_ID, environments={}, module_type=NO_MODULE_TYPE, inputs={}
+    kiara_id=VOID_KIARA_ID,
+    environments={},
+    module_type=NO_MODULE_TYPE,
+    inputs={},
+    is_resolved=True,
 )
 # GENESIS_PEDIGREE = None
