@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 import datetime
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Type, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, List, Mapping, Type, Union
 
-from dag_cbor import Kind
-from pydantic import Field, PrivateAttr, validator
+from dag_cbor import IPLDKind
+from pydantic import Field, PrivateAttr, field_validator
 from rich import box
 from rich.console import RenderableType
 from rich.panel import Panel
@@ -33,7 +33,6 @@ if TYPE_CHECKING:
 class WorkflowState(KiaraModel):
     @classmethod
     def create_from_workflow(self, workflow: "Workflow"):
-
         steps = list(workflow._steps.values())
         inputs = dict(workflow.current_pipeline_inputs)
         info = PipelineInfo.create_from_pipeline(
@@ -57,23 +56,21 @@ class WorkflowState(KiaraModel):
     )
 
     _pipeline: Union[Pipeline, None] = PrivateAttr(default=None)
-    _kiara: "Kiara" = PrivateAttr(default=None)
+    _kiara: Union["Kiara", None] = PrivateAttr(default=None)
 
-    def _retrieve_data_to_hash(self) -> Kind:
+    def _retrieve_data_to_hash(self) -> IPLDKind:
         return {
             "steps": [s.instance_cid for s in self.steps],
             "inputs": {k: str(v) for k, v in self.inputs.items()},
         }
 
     def set_inputs(self, **inputs: uuid.UUID):
-
         for k, v in inputs.items():
             if k in self.pipeline_config.structure.pipeline_inputs_schema.keys():
                 self.inputs[k] = v
 
     @property
     def pipeline_config(self) -> PipelineConfig:
-
         return self.pipeline_info.pipeline_config
 
     @property
@@ -81,7 +78,6 @@ class WorkflowState(KiaraModel):
         return self.pipeline_info.pipeline_config.structure
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         in_panel = config.get("in_panel", None)
         if in_panel is None:
             if is_jupyter():
@@ -102,7 +98,7 @@ class WorkflowState(KiaraModel):
 
 
 class WorkflowMetadata(KiaraModel):
-    _kiara_model_id = "instance.workflow"
+    _kiara_model_id: ClassVar = "instance.workflow"
 
     workflow_id: uuid.UUID = Field(
         description="The globaly unique uuid for this workflow."
@@ -143,7 +139,8 @@ class WorkflowMetadata(KiaraModel):
     _kiara: Union["Kiara", None] = PrivateAttr(default=None)
     # _last_update: datetime.datetime = PrivateAttr(default_factory=datetime.datetime.now)
 
-    @validator("documentation", pre=True)
+    @field_validator("documentation", mode="before")
+    @classmethod
     def validate_doc(cls, value):
         if not isinstance(value, DocumentationMetadataModel):
             return DocumentationMetadataModel.create(value)
@@ -152,7 +149,6 @@ class WorkflowMetadata(KiaraModel):
 
     @property
     def last_state_id(self) -> Union[None, str]:
-
         if not self.workflow_history:
             return None
         last_date = max(self.workflow_history.keys())
@@ -161,13 +157,11 @@ class WorkflowMetadata(KiaraModel):
 
 
 class WorkflowInfo(ItemInfo):
-
-    _kiara_model_id = "info.workflow"
+    _kiara_model_id: ClassVar = "info.workflow"
 
     @classmethod
-    def create_from_workflow(cls, workflow: "Workflow"):
-
-        wf_info = WorkflowInfo.construct(
+    def create_from_workflow(cls, workflow: "Workflow") -> "WorkflowInfo":
+        wf_info = WorkflowInfo(
             type_name=str(workflow.workflow_id),
             workflow_metadata=workflow.workflow_metadata,
             workflow_state_ids=workflow.all_state_ids,
@@ -194,7 +188,6 @@ class WorkflowInfo(ItemInfo):
 
     @classmethod
     def create_from_instance(cls, kiara: "Kiara", instance: "Workflow", **kwargs):
-
         return cls.create_from_workflow(workflow=instance)
 
     workflow_metadata: WorkflowMetadata = Field(description="The workflow details.")
@@ -216,7 +209,6 @@ class WorkflowInfo(ItemInfo):
     )
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         in_panel = config.get("in_panel", None)
         if in_panel is None:
             if is_jupyter():
@@ -288,8 +280,7 @@ class WorkflowInfo(ItemInfo):
 
 
 class WorkflowGroupInfo(InfoItemGroup):
-
-    _kiara_model_id = "info.workflows"
+    _kiara_model_id: ClassVar = "info.workflows"
 
     @classmethod
     def base_info_class(cls) -> Type[ItemInfo]:
@@ -300,16 +291,15 @@ class WorkflowGroupInfo(InfoItemGroup):
         cls,
         *items: "Workflow",
         group_title: Union[str, None] = None,
-        alias_map: Union[None, Mapping[str, uuid.UUID]] = None
+        alias_map: Union[None, Mapping[str, uuid.UUID]] = None,
     ) -> "WorkflowGroupInfo":
-
         workflow_infos = {
             str(w.workflow_id): WorkflowInfo.create_from_workflow(workflow=w)
             for w in items
         }
         if alias_map is None:
             alias_map = {}
-        workflow_group_info = cls.construct(
+        workflow_group_info = cls(
             group_title=group_title, item_infos=workflow_infos, aliases=alias_map
         )
         return workflow_group_info
@@ -322,7 +312,6 @@ class WorkflowGroupInfo(InfoItemGroup):
     )
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         table = Table(box=box.SIMPLE, show_header=True)
         table.add_column("alias(es)", style="i")
         table.add_column("workflow_id")
@@ -332,7 +321,6 @@ class WorkflowGroupInfo(InfoItemGroup):
         table.add_column("description")
 
         for workflow_id, wf in self.item_infos.items():
-
             aliases = [k for k, v in self.aliases.items() if str(v) == workflow_id]
             steps = len(wf.pipeline_info.pipeline_config.structure.steps)
             stages = len(wf.pipeline_info.pipeline_config.structure.processing_stages)
