@@ -23,7 +23,7 @@ from typing import (
 
 import structlog
 from multiformats import CID
-from pydantic import BaseModel, Field, ValidationError
+from pydantic import BaseModel, ConfigDict, Field, ValidationError
 from rich.console import RenderableType
 
 from kiara.exceptions import (
@@ -32,10 +32,11 @@ from kiara.exceptions import (
     KiaraModuleConfigException,
     KiaraProcessingException,
 )
+from kiara.models.documentation import DocumentationMetadataModel
 from kiara.models.module import KiaraModuleConfig
 from kiara.models.module.jobs import JobLog
 from kiara.models.values.value_schema import ValueSchema
-from kiara.utils import is_debug, log_exception
+from kiara.utils import is_debug, is_develop, log_exception
 from kiara.utils.hashing import compute_cid
 from kiara.utils.values import (
     augment_values,
@@ -50,6 +51,7 @@ if TYPE_CHECKING:
     from kiara.models.values.value import ValueMap
     from kiara.operations import Operation
 
+
 yaml = StringYAML()
 
 KIARA_CONFIG = TypeVar("KIARA_CONFIG", bound=KiaraModuleConfig)
@@ -63,7 +65,8 @@ log = structlog.getLogger()
 
 
 class InputOutputObject(abc.ABC):
-    """Abstract base class for classes that define inputs and outputs schemas.
+    """
+    Abstract base class for classes that define inputs and outputs schemas.
 
     Both the 'create_inputs_schema` and `creawte_outputs_schema` methods implemented by child classes return a description of the input schema of this module.
 
@@ -89,7 +92,6 @@ class InputOutputObject(abc.ABC):
         allow_empty_inputs_schema: bool = False,
         allow_empty_outputs_schema: bool = False,
     ):
-
         self._alias: str = alias
         self._inputs_schema: Mapping[str, ValueSchema] = None  # type: ignore
         self._full_inputs_schema: Mapping[str, ValueSchema] = None  # type: ignore
@@ -108,7 +110,6 @@ class InputOutputObject(abc.ABC):
         return self._alias
 
     def input_required(self, input_name: str):
-
         if input_name not in self._inputs_schema.keys():
             raise Exception(
                 f"No input '{input_name}', available inputs: {', '.join(self._inputs_schema)}"
@@ -137,7 +138,6 @@ class InputOutputObject(abc.ABC):
     @property
     def inputs_schema(self) -> Mapping[str, ValueSchema]:
         """The input schema for this module."""
-
         if self._inputs_schema is None:
             self._create_inputs_schema()
 
@@ -145,7 +145,6 @@ class InputOutputObject(abc.ABC):
 
     @property
     def full_inputs_schema(self) -> Mapping[str, ValueSchema]:
-
         if self._full_inputs_schema is not None:
             return self._full_inputs_schema
 
@@ -155,13 +154,13 @@ class InputOutputObject(abc.ABC):
 
     @property
     def constants(self) -> Mapping[str, ValueSchema]:
-
         if self._constants is None:
             self._create_inputs_schema()
         return self._constants  # type: ignore
 
     def _create_inputs_schema(self) -> None:
-        """Assemble the inputs schema and assign it to the approriate instance attributes.
+        """
+        Assemble the inputs schema and assign it to the approriate instance attributes.
 
         DEV NOTE: if anything in this method is changed, also change the method of the AutoInputsKiaraModule
         in the kiara_pluginc.core_types package, since it's a carbon copy if this, except for a small change.
@@ -180,6 +179,7 @@ class InputOutputObject(abc.ABC):
                 )
             try:
                 _input_schemas = create_schema_dict(schema_config=_input_schemas_data)
+
             except Exception as e:
                 raise Exception(f"Can't create input schemas for '{self.alias}': {e}")
 
@@ -203,12 +203,13 @@ class InputOutputObject(abc.ABC):
             )
 
         except Exception as e:
-            raise KiaraException(f"Can't create input schemas for instance of '{self.alias}'.", parent=e)  # type: ignore
+            raise KiaraException(
+                f"Can't create input schemas for instance of '{self.alias}'.", parent=e
+            )  # type: ignore
 
     @property
     def outputs_schema(self) -> Mapping[str, ValueSchema]:
         """The output schema for this module."""
-
         if self._outputs_schema is not None:
             return self._outputs_schema
 
@@ -254,7 +255,6 @@ class InputOutputObject(abc.ABC):
         return self.outputs_schema.keys()
 
     def augment_module_inputs(self, inputs: Mapping[str, Any]) -> Dict[str, Any]:
-
         augmented = augment_values(
             values=inputs, schemas=self.inputs_schema, constants=self.constants
         )
@@ -266,8 +266,7 @@ class InputOutputObject(abc.ABC):
 
 
 class ModuleCharacteristics(BaseModel):
-    class Config:
-        allow_mutation = False
+    model_config = ConfigDict(frozen=True)
 
     is_idempotent: bool = Field(
         description="Whether this module is idempotent (aka always produces the same output with the same inputs.",
@@ -293,7 +292,8 @@ DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS = ModuleCharacteristics(
 
 
 class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
-    """The base class that every custom module in *Kiara* needs to inherit from.
+    """
+    The base class that every custom module in *Kiara* needs to inherit from.
 
     The core of every ``KiaraModule`` is a ``process`` method, which should be a 'pure',
      idempotent function that creates one or several output values from the given input(s), and its purpose is to transfor
@@ -309,14 +309,16 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
      ``_module_type_name`` class attribute.
 
     Examples:
-
+    --------
         A simple example would be an 'addition' module, with ``a`` and ``b`` configured as inputs, and ``z`` as the output field name.
 
         An implementing class would look something like this:
 
-        TODO
+    Todo:
+    ----
 
     Arguments:
+    ---------
         module_config: the configuation for this module
     """
 
@@ -332,21 +334,20 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     def _calculate_module_cid(
         cls, module_type_config: Union[Mapping[str, Any], KIARA_CONFIG]
     ) -> CID:
-
         if isinstance(module_type_config, Mapping):
             module_type_config = cls._resolve_module_config(**module_type_config)
 
         obj = {
             "module_type": cls._module_type_name,  # type: ignore
-            "module_type_config": module_type_config.dict(),
+            "module_type_config": module_type_config.instance_cid,
         }
+
         _, cid = compute_cid(data=obj)
         return cid
 
     @classmethod
     def _resolve_module_config(cls, **config: Any) -> KIARA_CONFIG:
-
-        _config = cls._config_cls(**config)  # type: ignore
+        _config: KIARA_CONFIG = cls._config_cls(**config)
 
         return _config
 
@@ -376,6 +377,7 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         self._module_cid: Union[CID, None] = None
         self._characteristics: Union[ModuleCharacteristics, None] = None
 
+        self._cached_doc: Union[None, DocumentationMetadataModel] = None
         super().__init__(alias=self.__class__._module_type_name, config=self._config)  # type: ignore
 
         self._operation: Union[Operation, None] = None
@@ -385,13 +387,27 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     @property
     def manifest(self) -> "Manifest":
         if self._manifest_cache is None:
+            from kiara.models.module.manifest import Manifest
+
             self._manifest_cache = Manifest(
                 module_type=self.module_type_name,
-                module_config=self.config.dict(),
+                module_config=self.config.model_dump(),
                 is_resolved=True,
             )
         return self._manifest_cache
 
+    @property
+    def doc(self) -> "DocumentationMetadataModel":
+        if self._cached_doc is None:
+            if hasattr(self, "get_operation_doc"):
+                doc = self.get_operation_doc()
+                self._cached_doc = DocumentationMetadataModel.create(doc)
+            else:
+                self._cached_doc = DocumentationMetadataModel.from_class_doc(
+                    self.__class__
+                )
+        return self._cached_doc
+
     @property
     def module_id(self) -> uuid.UUID:
         """The id of this module."""
@@ -407,16 +423,17 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
 
     @property
     def config(self) -> KIARA_CONFIG:
-        """Retrieve the configuration object for this module.
+        """
+        Retrieve the configuration object for this module.
 
         Returns:
+        -------
             the module-class-specific config object
         """
         return self._config
 
     @property
     def module_instance_cid(self) -> CID:
-
         if self._module_cid is None:
             self._module_cid = self.__class__._calculate_module_cid(self._config)
         return self._module_cid
@@ -430,19 +447,20 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         return self._characteristics
 
     def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
-
         return DEFAULT_IDEMPOTENT_MODULE_CHARACTERISTICS
 
     def get_config_value(self, key: str) -> Any:
-        """Retrieve the value for a specific configuration option.
+        """
+        Retrieve the value for a specific configuration option.
 
         Arguments:
+        ---------
             key: the config key
 
         Returns:
+        -------
             the value for the provided key
         """
-
         try:
             return self.config.get(key)
         except Exception:
@@ -451,18 +469,20 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
             )
 
     def run(self, kiara: "Kiara", **inputs: Any) -> "ValueMap":
-        """Run the module ad-hoc.
+        """
+        Run the module ad-hoc.
 
         This is mostly used in unit tests, you typically want to run a module via the KiaraAPI instance.
 
         Arguments:
+        ---------
             kiara: the kiara context
             inputs: the inputs for this module
 
         Returns:
+        -------
             the outputs of this module run as a ValueMap instance
         """
-
         from kiara.models.values.value import ValueMap, ValueMapWritable, ValuePedigree
 
         _inputs: ValueMap = kiara.data_registry.create_valuemap(
@@ -482,7 +502,7 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         result_pedigree = ValuePedigree(
             kiara_id=kiara.id,
             module_type=self.module_type_name,
-            module_config=self.config.dict(),
+            module_config=self.config.model_dump(),
             inputs={k: v.value_id for k, v in _inputs.items()},
             environments=environments,
         )
@@ -503,16 +523,17 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     def process_step(
         self, inputs: "ValueMap", outputs: "ValueMap", job_log: JobLog
     ) -> None:
-        """Kick off processing for a specific set of input/outputs.
+        """
+        Kick off processing for a specific set of input/outputs.
 
         This method calls the implemented [process][kiara.module.KiaraModule.process] method of the inheriting class,
         as well as wrapping input/output-data related functionality.
 
         Arguments:
+        ---------
             inputs: the input value set
             outputs: the output value set
         """
-
         signature = inspect.signature(self.process)  # type: ignore
 
         process_inputs: Dict[str, Any] = {
@@ -529,8 +550,13 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
             log_exception(kpe)
             raise kpe
         except Exception as e:
-            log_exception(e)
-            raise KiaraProcessingException(e, module=self, inputs=inputs)
+            if is_debug() or is_develop():
+                import traceback
+
+                traceback.print_exc()
+            exc = KiaraProcessingException(e, module=self, inputs=inputs)
+            log_exception(exc)
+            raise exc
 
     def __eq__(self, other):
         if self.__class__ != other.__class__:
@@ -545,7 +571,6 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
 
     @property
     def operation(self) -> "Operation":
-
         if self._operation is not None:
             return self._operation
 
@@ -555,5 +580,4 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         return self._operation
 
     def create_renderable(self, **config) -> RenderableType:
-
         return self.operation.create_renderable(**config)
