@@ -19,13 +19,13 @@ from kiara.registries.models import ModelRegistry
 
 
 class MetadataModuleConfig(KiaraModuleConfig):
-
     data_type: str = Field(description="The data type this module will be used for.")
     kiara_model_id: str = Field(description="The id of the kiara (metadata) model.")
 
 
 class ExtractMetadataModule(KiaraModule):
-    """Base class to use when writing a module to extract metadata from a file.
+    """
+    Base class to use when writing a module to extract metadata from a file.
 
     It's possible to use any arbitrary *kiara* module for this purpose, but sub-classing this makes it easier.
     """
@@ -34,7 +34,6 @@ class ExtractMetadataModule(KiaraModule):
     _module_type_name: str = "value.extract_metadata"
 
     def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
-
         return ModuleCharacteristics(
             is_idempotent=True, is_internal=True, unique_result_values=True
         )
@@ -42,7 +41,6 @@ class ExtractMetadataModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         data_type_name = self.get_config_value("data_type")
         inputs = {
             "value": {
@@ -56,7 +54,6 @@ class ExtractMetadataModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         kiara_model_id: str = self.get_config_value("kiara_model_id")
 
         # TODO: check it's subclassing the right class
@@ -72,13 +69,14 @@ class ExtractMetadataModule(KiaraModule):
         return outputs
 
     def process(self, inputs: ValueMap, outputs: ValueMap) -> None:
-
         value = inputs.get_value_obj("value")
 
         kiara_model_id: str = self.get_config_value("kiara_model_id")
 
         model_registry = ModelRegistry.instance()
-        metadata_model_cls: Type[ValueMetadata] = model_registry.get_model_cls(kiara_model_id=kiara_model_id, required_subclass=ValueMetadata)  # type: ignore
+        metadata_model_cls: Type[ValueMetadata] = model_registry.get_model_cls(
+            kiara_model_id=kiara_model_id, required_subclass=ValueMetadata
+        )  # type: ignore
 
         metadata = metadata_model_cls.create_value_metadata(value=value)
 
