@@ -5,7 +5,7 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
-from typing import Iterable, Mapping, Type, Union
+from typing import ClassVar, Iterable, Mapping, Type, Union
 
 from pydantic import Field
 
@@ -51,7 +51,8 @@ class ExtractMetadataDetails(BaseOperationDetails):
 
 
 class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
-    """An operation that extracts metadata of a specific type from value data.
+    """
+    An operation that extracts metadata of a specific type from value data.
 
     For a module profile to be picked up by this operation type, it needs to have:
     - exactly one input field
@@ -59,12 +60,11 @@ class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
     - exactly one output field, whose field name is called 'value_metadata', and where the value has the type 'internal_model'
     """
 
-    _operation_type_name = "extract_metadata"
+    _operation_type_name: ClassVar[str] = "extract_metadata"
 
     def retrieve_included_operation_configs(
         self,
     ) -> Iterable[Union[Mapping, OperationConfig]]:
-
         model_registry = ModelRegistry.instance()
         all_models = model_registry.get_models_of_type(ValueMetadata)
 
@@ -76,7 +76,6 @@ class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
             if isinstance(data_types, str):
                 data_types = [data_types]
             for data_type in data_types:
-
                 config = {
                     "module_type": "value.extract_metadata",
                     "module_config": {
@@ -92,7 +91,6 @@ class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
     def check_matching_operation(
         self, module: "KiaraModule"
     ) -> Union[ExtractMetadataDetails, None]:
-
         if len(module.outputs_schema) != 1:
             return None
         if (
@@ -106,7 +104,7 @@ class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
         input_field_name = next(iter(module.inputs_schema.keys()))
         input_schema = module.inputs_schema.get(input_field_name)
         assert input_schema is not None
-        if input_field_name != input_schema.type and input_field_name != "value":
+        if input_field_name not in (input_schema.type, "value"):
             return None
 
         data_type_name = module.inputs_schema["value"].type
@@ -124,29 +122,33 @@ class ExtractMetadataOperationType(OperationType[ExtractMetadataDetails]):
         else:
             op_id = f"extract.{metadata_key}.metadata.from.{data_type_name}"
 
-        details = ExtractMetadataDetails.create_operation_details(
-            module_inputs_schema=module.inputs_schema,
-            module_outputs_schema=module.outputs_schema,
-            operation_id=op_id,
-            data_type=data_type_name,
-            metadata_key=metadata_key,
-            input_field_name=input_field_name,
-            result_field_name="value_metadata",
-            is_internal_operation=True,
+        details: ExtractMetadataDetails = (
+            ExtractMetadataDetails.create_operation_details(
+                module_inputs_schema=module.inputs_schema,
+                module_outputs_schema=module.outputs_schema,
+                operation_id=op_id,
+                data_type=data_type_name,
+                metadata_key=metadata_key,
+                input_field_name=input_field_name,
+                result_field_name="value_metadata",
+                is_internal_operation=True,
+            )
         )
 
         return details
 
     def get_operations_for_data_type(self, data_type: str) -> Mapping[str, Operation]:
-        """Return all available metadata extract operations for the provided type (and it's parent types).
+        """
+        Return all available metadata extract operations for the provided type (and it's parent types).
 
         Arguments:
+        ---------
             data_type: the value type
 
         Returns:
+        -------
             a mapping with the metadata type as key, and the operation as value
         """
-
         lineage = set(
             self._kiara.type_registry.get_type_lineage(data_type_name=data_type)
         )
