@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 import os
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Iterable, Mapping, Union
 
 import structlog
 from pydantic import Field, PrivateAttr
@@ -34,7 +34,6 @@ yaml = YAML(typ="safe")
 
 
 class PipelineOperationDetails(OperationDetails):
-
     pipeline_inputs_schema: Mapping[str, ValueSchema] = Field(
         description="The input schema for the pipeline."
     )
@@ -42,10 +41,9 @@ class PipelineOperationDetails(OperationDetails):
         description="The output schema for the pipeline."
     )
     pipeline_config: PipelineConfig = Field(description="The pipeline config.")
-    _op_schema: OperationSchema = PrivateAttr(default=None)
+    _op_schema: Union[OperationSchema, None] = PrivateAttr(default=None)
 
     def get_operation_schema(self) -> OperationSchema:
-
         if self._op_schema is not None:
             return self._op_schema
 
@@ -64,24 +62,21 @@ class PipelineOperationDetails(OperationDetails):
 
 
 class PipelineOperationType(OperationType[PipelineOperationDetails]):
-
-    _operation_type_name = "pipeline"
+    _operation_type_name: ClassVar[str] = "pipeline"
 
     def __init__(self, kiara: "Kiara", op_type_name: str) -> None:
-
         super().__init__(kiara=kiara, op_type_name=op_type_name)
         self._pipelines: Union[None, Mapping[str, Mapping[str, Any]]] = None
 
     @property
     def pipeline_data(self) -> Mapping[str, Mapping[str, Any]]:
-
         if self._pipelines is not None:
             return self._pipelines
 
         ignore_errors = False
-        pipeline_paths: Dict[
-            str, Union[Dict[str, Any], None]
-        ] = find_all_kiara_pipeline_paths(skip_errors=ignore_errors)
+        pipeline_paths: Dict[str, Union[Dict[str, Any], None]] = (
+            find_all_kiara_pipeline_paths(skip_errors=ignore_errors)
+        )
 
         for ep in self._kiara.context_config.extra_pipelines:
             ep = os.path.realpath(ep)
@@ -93,7 +88,6 @@ class PipelineOperationType(OperationType[PipelineOperationDetails]):
     def retrieve_included_operation_configs(
         self,
     ) -> Iterable[Union[Mapping, OperationConfig]]:
-
         op_configs = []
         for pipeline_name, pipeline_data in self.pipeline_data.items():
             pipeline_config: Dict[str, Any] = dict(pipeline_data["data"])
@@ -113,14 +107,14 @@ class PipelineOperationType(OperationType[PipelineOperationDetails]):
     def check_matching_operation(
         self, module: "KiaraModule"
     ) -> Union[PipelineOperationDetails, None]:
-
         if isinstance(module, PipelineModule):
-
-            op_details = PipelineOperationDetails.create_operation_details(
-                operation_id=module.config.pipeline_name,
-                pipeline_inputs_schema=module.inputs_schema,
-                pipeline_outputs_schema=module.outputs_schema,
-                pipeline_config=module.config,
+            op_details: PipelineOperationDetails = (
+                PipelineOperationDetails.create_operation_details(
+                    operation_id=module.config.pipeline_name,
+                    pipeline_inputs_schema=module.inputs_schema,
+                    pipeline_outputs_schema=module.outputs_schema,
+                    pipeline_config=module.config,
+                )
             )
             return op_details
         else:
