@@ -5,7 +5,7 @@
 #
 # Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Iterable, List, Mapping, Union
 
 from pydantic import Field
 
@@ -24,7 +24,6 @@ if TYPE_CHECKING:
 
 
 class DeSerializeDetails(BaseOperationDetails):
-
     value_type: str = Field(
         "The name of the input field for the serialized version of the value."
     )
@@ -66,7 +65,8 @@ class DeSerializeDetails(BaseOperationDetails):
 
 
 class DeSerializeOperationType(OperationType[DeSerializeDetails]):
-    """An operation that takes a value, and serializes it into the format suitable to the [`serialized_value`][kiara.data_types.included_core_types.SeriailzedValue] value type.
+    """
+    An operation that takes a value, and serializes it into the format suitable to the [`serialized_value`][kiara.data_types.included_core_types.SeriailzedValue] value type.
 
     For a module profile to be picked up by this operation type, it needs to have:
     - exactly one output field of type `serialized_value`
@@ -76,14 +76,13 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
       - an input field called 'value'
     """
 
-    _operation_type_name = "deserialize"
+    _operation_type_name: ClassVar[str] = "deserialize"
 
     def retrieve_included_operation_configs(
         self,
     ) -> Iterable[Union[Mapping, OperationConfig]]:
         result = []
         for name, module_cls in self._kiara.module_type_classes.items():
-
             if not hasattr(module_cls, "retrieve_serialized_value_type"):
                 continue
             if not hasattr(module_cls, "retrieve_supported_target_profiles"):
@@ -98,7 +97,9 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
                     f"Can't retrieve source value type for deserialization module '{module_cls.__name__}'. This is most likely a bug, maybe you are missing a '@classmethod' annotation on the 'retrieve_source_value_type' method?"
                 )
             try:
-                serialization_profile = module_cls.retrieve_supported_serialization_profile()  # type: ignore
+                serialization_profile = (
+                    module_cls.retrieve_supported_serialization_profile()  # type: ignore
+                )  # type: ignore
             except TypeError:
                 raise Exception(
                     f"Can't retrieve supported serialization profiles for deserialization module '{module_cls.__name__}'. This is most likely a bug, maybe you are missing a '@classmethod' annotation on the 'retrieve_supported_serialization_profile' method?"
@@ -118,7 +119,7 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
                 mc = {
                     "value_type": value_type,
                     "target_profile": _profile_name,
-                    "serialization_profile": serialization_profile
+                    "serialization_profile": serialization_profile,
                     # "target_class": PythonClass.from_class(cls),
                 }
                 oc = ManifestOperationConfig(
@@ -131,7 +132,6 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
     def check_matching_operation(
         self, module: "KiaraModule"
     ) -> Union[DeSerializeDetails, None]:
-
         details = self.extract_details(module)
 
         if details is None:
@@ -140,7 +140,6 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
             return details
 
     def extract_details(self, module: "KiaraModule") -> Union[DeSerializeDetails, None]:
-
         result_field_name = None
         for field_name, schema in module.outputs_schema.items():
             if schema.type != "python_object":
@@ -189,7 +188,9 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
             )
             return None
 
-        if value_type not in self._kiara.type_registry.data_type_names:
+        if value_type not in self._kiara.type_registry.get_data_type_names(
+            include_profiles=True
+        ):
             log_message(
                 "ignore.operation",
                 reason=f"Invalid value type: {value_type}",
@@ -215,13 +216,12 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
             "is_internal_operation": True,
         }
 
-        result = DeSerializeDetails.construct(**details)
+        result = DeSerializeDetails(**details)
         return result
 
     def find_deserialization_operations_for_type(
         self, type_name: str
     ) -> List[Operation]:
-
         lineage = self._kiara.type_registry.get_type_lineage(type_name)
         result = []
         for data_type in lineage:
@@ -238,7 +238,6 @@ class DeSerializeOperationType(OperationType[DeSerializeDetails]):
     def find_deserialzation_operation_for_type_and_profile(
         self, type_name: str, serialization_profile: str
     ) -> List[Operation]:
-
         lineage = self._kiara.type_registry.get_type_lineage(type_name)
         serialize_ops: List[Operation] = []
         for data_type in lineage:
