@@ -9,7 +9,9 @@ import os
 import shutil
 import uuid
 from pathlib import Path
-from typing import Mapping, Set, Union
+from typing import Any, Mapping, Set, Union
+
+import orjson
 
 from kiara.registries import ARCHIVE_CONFIG_CLS, FileSystemArchiveConfig
 from kiara.registries.aliases import AliasArchive, AliasStore
@@ -17,19 +19,45 @@ from kiara.utils.windows import fix_windows_longpath
 
 
 class FileSystemAliasArchive(AliasArchive):
-
     _archive_type_name = "filesystem_alias_archive"
     _config_cls = FileSystemArchiveConfig  # type: ignore
 
-    def __init__(self, archive_id: uuid.UUID, config: ARCHIVE_CONFIG_CLS):
-
-        super().__init__(archive_id=archive_id, config=config)
+    def __init__(
+        self,
+        archive_name: str,
+        archive_config: ARCHIVE_CONFIG_CLS,
+        force_read_only: bool = False,
+    ):
+        super().__init__(
+            archive_name=archive_name,
+            archive_config=archive_config,
+            force_read_only=force_read_only,
+        )
 
         self._base_path: Union[Path, None] = None
 
+    def _retrieve_archive_metadata(self) -> Mapping[str, Any]:
+        if not self.archive_metadata_path.is_file():
+            _archive_metadata = {}
+        else:
+            _archive_metadata = orjson.loads(self.archive_metadata_path.read_bytes())
+
+        archive_id = _archive_metadata.get("archive_id", None)
+        if not archive_id:
+            try:
+                _archive_id = uuid.UUID(
+                    self.alias_store_path.name
+                )  # just to check if it's a valid UUID
+                _archive_metadata["archive_id"] = str(_archive_id)
+            except Exception:
+                raise Exception(
+                    f"Could not retrieve archive id for alias archive '{self.archive_name}'."
+                )
+
+        return _archive_metadata
+
     @property
     def alias_store_path(self) -> Path:
-
         if self._base_path is not None:
             return self._base_path
 
@@ -38,6 +66,10 @@ class FileSystemAliasArchive(AliasArchive):
         self._base_path.mkdir(parents=True, exist_ok=True)
         return self._base_path
 
+    @property
+    def archive_metadata_path(self) -> Path:
+        return self.alias_store_path / "store_metadata.json"
+
     @property
     def aliases_path(self) -> Path:
         return self.alias_store_path / "aliases"
@@ -50,7 +82,6 @@ class FileSystemAliasArchive(AliasArchive):
         shutil.rmtree(self.alias_store_path)
 
     def _translate_alias(self, alias: str) -> Path:
-
         if "." in alias:
             tokens = alias.split(".")
             alias_path = (
@@ -61,7 +92,6 @@ class FileSystemAliasArchive(AliasArchive):
         return alias_path
 
     def _translate_alias_path(self, alias_path: Path) -> str:
-
         relative = (
             alias_path.absolute()
             .relative_to(self.aliases_path.absolute())
@@ -78,7 +108,6 @@ class FileSystemAliasArchive(AliasArchive):
         return alias
 
     def _translate_value_id(self, value_id: uuid.UUID) -> Path:
-
         tokens = str(value_id).split("-")
         value_id_path = (
             self.value_id_path.joinpath(*tokens[0:-1]) / f"{tokens[-1]}.value"
@@ -86,7 +115,6 @@ class FileSystemAliasArchive(AliasArchive):
         return value_id_path
 
     def _translate_value_path(self, value_path: Path) -> uuid.UUID:
-
         relative = (
             fix_windows_longpath(value_path.absolute())
             .relative_to(fix_windows_longpath(self.value_id_path.absolute()))
@@ -98,7 +126,6 @@ class FileSystemAliasArchive(AliasArchive):
         return uuid.UUID(value_id_str)
 
     def retrieve_all_aliases(self) -> Mapping[str, uuid.UUID]:
-
         all_aliases = self.aliases_path.rglob("*.alias")
         result = {}
         for alias_path in all_aliases:
@@ -116,7 +143,6 @@ class FileSystemAliasArchive(AliasArchive):
         return self._find_value_id_for_alias_path(alias_path=alias_path)
 
     def _find_value_id_for_alias_path(self, alias_path: Path) -> Union[uuid.UUID, None]:
-
         resolved = alias_path.resolve()
 
         assert resolved.name.endswith(".value")
@@ -129,15 +155,9 @@ class FileSystemAliasArchive(AliasArchive):
 
 
 class FileSystemAliasStore(FileSystemAliasArchive, AliasStore):
-
     _archive_type_name = "filesystem_alias_store"
 
-    @classmethod
-    def is_writeable(cls) -> bool:
-        return True
-
     def register_aliases(self, value_id: uuid.UUID, *aliases: str):
-
         value_path = self._translate_value_id(value_id=value_id)
         value_path.parent.mkdir(parents=True, exist_ok=True)
         value_path.touch()
