@@ -7,14 +7,18 @@
 import abc
 import copy
 import uuid
+from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
     Dict,
+    Generator,
+    Iterable,
     List,
     Mapping,
     Protocol,
+    Sequence,
     Set,
     Tuple,
     Union,
@@ -22,10 +26,15 @@ from typing import (
 
 import structlog
 from rich.console import RenderableType
+from sqlalchemy.exc import OperationalError
 
 from kiara.data_types import DataType
 from kiara.data_types.included_core_types import NoneType
 from kiara.defaults import (
+    DATA_ARCHIVE_DEFAULT_VALUE_MARKER,
+    DEFAULT_DATA_STORE_MARKER,
+    DEFAULT_STORE_MARKER,
+    ENVIRONMENT_MARKER_KEY,
     INVALID_HASH_MARKER,
     NO_SERIALIZATION_MARKER,
     NONE_STORE_ID,
@@ -37,6 +46,7 @@ from kiara.defaults import (
 )
 from kiara.exceptions import (
     InvalidValuesException,
+    KiaraException,
     NoSuchValueAliasException,
     NoSuchValueException,
     NoSuchValueIdException,
@@ -70,19 +80,24 @@ from kiara.registries.ids import ID_REGISTRY
 from kiara.utils import log_exception, log_message
 from kiara.utils.data import pretty_print_data
 from kiara.utils.hashing import NONE_CID
+from kiara.utils.stores import check_external_archive
 
 if TYPE_CHECKING:
+    from multiformats.varint import BytesLike
+
     from kiara.context import Kiara
+    from kiara.models.module.destiny import Destiny
+    from kiara.models.module.manifest import Manifest
+
 
 logger = structlog.getLogger()
 
 
 class ValueLink(Protocol):
-
     value_id: uuid.UUID
 
 
-NONE_PERSISTED_DATA = PersistedData.construct(
+NONE_PERSISTED_DATA = PersistedData(
     data_type="none",
     data_type_config={},
     serialization_profile="none",
@@ -95,7 +110,6 @@ NONE_PERSISTED_DATA = PersistedData.construct(
 
 class AliasResolver(abc.ABC):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: "Kiara" = kiara
 
     @abc.abstractmethod
@@ -103,12 +117,17 @@ class AliasResolver(abc.ABC):
         pass
 
 
+ARCHIVE_REF_TYPE_NAME = "archive"
+
+
 class DefaultAliasResolver(AliasResolver):
     def __init__(self, kiara: "Kiara"):
-
         super().__init__(kiara=kiara)
 
     def resolve_alias(self, alias: str) -> uuid.UUID:
+        # preprocessing alias
+        if alias.endswith(".kiarchive"):
+            alias = f"archive:{alias}"
 
         if ":" in alias:
             ref_type, rest = alias.split(":", maxsplit=1)
@@ -124,6 +143,52 @@ class DefaultAliasResolver(AliasResolver):
                         alias=rest,
                         msg=f"Can't retrive value for alias '{rest}': no such alias registered.",
                     )
+            elif ref_type == ARCHIVE_REF_TYPE_NAME:
+                if "#" in rest:
+                    archive_ref, path_in_archive = rest.split("#", maxsplit=1)
+                else:
+                    archive_ref = rest
+                    path_in_archive = None
+
+                archives = check_external_archive(
+                    archive=archive_ref, allow_write_access=False
+                )
+
+                if archives:
+                    data_archive: DataArchive = archives.get("data", None)  # type: ignore
+                    if data_archive:
+                        self._kiara.data_registry.register_data_archive(data_archive)
+
+                        if not path_in_archive:
+                            default_value = data_archive.get_archive_metadata(
+                                DATA_ARCHIVE_DEFAULT_VALUE_MARKER
+                            )
+                            if default_value is None:
+                                raise NoSuchValueException(
+                                    f"No default value found for uri: {alias}"
+                                )
+                            _value_id = uuid.UUID(default_value)
+                        else:
+                            from kiara.registries.aliases import AliasArchive
+
+                            alias_archive: AliasArchive = archives.get("alias", None)  # type: ignore
+                            if alias_archive:
+                                _value_id = alias_archive.find_value_id_for_alias(
+                                    alias=path_in_archive
+                                )
+                            else:
+                                raise NoSuchValueException(
+                                    msg=f"No alias archive found for '{archive_ref}'."
+                                )
+                    else:
+                        raise NoSuchValueException(
+                            "No data archive found in '{archive_ref}'."
+                        )
+                else:
+                    raise NoSuchValueException(
+                        msg=f"No archive found for '{archive_ref}'."
+                    )
+
             else:
                 raise Exception(
                     f"Can't retrieve value for '{alias}': invalid reference type '{ref_type}'."
@@ -144,7 +209,6 @@ class DefaultAliasResolver(AliasResolver):
 
 class DataRegistry(object):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: Kiara = kiara
 
         self._event_callback: Callable = self._kiara.event_registry.add_producer(self)
@@ -166,7 +230,7 @@ class DataRegistry(object):
 
         # initialize special values
         special_value_cls = PythonClass.from_class(NoneType)
-        data_type_info = DataTypeInfo.construct(
+        data_type_info = DataTypeInfo(
             data_type_name="none",
             characteristics=DEFAULT_SCALAR_DATATYPE_CHARACTERISTICS,
             data_type_class=special_value_cls,
@@ -192,6 +256,7 @@ class DataRegistry(object):
         self._cached_data[NOT_SET_VALUE_ID] = SpecialValue.NOT_SET
         self._registered_values[NOT_SET_VALUE_ID] = self._not_set_value
         self._persisted_value_descs[NOT_SET_VALUE_ID] = NONE_PERSISTED_DATA
+        # self._env_cache: Dict[str, Dict[str, RuntimeEnvironment]] = {}
 
         self._none_value: Value = Value(
             value_id=NONE_VALUE_ID,
@@ -215,6 +280,13 @@ class DataRegistry(object):
         self._registered_values[NONE_VALUE_ID] = self._none_value
         self._persisted_value_descs[NONE_VALUE_ID] = NONE_PERSISTED_DATA
 
+        self._cached_value_aliases: Dict[
+            uuid.UUID, Dict[str, Union[Destiny, None]]
+        ] = {}
+
+        self._destinies: Dict[uuid.UUID, Destiny] = {}
+        self._destinies_by_value: Dict[uuid.UUID, Dict[str, Destiny]] = {}
+
     @property
     def kiara_id(self) -> uuid.UUID:
         return self._kiara.id
@@ -228,9 +300,8 @@ class DataRegistry(object):
         return self._none_value
 
     def retrieve_all_available_value_ids(self) -> Set[uuid.UUID]:
-
         result: Set[uuid.UUID] = set()
-        for store in self._data_archives.values():
+        for alias, store in self._data_archives.items():
             ids = store.value_ids
             if ids:
                 result.update(ids)
@@ -240,19 +311,20 @@ class DataRegistry(object):
     def register_data_archive(
         self,
         archive: DataArchive,
-        alias: Union[str, None] = None,
         set_as_default_store: Union[bool, None] = None,
-    ):
+    ) -> str:
+        alias = archive.archive_name
 
-        data_store_id = archive.archive_id
-        archive.register_archive(kiara=self._kiara)
-        if alias is None:
-            alias = str(data_store_id)
+        if not alias:
+            raise Exception("Invalid data archive alias: can't be empty.")
 
         if alias in self._data_archives.keys():
             raise Exception(
                 f"Can't add data archive, alias '{alias}' already registered."
             )
+
+        archive.register_archive(kiara=self._kiara)
+
         self._data_archives[alias] = archive
         is_store = False
         is_default_store = False
@@ -268,7 +340,7 @@ class DataRegistry(object):
                 is_default_store = True
                 self._default_data_store = alias
 
-        event = DataArchiveAddedEvent.construct(
+        event = DataArchiveAddedEvent(
             kiara_id=self._kiara.id,
             data_archive_id=archive.archive_id,
             data_archive_alias=alias,
@@ -277,6 +349,8 @@ class DataRegistry(object):
         )
         self._event_callback(event)
 
+        return alias
+
     @property
     def default_data_store(self) -> str:
         if self._default_data_store is None:
@@ -288,43 +362,45 @@ class DataRegistry(object):
         return self._data_archives
 
     def get_archive(
-        self, archive_id: Union[None, uuid.UUID, str] = None
+        self, archive_id_or_alias: Union[None, uuid.UUID, str] = None
     ) -> DataArchive:
-
-        if archive_id is None:
-            archive_id = self.default_data_store
-            if archive_id is None:
+        if archive_id_or_alias in (
+            None,
+            DEFAULT_STORE_MARKER,
+            DEFAULT_DATA_STORE_MARKER,
+        ):
+            archive_id_or_alias = self.default_data_store
+            if archive_id_or_alias is None:
                 raise Exception("Can't retrieve default data archive, none set (yet).")
 
-        if isinstance(archive_id, uuid.UUID):
+        if isinstance(archive_id_or_alias, uuid.UUID):
             for archive in self._data_archives.values():
-                if archive.archive_id == archive_id:
+                if archive.archive_id == archive_id_or_alias:
                     return archive
 
             raise Exception(
-                f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+                f"Can't retrieve archive with id '{archive_id_or_alias}': no archive with that id registered."
             )
 
-        if archive_id in self._data_archives.keys():
-            return self._data_archives[archive_id]
+        if archive_id_or_alias in self._data_archives.keys():
+            return self._data_archives[archive_id_or_alias]
         else:
             try:
-                _archive_id = uuid.UUID(archive_id)
+                _archive_id = uuid.UUID(archive_id_or_alias)
                 for archive in self._data_archives.values():
                     if archive.archive_id == _archive_id:
                         return archive
                     raise Exception(
-                        f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+                        f"Can't retrieve archive with id '{archive_id_or_alias}': no archive with that id registered."
                     )
             except Exception:
                 pass
 
         raise Exception(
-            f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+            f"Can't retrieve archive with id '{archive_id_or_alias}': no archive with that id registered."
         )
 
     def find_store_id_for_value(self, value_id: uuid.UUID) -> Union[str, None]:
-
         if value_id in self._value_archive_lookup_map.keys():
             return self._value_archive_lookup_map[value_id]
 
@@ -344,7 +420,7 @@ class DataRegistry(object):
         self._value_archive_lookup_map[value_id] = matches[0]
         return matches[0]
 
-    def get_value(self, value: Union[uuid.UUID, ValueLink, str]) -> Value:
+    def get_value(self, value: Union[uuid.UUID, ValueLink, str, Path]) -> Value:
         _value_id = None
 
         if not isinstance(value, uuid.UUID):
@@ -354,7 +430,6 @@ class DataRegistry(object):
                 if isinstance(_value_id, str):
                     _value_id = uuid.UUID(_value_id)
             else:
-
                 try:
                     _value_id = uuid.UUID(
                         value  # type: ignore
@@ -363,13 +438,17 @@ class DataRegistry(object):
                     _value_id = None
 
                 if _value_id is None:
-
+                    if isinstance(value, Path):
+                        raise NotImplementedError()
                     if not isinstance(value, str):
                         raise Exception(
                             f"Can't retrieve value for '{value}': invalid type '{type(value)}'."
                         )
-
-                    _value_id = self._alias_resolver.resolve_alias(value)
+                    try:
+                        _value_id = self._alias_resolver.resolve_alias(value)
+                    except Exception as e:
+                        log_exception(e)
+                        raise e
         else:
             _value_id = value
 
@@ -379,78 +458,136 @@ class DataRegistry(object):
             _value = self._registered_values[_value_id]
             return _value
 
-        matches = []
-        for store_id, store in self.data_archives.items():
-            match = store.has_value(value_id=_value_id)
-            if match:
-                matches.append(store_id)
+        default_store: DataArchive = self.get_archive(
+            archive_id_or_alias=self.default_data_store
+        )
+        if not default_store.has_value(value_id=_value_id):
+            matches = []
+            for store_id, store in self.data_archives.items():
+                match = store.has_value(value_id=_value_id)
+                if match:
+                    matches.append(store_id)
 
-        if len(matches) == 0:
-            raise NoSuchValueIdException(
-                value_id=_value_id, msg=f"No value registered with id: {value}"
-            )
-        elif len(matches) > 1:
-            raise NoSuchValueIdException(
-                value_id=_value_id,
-                msg=f"Found value with id '{value}' in multiple archives, this is not supported (yet): {matches}",
-            )
+            if len(matches) == 0:
+                raise NoSuchValueIdException(
+                    value_id=_value_id, msg=f"No value registered with id: {value}"
+                )
+            elif len(matches) > 1:
+                raise NoSuchValueIdException(
+                    value_id=_value_id,
+                    msg=f"Found value with id '{value}' in multiple archives, this is not supported (yet): {matches}",
+                )
+            store_that_has_it = matches[0]
+        else:
+            store_that_has_it = self.default_data_store
 
-        self._value_archive_lookup_map[_value_id] = matches[0]
-        stored_value = self.get_archive(matches[0]).retrieve_value(value_id=_value_id)
+        self._value_archive_lookup_map[_value_id] = store_that_has_it
+
+        stored_value = self.get_archive(store_that_has_it).retrieve_value(
+            value_id=_value_id
+        )
         stored_value._set_registry(self)
         stored_value._is_stored = True
 
         self._registered_values[_value_id] = stored_value
         return self._registered_values[_value_id]
 
+    def _persist_environment(self, env_hash: str, store: Union[str, None]):
+        # cached = self._env_cache.get(env_type, {}).get(env_hash, None)
+        # if cached is not None:
+        #     return
+
+        environment = self._kiara.metadata_registry.retrieve_environment_item(env_hash)
+
+        if not environment:
+            raise KiaraException(
+                f"Can't persist data environment with hash '{env_hash}': no such environment registered."
+            )
+
+        self._kiara.metadata_registry.register_metadata_item(
+            key=ENVIRONMENT_MARKER_KEY, item=environment, store=store
+        )
+        # self._env_cache.setdefault(env_type, {})[env_hash] = environment
+
     def store_value(
         self,
         value: Union[ValueLink, uuid.UUID, str],
-        store_id: Union[str, None] = None,
+        data_store: Union[str, None] = None,
     ) -> Union[PersistedData, None]:
+        """Store a value into a data store.
 
-        if store_id is None:
-            store_id = self.default_data_store
+        If 'data_store' is not provided, the default data store is used. If the 'data_store' argument is of
+        type uuid, the archive_id is used, if string, first it will be converted to an uuid, if that works,
+        again, the archive_id is used, if not, the string is used as the archive alias.
 
-        _value = self.get_value(value)
+        """
 
-        store: DataStore = self.get_archive(archive_id=store_id)  # type: ignore
-        if not isinstance(store, DataStore):
-            raise Exception(f"Can't store value into store '{store_id}': not writable.")
+        try:
+            _value = self.get_value(value)
 
-        # make sure all property values are available
-        if _value.pedigree != ORPHAN:
-            for value_id in _value.pedigree.inputs.values():
-                self.store_value(value=value_id, store_id=store_id)
+            # first, persist environment information
+            for env_hash in _value.pedigree.environments.values():
+                self._persist_environment(env_hash, store=data_store)
 
-        if not store.has_value(_value.value_id):
-            event = ValuePreStoreEvent.construct(kiara_id=self._kiara.id, value=_value)
-            self._event_callback(event)
-            persisted_value = store.store_value(_value)
-            _value._is_stored = True
-            self._value_archive_lookup_map[_value.value_id] = store_id
-            self._persisted_value_descs[_value.value_id] = persisted_value
-            property_values = _value.property_values
+            store: DataStore = self.get_archive(archive_id_or_alias=data_store)  # type: ignore
+            if not store.is_writeable():
+                if data_store:
+                    raise Exception(
+                        f"Can't write value into store '{data_store}': not writable."
+                    )
+                else:
+                    raise Exception("Can't write value into store: not writable.")
 
-            for property, property_value in property_values.items():
-                self.store_value(value=property_value, store_id=store_id)
-        else:
-            persisted_value = None
+            _data_store = store.archive_name
+            # make sure all property values are available
+            if _value.pedigree != ORPHAN:
+                for value_id in _value.pedigree.inputs.values():
+                    self.store_value(value=value_id, data_store=_data_store)
 
-        store_event = ValueStoredEvent.construct(kiara_id=self._kiara.id, value=_value)
-        self._event_callback(store_event)
+            if not store.has_value(_value.value_id):
+                event = ValuePreStoreEvent(kiara_id=self._kiara.id, value=_value)
+                self._event_callback(event)
+                persisted_value = store.store_value(_value)
+                _value._is_stored = True
 
-        return persisted_value
+                self._value_archive_lookup_map[_value.value_id] = _data_store
+                self._persisted_value_descs[_value.value_id] = persisted_value
+                property_values = _value.property_values
+
+                for property, property_value in property_values.items():
+                    self.store_value(value=property_value, data_store=_data_store)
+
+                store_required = True
+            else:
+                persisted_value = None
+                store_required = False
+
+            store_event = ValueStoredEvent(
+                kiara_id=self._kiara.id, value=_value, storing_required=store_required
+            )
+            self._event_callback(store_event)
+
+            if _value.job_id:
+                self._kiara.job_registry.store_job_record(
+                    job_id=_value.job_id, store=data_store
+                )
+
+            return persisted_value
+        except OperationalError as oe:
+            if "has no column named" in str(oe):
+                raise KiaraException(
+                    "Your kiara data store is using an old version of the table schema, this is not supported. Either downgrade kiara to the version that was used to create the data store, or delete your current context (`kiara context delete')."
+                )
+            else:
+                raise oe
 
     def lookup_aliases(self, value: Union[Value, uuid.UUID]) -> Set[str]:
-
         if isinstance(value, Value):
             value = value.value_id
 
         return self._kiara.alias_registry.find_aliases_for_value_id(value_id=value)
 
     def create_value_info(self, value: Union[Value, uuid.UUID]) -> ValueInfo:
-
         if isinstance(value, uuid.UUID):
             value = self.get_value(value=value)
 
@@ -458,9 +595,11 @@ class DataRegistry(object):
         return value_info
 
     def find_values(self, matcher: ValueMatcher) -> Dict[uuid.UUID, Value]:
-
         matches: Dict[uuid.UUID, Value] = {}
         for store_id, store in self.data_archives.items():
+            if matcher.in_data_archives and store_id not in matcher.in_data_archives:
+                continue
+
             try:
                 _matches = store.find_values(matcher=matcher)
                 for value in _matches:
@@ -477,6 +616,13 @@ class DataRegistry(object):
                         value.value_id
                     )
             except NotImplementedError:
+                log_message(
+                    "store.feature.missing",
+                    feature="find_value",
+                    reasong=f"find_values not implemented for store: {store_id}",
+                    solution="return all values",
+                )
+
                 all_value_ids = store.value_ids
                 if all_value_ids is None:
                     continue
@@ -498,8 +644,7 @@ class DataRegistry(object):
         return matches
 
     def find_values_with_aliases(self, matcher: ValueMatcher) -> Dict[str, Value]:
-
-        matcher = matcher.copy(update={"has_aliases": True})
+        matcher = matcher.model_copy(update={"has_aliases": True})
         all_values = self.find_values(matcher)
         result = {}
         for value in all_values.values():
@@ -515,7 +660,6 @@ class DataRegistry(object):
     def find_values_for_hash(
         self, value_hash: str, data_type_name: Union[str, None] = None
     ) -> Set[Value]:
-
         if data_type_name:
             raise NotImplementedError()
 
@@ -543,19 +687,21 @@ class DataRegistry(object):
 
         return {self.get_value(value=v_id) for v_id in stored}
 
-    def find_destinies_for_value(
+    # ==============================================================================================
+    # destiny stuff
+
+    def retrieve_destinies_for_value_from_archives(
         self, value_id: uuid.UUID, alias_filter: Union[str, None] = None
     ) -> Mapping[str, uuid.UUID]:
-
         if alias_filter:
             raise NotImplementedError()
 
         all_destinies: Dict[str, uuid.UUID] = {}
         for archive_id, archive in self._data_archives.items():
-            destinies: Union[
-                Mapping[str, uuid.UUID], None
-            ] = archive.find_destinies_for_value(
-                value_id=value_id, alias_filter=alias_filter
+            destinies: Union[Mapping[str, uuid.UUID], None] = (
+                archive.find_destinies_for_value(
+                    value_id=value_id, alias_filter=alias_filter
+                )
             )
             if not destinies:
                 continue
@@ -566,15 +712,121 @@ class DataRegistry(object):
 
         return all_destinies
 
+    def get_destiny_aliases_for_value(
+        self, value_id: uuid.UUID, alias_filter: Union[str, None] = None
+    ) -> Iterable[str]:
+        # TODO: cache the result of this
+
+        if alias_filter is not None:
+            raise NotImplementedError()
+
+        aliases: Set[str] = set()
+        aliases.update(
+            self.retrieve_destinies_for_value_from_archives(value_id=value_id).keys()
+        )
+
+        # all_stores = self._all_values_store_map.get(value_id)
+        # if all_stores:
+        #     for prefix in all_stores:
+        #         all_aliases = self._destiny_archives[
+        #             prefix
+        #         ].get_destiny_aliases_for_value(value_id=value_id)
+        #         if all_aliases is not None:
+        #             aliases.update((f"{prefix}.{a}" for a in all_aliases))
+
+        current = self._destinies_by_value.get(value_id, None)
+        if current:
+            aliases.update(current.keys())
+
+        return sorted(aliases)
+
+    def register_destiny(
+        self,
+        destiny_alias: str,
+        values: Dict[str, uuid.UUID],
+        manifest: "Manifest",
+        result_field_name: Union[str, None] = None,
+    ) -> "Destiny":
+        """
+        Add a destiny for one (or in some rare cases several) values.
+
+        A destiny alias must be unique for every one of the involved input values.
+        """
+        if not values:
+            raise Exception("Can't add destiny, no values provided.")
+
+        from kiara.models.module.destiny import Destiny
+
+        destiny = Destiny.create_from_values(
+            kiara=self._kiara,
+            destiny_alias=destiny_alias,
+            manifest=manifest,
+            result_field_name=result_field_name,
+            values=values,
+        )
+
+        for value_id in destiny.fixed_inputs.values():
+            self._destinies[destiny.destiny_id] = destiny
+            # TODO: store history?
+            self._destinies_by_value.setdefault(value_id, {})[destiny_alias] = destiny
+            self._cached_value_aliases.setdefault(value_id, {})[destiny_alias] = destiny
+
+        return destiny
+
+    def attach_destiny_as_property(
+        self,
+        destiny: Union[uuid.UUID, "Destiny"],
+        field_names: Union[Iterable[str], None] = None,
+    ):
+        if field_names:
+            raise NotImplementedError()
+
+        if isinstance(destiny, uuid.UUID):
+            destiny = self._destinies[destiny]
+
+        values = self.load_values(destiny.fixed_inputs)
+
+        already_stored: List[uuid.UUID] = []
+        for v in values.values():
+            if v.is_stored:
+                already_stored.append(v.value_id)
+
+        if already_stored:
+            stored = (str(v) for v in already_stored)
+            raise Exception(
+                f"Can't attach destiny as property, value(s) already stored: {', '.join(stored)}"
+            )
+
+        if destiny.result_value_id is None:
+            destiny.execute(kiara=self._kiara)
+
+        for v in values.values():
+            assert destiny.result_value_id is not None
+            v.add_property(
+                value_id=destiny.result_value_id,
+                property_path=destiny.destiny_alias,
+                add_origin_to_property_value=True,
+            )
+
+    def get_registered_destiny(
+        self, value_id: uuid.UUID, destiny_alias: str
+    ) -> "Destiny":
+        destiny = self._destinies_by_value.get(value_id, {}).get(destiny_alias, None)
+        if destiny is None:
+            raise Exception(
+                f"No destiny '{destiny_alias}' available for value '{value_id}'."
+            )
+
+        return destiny
+
     def register_data(
         self,
         data: Any,
-        schema: Union[ValueSchema, str, None] = None,
+        schema: Union[ValueSchema, str, None, Mapping[str, Any]] = None,
         pedigree: Union[ValuePedigree, None] = None,
         pedigree_output_name: Union[str, None] = None,
         reuse_existing: bool = True,
     ) -> Value:
-
         value, newly_created = self._create_value(
             data=data,
             schema=schema,
@@ -604,14 +856,11 @@ class DataRegistry(object):
         str,
         int,
     ]:
-
         if schema is None:
             raise NotImplementedError()
 
         if isinstance(data, Value):
-
             if data.value_id in self._registered_values.keys():
-
                 if data.is_set and data.is_serializable:
                     serialized: Union[str, SerializedData] = data.serialized_data
                 else:
@@ -701,18 +950,21 @@ class DataRegistry(object):
     def _create_value(
         self,
         data: Any,
-        schema: Union[None, str, ValueSchema] = None,
+        schema: Union[None, str, ValueSchema, Mapping[str, Any]] = None,
         pedigree: Union[ValuePedigree, None] = None,
         pedigree_output_name: Union[str, None] = None,
         reuse_existing: bool = True,
     ) -> Tuple[Value, bool]:
-        """Create a new value, or return an existing one that matches the incoming data or reference.
+        """
+        Create a new value, or return an existing one that matches the incoming data or reference.
 
         Arguments:
+        ---------
             data: the (raw) data, or a reference to an existing value
 
 
         Returns:
+        -------
             a tuple containing of the value object, and a boolean indicating whether the value was newly created (True), or already existing (False)
         """
 
@@ -720,6 +972,12 @@ class DataRegistry(object):
             raise NotImplementedError()
         elif isinstance(schema, str):
             schema = ValueSchema(type=schema)
+        elif isinstance(schema, Mapping):
+            schema = ValueSchema(**schema)
+        elif not isinstance(schema, ValueSchema):
+            raise Exception(
+                f"Invalid schema type: {type(schema)}, expected: {ValueSchema}"
+            )
 
         if schema.type not in self._kiara.data_type_names:
             raise Exception(
@@ -733,13 +991,25 @@ class DataRegistry(object):
             else:
                 data = copy.deepcopy(schema.default)
 
+            reuse_existing = False
+
+        data_type: Union[None, DataType] = None
+        if reuse_existing and not isinstance(data, (Value, uuid.UUID, SpecialValue)):
+            data_type = self._kiara.type_registry.retrieve_data_type(
+                data_type_name=schema.type, data_type_config=schema.type_config
+            )
+            if data_type.characteristics.is_scalar:
+                reuse_existing = False
+
         if data is None:
             data = SpecialValue.NO_VALUE
+        elif isinstance(data, uuid.UUID):
+            if data == NONE_VALUE_ID:
+                data = SpecialValue.NO_VALUE
+            elif data == NOT_SET_VALUE_ID:
+                data = SpecialValue.NOT_SET
 
-        if reuse_existing and data not in [
-            SpecialValue.NO_VALUE,
-            SpecialValue.NOT_SET,
-        ]:
+        if reuse_existing and data not in [SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
             (
                 _existing,
                 data_type,
@@ -754,9 +1024,10 @@ class DataRegistry(object):
                 # TODO: check pedigree
                 return (_existing, False)
         else:
-            data_type = self._kiara.type_registry.retrieve_data_type(
-                data_type_name=schema.type, data_type_config=schema.type_config
-            )
+            if data_type is None:
+                data_type = self._kiara.type_registry.retrieve_data_type(
+                    data_type_name=schema.type, data_type_config=schema.type_config
+                )
 
             (
                 data,
@@ -775,6 +1046,9 @@ class DataRegistry(object):
             else:
                 raise NotImplementedError()
 
+        if not pedigree.is_resolved:
+            pedigree = self._kiara.module_registry.resolve_manifest(pedigree)  # type: ignore
+
         v_id = ID_REGISTRY.generate(
             type="value", kiara_id=self._kiara.id, pre_registered=False
         )
@@ -802,7 +1076,6 @@ class DataRegistry(object):
         return (value, True)
 
     def retrieve_persisted_value_details(self, value_id: uuid.UUID) -> PersistedData:
-
         if (
             value_id in self._persisted_value_descs.keys()
             and self._persisted_value_descs[value_id] is not None
@@ -838,33 +1111,60 @@ class DataRegistry(object):
         self, value_id: uuid.UUID
     ) -> Union[SerializedData, None]:
         """Create a LoadConfig object from the details of the persisted version of this value."""
-
         pv = self.retrieve_persisted_value_details(value_id=value_id)
         if pv is None:
             return None
 
         return pv
 
-    def retrieve_chunk(
+    # def retrieve_chunk(
+    #     self,
+    #     chunk_id: str,
+    #     archive_id: Union[uuid.UUID, None] = None,
+    #     as_file: bool = True,
+    #     symlink_ok: bool = True,
+    # ) -> Union[str, "BytesLike"]:
+    #
+    #     if archive_id is None:
+    #         raise NotImplementedError()
+    #
+    #     archive = self.get_archive(archive_id)
+    #     chunk = archive.retrieve_chunk(chunk_id, as_file=as_file, symlink_ok=symlink_ok)
+    #
+    #     return chunk
+
+    def retrieve_chunks(
         self,
-        chunk_id: str,
-        archive_id: Union[uuid.UUID, None] = None,
-        as_file: Union[None, bool, str] = None,
+        chunk_ids: Sequence[str],
+        as_files: bool = True,
         symlink_ok: bool = True,
-    ) -> Union[str, bytes]:
+        archive_id: Union[uuid.UUID, None] = None,
+    ) -> Generator[Union[str, "BytesLike"], None, None]:
+        """Return the chunk content in the same order as the 'chunk_ids' argument.
+
+        If 'as_files' is 'True', it will return strings representing paths to files containing the chunk data. If symlink_ok is also set to 'True', the returning Path could potentially be a symlink, which means the underlying function might not need to copy the file. In this case, you are responsible to not change the contents of the path, ever.
+
+        If 'as_files' is 'False', BytesLike objects will be returned, containing the chunk data bytes directly.
+        """
 
         if archive_id is None:
-            raise NotImplementedError()
+            raise NotImplementedError(
+                "Can't retrieve chunks without specifying an archive."
+            )
 
         archive = self.get_archive(archive_id)
-        chunk = archive.retrieve_chunk(chunk_id, as_file=as_file, symlink_ok=symlink_ok)
 
-        return chunk
+        chunks = archive.retrieve_chunks(
+            chunk_ids, as_files=as_files, symlink_ok=symlink_ok
+        )
+
+        return chunks
+        # for chunk_id in chunk_ids:
+        #     yield archive.retrieve_chunk(chunk_id)
 
     def retrieve_value_data(
         self, value: Union[uuid.UUID, Value], target_profile: Union[str, None] = None
     ) -> Any:
-
         if isinstance(value, uuid.UUID):
             value = self.get_value(value=value)
 
@@ -872,9 +1172,9 @@ class DataRegistry(object):
             return self._cached_data[value.value_id]
 
         if value._serialized_data is None:
-            serialized_data: Union[
-                str, SerializedData
-            ] = self.retrieve_persisted_value_details(value_id=value.value_id)
+            serialized_data: Union[str, SerializedData] = (
+                self.retrieve_persisted_value_details(value_id=value.value_id)
+            )
             value._serialized_data = serialized_data
         else:
             serialized_data = value._serialized_data
@@ -949,11 +1249,9 @@ class DataRegistry(object):
         self,
         values: Mapping[str, Union[uuid.UUID, None, str, ValueLink]],
         values_schema: Union[None, Mapping[str, ValueSchema]] = None,
-    ) -> ValueMap:
-
+    ) -> ValueMapReadOnly:
         value_items = {}
         if values_schema:
-
             schemas = values_schema
             for k in schemas.keys():
                 if k not in values.keys():
@@ -979,7 +1277,6 @@ class DataRegistry(object):
     def load_data(
         self, values: Mapping[str, Union[uuid.UUID, None]]
     ) -> Mapping[str, Any]:
-
         result_values = self.load_values(values=values)
         return {k: v.data for k, v in result_values.items()}
 
@@ -987,8 +1284,8 @@ class DataRegistry(object):
         self, data: Mapping[str, Any], schema: Mapping[str, ValueSchema]
     ) -> ValueMap:
         """Extract a set of [Value][kiara.data.values.Value] from Python data and ValueSchemas."""
-
         input_details = {}
+
         for input_name, value_schema in schema.items():
             input_details[input_name] = {"schema": value_schema}
 
@@ -1013,7 +1310,15 @@ class DataRegistry(object):
                 try:
                     _d = uuid.UUID(_d)
                 except Exception:
-                    pass
+                    if schema[input_name].type == "string" and not (
+                        _d.startswith("alias:") or _d.startswith("value:")
+                    ):
+                        pass
+                    else:
+                        try:
+                            _d = self._alias_resolver.resolve_alias(_d)
+                        except Exception as e:
+                            log_exception(e)
 
             if isinstance(_d, Value):
                 _resolved[input_name] = _d
@@ -1027,7 +1332,6 @@ class DataRegistry(object):
             values[input_name] = _value
 
         for input_name, _data in _unresolved.items():
-
             value_schema = input_details[input_name]["schema"]
 
             if input_name not in data.keys():
@@ -1048,7 +1352,6 @@ class DataRegistry(object):
                 values[input_name] = value
 
             except Exception as e:
-
                 log_exception(e)
 
                 msg: Any = str(e)
@@ -1065,15 +1368,16 @@ class DataRegistry(object):
                 if not str(v):
                     _v = type(v).__name__
                 msg.append(f"{k}: {_v}")
+
             raise InvalidValuesException(
                 msg=f"Can't create values instance: {', '.join(msg)}",
                 invalid_values={k: str(v) for k, v in failed.items()},
             )
+
         return ValueMapReadOnly(value_items=values, values_schema=schema)  # type: ignore
 
     def create_renderable(self, **config: Any) -> RenderableType:
         """Create a renderable for this module configuration."""
-
         from kiara.utils.output import create_renderable_from_values
 
         all_values = {str(i): v for i, v in self._registered_values.items()}
@@ -1087,7 +1391,6 @@ class DataRegistry(object):
         target_type="terminal_renderable",
         **render_config: Any,
     ) -> Any:
-
         assert isinstance(value_id, uuid.UUID)
 
         return pretty_print_data(
