@@ -9,29 +9,37 @@ import uuid
 from enum import Enum
 from io import BytesIO
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, Mapping, Set, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Generator,
+    Generic,
+    Iterable,
+    Mapping,
+    Sequence,
+    Set,
+    Union,
+)
 
 import orjson
 import structlog
 
+from kiara.exceptions import KiaraException
 from kiara.models.module.jobs import JobRecord
 from kiara.models.values.value import (
-    SERIALIZE_TYPES,
     PersistedData,
-    SerializedChunkIDs,
-    SerializedData,
     Value,
 )
-from kiara.registries import ArchiveDetails, FileSystemArchiveConfig
+from kiara.registries import ARCHIVE_CONFIG_CLS, ArchiveDetails, FileSystemArchiveConfig
 from kiara.registries.data.data_store import BaseDataStore, DataArchive
-from kiara.registries.jobs import JobArchive
 from kiara.utils import log_message
 from kiara.utils.hashfs import HashAddress, HashFS
 from kiara.utils.json import orjson_dumps
 from kiara.utils.windows import fix_windows_longpath, fix_windows_symlink
 
 if TYPE_CHECKING:
-    pass
+    from multiformats import CID
+    from multiformats.varint import BytesLike
 
 logger = structlog.getLogger()
 
@@ -39,7 +47,6 @@ VALUE_DETAILS_FILE_NAME = "value.json"
 
 
 class EntityType(Enum):
-
     VALUE = "values"
     VALUE_DATA = "value_data"
     ENVIRONMENT = "environments"
@@ -52,41 +59,75 @@ DEFAULT_HASHFS_WIDTH = 1
 DEFAULT_HASH_FS_ALGORITHM = "sha256"
 
 
-class FileSystemDataArchive(DataArchive, JobArchive):
+class FileSystemDataArchive(
+    DataArchive[FileSystemArchiveConfig], Generic[ARCHIVE_CONFIG_CLS]
+):
     """Data store that loads data from the local filesystem."""
 
     _archive_type_name = "filesystem_data_archive"
     _config_cls = FileSystemArchiveConfig  # type: ignore
 
-    # @classmethod
-    # def supported_item_types(cls) -> Iterable[str]:
-    #
-    #     return ["data", "job_record"]
-
-    @classmethod
-    def is_writeable(cls) -> bool:
-        return False
-
-    def __init__(self, archive_id: uuid.UUID, config: FileSystemArchiveConfig):
-
-        DataArchive.__init__(self, archive_id=archive_id, config=config)
+    def __init__(
+        self,
+        archive_name: str,
+        archive_config: FileSystemArchiveConfig,
+        force_read_only: bool = False,
+    ):
+        super().__init__(
+            archive_name=archive_name,
+            archive_config=archive_config,
+            force_read_only=force_read_only,
+        )
         self._base_path: Union[Path, None] = None
         self._hashfs_path: Union[Path, None] = None
         self._hashfs: Union[HashFS, None] = None
+        # self._archive_metadata: Union[Mapping[str, Any], None] = None
 
-    # def get_job_archive_id(self) -> uuid.UUID:
-    #     return self._kiara.id
+    def _retrieve_archive_metadata(self) -> Mapping[str, Any]:
+        if not self.archive_metadata_path.is_file():
+            _archive_metadata = {}
+        else:
+            _archive_metadata = orjson.loads(self.archive_metadata_path.read_bytes())
+
+        archive_id = _archive_metadata.get("archive_id", None)
+        if not archive_id:
+            try:
+                _archive_id = uuid.UUID(
+                    self.data_store_path.name
+                )  # just to test it's a valid uuid
+                _archive_metadata["archive_id"] = str(_archive_id)
+            except Exception:
+                raise Exception(
+                    f"Could not retrieve archive id for alias archive '{self.archive_name}'."
+                )
+
+        return _archive_metadata
+
+    @property
+    def archive_metadata_path(self) -> Path:
+        return self.data_store_path / "store_metadata.json"
 
     def get_archive_details(self) -> ArchiveDetails:
-
         size = sum(
             f.stat().st_size for f in self.data_store_path.glob("**/*") if f.is_file()
         )
-        return ArchiveDetails(size=size)
+        all_values = self.value_ids
+
+        if all_values is not None:
+            _all_values = list(all_values)
+            details = {
+                "size": size,
+                "no_values": len(_all_values),
+                "value_ids": sorted((str(x) for x in _all_values)),
+                "dynamic_archive": False,
+            }
+        else:
+            # will probably never happen
+            details = {"size": size, "dynamic_archive": True}
+        return ArchiveDetails(root=details)
 
     @property
     def data_store_path(self) -> Path:
-
         if self._base_path is not None:
             return self._base_path
 
@@ -100,14 +141,12 @@ class FileSystemDataArchive(DataArchive, JobArchive):
 
     @property
     def hash_fs_path(self) -> Path:
-
         if self._hashfs_path is None:
             self._hashfs_path = self.data_store_path / "hash_fs"
         return self._hashfs_path
 
     @property
     def hashfs(self) -> HashFS:
-
         if self._hashfs is None:
             self._hashfs = HashFS(
                 self.hash_fs_path.as_posix(),
@@ -136,31 +175,29 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         result.mkdir(parents=True, exist_ok=True)
         return result
 
-    def _retrieve_environment_details(
-        self, env_type: str, env_hash: str
-    ) -> Mapping[str, Any]:
-
-        base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
-        env_details_file = base_path / f"{env_type}_{env_hash}.json"
-
-        if not env_details_file.exists():
-            raise Exception(
-                f"Can't load environment details, file does not exist: {env_details_file.as_posix()}"
-            )
-
-        environment = orjson.loads(env_details_file.read_text())
-        return environment
+    # def _retrieve_environment_details(
+    #     self, env_type: str, env_hash: str
+    # ) -> Mapping[str, Any]:
+    #
+    #     base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
+    #     env_details_file = base_path / f"{env_type}_{env_hash}.json"
+    #
+    #     if not env_details_file.exists():
+    #         raise Exception(
+    #             f"Can't load environment details, file does not exist: {env_details_file.as_posix()}"
+    #         )
+    #
+    #     environment: Mapping[str, Any] = orjson.loads(env_details_file.read_text())
+    #     return environment
 
     def retrieve_all_job_hashes(
         self,
         manifest_hash: Union[str, None] = None,
         inputs_hash: Union[str, None] = None,
     ) -> Iterable[str]:
-
         raise NotImplementedError()
 
     def _retrieve_record_for_job_hash(self, job_hash: str) -> JobRecord:
-
         raise NotImplementedError()
 
     # def find_matching_job_record(
@@ -227,7 +264,6 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         value_size: Union[int, None] = None,
         data_type_name: Union[str, None] = None,
     ) -> Set[uuid.UUID]:
-
         value_data_folder = self.get_path(entity_type=EntityType.VALUE_DATA)
 
         glob = f"*/{value_hash}/value_id__*.json"
@@ -251,7 +287,6 @@ class FileSystemDataArchive(DataArchive, JobArchive):
     def _find_destinies_for_value(
         self, value_id: uuid.UUID, alias_filter: Union[str, None] = None
     ) -> Union[Mapping[str, uuid.UUID], None]:
-
         destiny_dir = self.get_path(entity_type=EntityType.DESTINY_LINK)
         destiny_value_dir = destiny_dir / str(value_id)
 
@@ -274,7 +309,6 @@ class FileSystemDataArchive(DataArchive, JobArchive):
     def _retrieve_all_value_ids(
         self, data_type_name: Union[str, None] = None
     ) -> Iterable[uuid.UUID]:
-
         if data_type_name is not None:
             raise NotImplementedError()
 
@@ -283,15 +317,19 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         return folders
 
     def has_value(self, value_id: uuid.UUID) -> bool:
-        """Check whether the specific value_id is persisted in this data store.
+        """
+        Check whether the specific value_id is persisted in this data store.
         way to quickly determine whether a value id is valid for this data store.
 
         Arguments:
+        ---------
             value_id: the id of the value to check.
+
+
         Returns:
+        -------
             whether this data store contains the value with the specified id
         """
-
         base_path = (
             self.get_path(entity_type=EntityType.VALUE)
             / str(value_id)
@@ -300,7 +338,6 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         return base_path.is_file()
 
     def _retrieve_value_details(self, value_id: uuid.UUID) -> Mapping[str, Any]:
-
         base_path = (
             self.get_path(entity_type=EntityType.VALUE)
             / str(value_id)
@@ -311,11 +348,10 @@ class FileSystemDataArchive(DataArchive, JobArchive):
                 f"Can't retrieve details for value with id '{value_id}': no value with that id stored."
             )
 
-        value_data = orjson.loads(base_path.read_text())
+        value_data: Mapping[str, Any] = orjson.loads(base_path.read_text())
         return value_data
 
     def _retrieve_serialized_value(self, value: Value) -> PersistedData:
-
         base_path = self.get_path(entity_type=EntityType.VALUE_DATA)
         data_dir = base_path / value.data_type_name / str(value.value_hash)
 
@@ -324,48 +360,59 @@ class FileSystemDataArchive(DataArchive, JobArchive):
 
         return PersistedData(**data)
 
-    def retrieve_chunk(
+    def _retrieve_chunk(
         self,
         chunk_id: str,
-        as_file: Union[bool, str, None] = None,
+        as_file: bool = True,
         symlink_ok: bool = True,
     ) -> Union[bytes, str]:
-
         addr = self.hashfs.get(chunk_id)
+        if addr is None:
+            raise KiaraException(f"Can't find chunk with id '{chunk_id}'")
 
-        if as_file in (None, True):
-            return addr.abspath
+        if as_file is True:
+            result: str = addr.abspath
+            return result
         elif as_file is False:
             return Path(addr.abspath).read_bytes()
         else:
             raise NotImplementedError()
 
+    def retrieve_chunks(
+        self,
+        chunk_ids: Sequence[str],
+        as_files: bool = True,
+        symlink_ok: bool = True,
+    ) -> Generator[Union["BytesLike", str], None, None]:
+        for chunk_id in chunk_ids:
+            yield self._retrieve_chunk(
+                chunk_id, as_file=as_files, symlink_ok=symlink_ok
+            )
+
 
 class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
     """Data store that stores data as files on the local filesystem."""
 
     _archive_type_name = "filesystem_data_store"
 
-    def _persist_environment_details(
-        self, env_type: str, env_hash: str, env_data: Mapping[str, Any]
-    ):
-
-        base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
-        env_details_file = base_path / f"{env_type}_{env_hash}.json"
-
-        if not env_details_file.exists():
-            env_details_file.write_text(orjson_dumps(env_data))
+    # def _persist_environment_details(
+    #     self, env_type: str, env_hash: str, env_data: Mapping[str, Any]
+    # ):
+    #
+    #     base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
+    #     env_details_file = base_path / f"{env_type}_{env_hash}.json"
+    #
+    #     if not env_details_file.exists():
+    #         env_details_file.write_text(orjson_dumps(env_data))
 
     def _persist_stored_value_info(self, value: Value, persisted_value: PersistedData):
-
         working_dir = self.get_path(entity_type=EntityType.VALUE_DATA)
         data_dir = working_dir / value.data_type_name / str(value.value_hash)
         sv_file = data_dir / ".serialized_value.json"
         data_dir.mkdir(exist_ok=True, parents=True)
-        sv_file.write_text(persisted_value.json())
+        sv_file.write_text(persisted_value.model_dump_json())
 
     def _persist_value_details(self, value: Value):
-
         value_dir = self.get_path(entity_type=EntityType.VALUE) / str(value.value_id)
 
         if value_dir.exists():
@@ -376,15 +423,13 @@ class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
             value_dir.mkdir(parents=True, exist_ok=False)
 
         value_file = value_dir / VALUE_DETAILS_FILE_NAME
-        value_data = value.dict()
+        value_data = value.model_dump()
         value_file.write_text(orjson_dumps(value_data, option=orjson.OPT_NON_STR_KEYS))
 
     def _persist_destiny_backlinks(self, value: Value):
-
         destiny_dir = self.get_path(entity_type=EntityType.DESTINY_LINK)
 
         for value_id, backlink in value.destiny_backlinks.items():
-
             destiny_value_dir = destiny_dir / str(value_id)
             destiny_value_dir.mkdir(parents=True, exist_ok=True)
             destiny_file = destiny_value_dir / f"{backlink}.json"
@@ -398,60 +443,70 @@ class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
 
             fix_windows_symlink(value_file, destiny_file)
 
-    def _persist_value_data(self, value: Value) -> PersistedData:
+    def _persist_chunks(self, chunks: Mapping["CID", Union[str, BytesIO]]):
+        for cid, chunk in chunks.items():
+            self._persist_chunk(str(cid), chunk)
 
-        serialized_value: SerializedData = value.serialized_data
+    def _persist_chunk(self, chunk_id: str, chunk: Union[str, BytesIO]):
+        addr: HashAddress = self.hashfs.put_with_precomputed_hash(chunk, chunk_id)
 
-        chunk_id_map = {}
-        for key in serialized_value.get_keys():
+        assert addr.id == chunk_id
+        # return addr
+        # chunk_ids.append(addr.id)
 
-            data_model = serialized_value.get_serialized_data(key)
-
-            if data_model.type == "chunk":  # type: ignore
-                chunks: Iterable[Union[str, BytesIO]] = [BytesIO(data_model.chunk)]  # type: ignore
-            elif data_model.type == "chunks":  # type: ignore
-                chunks = (BytesIO(c) for c in data_model.chunks)  # type: ignore
-            elif data_model.type == "file":  # type: ignore
-                chunks = [data_model.file]  # type: ignore
-            elif data_model.type == "files":  # type: ignore
-                chunks = data_model.files  # type: ignore
-            elif data_model.type == "inline-json":  # type: ignore
-                chunks = [BytesIO(data_model.as_json())]  # type: ignore
-            else:
-                raise Exception(
-                    f"Invalid serialized data type: {type(data_model)}. Available types: {', '.join(SERIALIZE_TYPES)}"
-                )
-
-            chunk_ids = []
-            for item in zip(serialized_value.get_cids_for_key(key), chunks):
-                cid = item[0]
-                _chunk = item[1]
-                addr: HashAddress = self.hashfs.put_with_precomputed_hash(
-                    _chunk, str(cid)
-                )
-                chunk_ids.append(addr.id)
-
-            scids = SerializedChunkIDs(
-                chunk_id_list=chunk_ids,
-                archive_id=self.archive_id,
-                size=data_model.get_size(),
-            )
-            scids._data_registry = self.kiara_context.data_registry
-            chunk_id_map[key] = scids
-
-        pers_value = PersistedData(
-            archive_id=self.archive_id,
-            chunk_id_map=chunk_id_map,
-            data_type=serialized_value.data_type,
-            data_type_config=serialized_value.data_type_config,
-            serialization_profile=serialized_value.serialization_profile,
-            metadata=serialized_value.metadata,
-        )
-
-        return pers_value
+    # def _persist_value_data(self, value: Value) -> PersistedData:
+    #
+    #     serialized_value: SerializedData = value.serialized_data
+    #
+    #     chunk_id_map = {}
+    #     for key in serialized_value.get_keys():
+    #
+    #         data_model = serialized_value.get_serialized_data(key)
+    #
+    #         if data_model.type == "chunk":  # type: ignore
+    #             chunks: Iterable[Union[str, BytesIO]] = [BytesIO(data_model.chunk)]  # type: ignore
+    #         elif data_model.type == "chunks":  # type: ignore
+    #             chunks = (BytesIO(c) for c in data_model.chunks)  # type: ignore
+    #         elif data_model.type == "file":  # type: ignore
+    #             chunks = [data_model.file]  # type: ignore
+    #         elif data_model.type == "files":  # type: ignore
+    #             chunks = data_model.files  # type: ignore
+    #         elif data_model.type == "inline-json":  # type: ignore
+    #             chunks = [BytesIO(data_model.as_json())]  # type: ignore
+    #         else:
+    #             raise Exception(
+    #                 f"Invalid serialized data type: {type(data_model)}. Available types: {', '.join(SERIALIZE_TYPES)}"
+    #             )
+    #
+    #         chunk_ids = []
+    #         for item in zip(serialized_value.get_cids_for_key(key), chunks):
+    #             cid = item[0]
+    #             _chunk = item[1]
+    #             addr: HashAddress = self.hashfs.put_with_precomputed_hash(
+    #                 _chunk, str(cid)
+    #             )
+    #             chunk_ids.append(addr.id)
+    #
+    #         scids = SerializedChunkIDs(
+    #             chunk_id_list=chunk_ids,
+    #             archive_id=self.archive_id,
+    #             size=data_model.get_size(),
+    #         )
+    #         scids._data_registry = self.kiara_context.data_registry
+    #         chunk_id_map[key] = scids
+    #
+    #     pers_value = PersistedData(
+    #         archive_id=self.archive_id,
+    #         chunk_id_map=chunk_id_map,
+    #         data_type=serialized_value.data_type,
+    #         data_type_config=serialized_value.data_type_config,
+    #         serialization_profile=serialized_value.serialization_profile,
+    #         metadata=serialized_value.metadata,
+    #     )
+    #
+    #     return pers_value
 
     def _persist_value_pedigree(self, value: Value):
-
         manifest_hash = value.pedigree.instance_cid
         jobs_hash = value.pedigree.job_hash
 
