@@ -7,7 +7,7 @@
 
 
 import inspect
-from typing import Any, Dict, Iterable, Mapping, Type, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Type, Union
 
 from pydantic import BaseModel, Field, create_model
 from rich import box
@@ -16,30 +16,34 @@ from rich.table import Table
 from kiara.models.runtime_environment import RuntimeEnvironment, logger
 from kiara.utils import _get_all_subclasses, is_debug, to_camel_case
 
+if TYPE_CHECKING:
+    pass
+
 
 class EnvironmentRegistry(object):
-
     _instance = None
 
     @classmethod
     def instance(cls) -> "EnvironmentRegistry":
         """The default *kiara* context. In most cases, it's recommended you create and manage your own, though."""
-
         if cls._instance is None:
             cls._instance = EnvironmentRegistry()
         return cls._instance
 
-    def __init__(
-        self,
-    ) -> None:
+    def __init__(self) -> None:
         self._environments: Union[Dict[str, RuntimeEnvironment], None] = None
         self._environment_hashes: Union[Dict[str, Mapping[str, str]], None] = None
 
         self._full_env_model: Union[BaseModel, None] = None
 
-    def get_environment_for_cid(self, env_cid: str) -> RuntimeEnvironment:
+        # self._kiara: Kiara = kiara
 
-        envs = [env for env in self.environments.values() if env.instance_id == env_cid]
+    def get_environment_for_cid(self, env_cid: str) -> RuntimeEnvironment:
+        envs = [
+            env
+            for env in self.environments.values()
+            if str(env.instance_cid) == env_cid
+        ]
         if len(envs) == 0:
             raise Exception(f"No environment with id '{env_cid}' available.")
         elif len(envs) > 1:
@@ -48,9 +52,14 @@ class EnvironmentRegistry(object):
             )
         return envs[0]
 
+    def has_environment(self, env_cid: str) -> bool:
+        for env in self.environments.values():
+            if str(env.instance_cid) == env_cid:
+                return True
+        return False
+
     @property
     def environment_hashes(self) -> Mapping[str, Mapping[str, str]]:
-
         if self._environment_hashes is not None:
             return self._environment_hashes
 
@@ -64,12 +73,11 @@ class EnvironmentRegistry(object):
     @property
     def environments(self) -> Mapping[str, RuntimeEnvironment]:
         """Return all environments in this kiara runtime context."""
-
         if self._environments is not None:
             return self._environments
 
         import kiara.models.runtime_environment.kiara
-        import kiara.models.runtime_environment.operating_system
+        import kiara.models.runtime_environment.operating_system  # nowa
         import kiara.models.runtime_environment.python  # noqa
 
         subclasses: Iterable[Type[RuntimeEnvironment]] = _get_all_subclasses(
@@ -92,7 +100,6 @@ class EnvironmentRegistry(object):
     @property
     def full_model(self) -> BaseModel:
         """A model containing all environment data, incl. schemas and hashes of each sub-environment."""
-
         if self._full_env_model is not None:
             return self._full_env_model
 
@@ -130,18 +137,17 @@ class EnvironmentRegistry(object):
         cls: Type[BaseModel] = create_model("KiaraRuntimeInfo", **models)  # type: ignore
         data = {}
         for k2, v2 in self.environments.items():
-            d = v2.dict()
+            d = v2.model_dump()
             assert "metadata_hash" not in d.keys()
             assert "metadata_schema" not in d.keys()
             d["metadata_hash"] = str(hashes[k2])
             d["metadata_schema"] = schemas[k]
             data[k2] = d
-        model = cls.construct(**data)  # type: ignore
+        model = cls(**data)  # type: ignore
         self._full_env_model = model
         return self._full_env_model
 
     def create_renderable(self, **config: Any):
-
         full_details = config.get("full_details", False)
 
         table = Table(show_header=True, box=box.SIMPLE)
