@@ -7,75 +7,52 @@
 
 import abc
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Type, Union
+from datetime import datetime
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Type, Union
 
 import structlog
 from bidict import bidict
+from rich.console import Group
 
-from kiara.context.config import JobCacheStrategy
-from kiara.exceptions import FailedJobException
+from kiara.defaults import (
+    DEFAULT_DATA_STORE_MARKER,
+    DEFAULT_JOB_STORE_MARKER,
+    DEFAULT_STORE_MARKER,
+    ENVIRONMENT_MARKER_KEY,
+)
+from kiara.exceptions import FailedJobException, KiaraException
 from kiara.models.events import KiaraEvent
 from kiara.models.events.job_registry import (
     JobArchiveAddedEvent,
     JobRecordPreStoreEvent,
     JobRecordStoredEvent,
 )
-from kiara.models.module.jobs import ActiveJob, JobConfig, JobRecord, JobStatus
+from kiara.models.module.jobs import (
+    ActiveJob,
+    JobConfig,
+    JobMatcher,
+    JobRecord,
+    JobStatus,
+)
 from kiara.models.module.manifest import InputsManifest, Manifest
-from kiara.models.values.value import ValueMap
+from kiara.models.values.value import ValueMap, ValueMapReadOnly
 from kiara.processing import ModuleProcessor
 from kiara.processing.synchronous import SynchronousProcessor
-from kiara.registries import BaseArchive
+from kiara.registries.jobs.job_store import JobArchive, JobStore
 from kiara.utils import get_dev_config, is_develop
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
-
+    from kiara.context.runtime_config import JobCacheStrategy
+    from kiara.models.runtime_environment import RuntimeEnvironment
 
 logger = structlog.getLogger()
 
 MANIFEST_SUB_PATH = "manifests"
 
 
-class JobArchive(BaseArchive):
-    # @abc.abstractmethod
-    # def find_matching_job_record(
-    #     self, inputs_manifest: InputsManifest
-    # ) -> Optional[JobRecord]:
-    #     pass
-
-    @abc.abstractmethod
-    def retrieve_all_job_hashes(
-        self,
-        manifest_hash: Union[str, None] = None,
-        inputs_hash: Union[str, None] = None,
-    ) -> Iterable[str]:
-        """Retrieve a list of all job record hashes (cids) that match the given filter arguments.
-
-        A job record hash includes information about the module type used in the job, the module configuration, as well as input field names and value ids for the values used in those inputs.
-
-        If the job archive retrieves its jobs in a dynamic way, this will return 'None'.
-        """
-
-    @abc.abstractmethod
-    def _retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
-        pass
-
-    def retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
-
-        job_record = self._retrieve_record_for_job_hash(job_hash=job_hash)
-        return job_record
-
-
-class JobStore(JobArchive):
-    @abc.abstractmethod
-    def store_job_record(self, job_record: JobRecord):
-        pass
-
-
-class JobMatcher(abc.ABC):
+class ExistingJobMatcher(abc.ABC):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: Kiara = kiara
 
     @abc.abstractmethod
@@ -85,22 +62,20 @@ class JobMatcher(abc.ABC):
         pass
 
 
-class NoneJobMatcher(JobMatcher):
+class NoneExistingJobMatcher(ExistingJobMatcher):
     def find_existing_job(
         self, inputs_manifest: InputsManifest
     ) -> Union[JobRecord, None]:
         return None
 
 
-class ValueIdJobMatcher(JobMatcher):
+class ValueIdExistingJobMatcher(ExistingJobMatcher):
     def find_existing_job(
         self, inputs_manifest: InputsManifest
     ) -> Union[JobRecord, None]:
-
         matches = []
 
         for store_id, archive in self._kiara.job_registry.job_archives.items():
-
             match = archive.retrieve_record_for_job_hash(
                 job_hash=inputs_manifest.job_hash
             )
@@ -120,15 +95,19 @@ class ValueIdJobMatcher(JobMatcher):
         return job_record
 
 
-class DataHashJobMatcher(JobMatcher):
+class DataHashExistingJobMatcher(ExistingJobMatcher):
     def find_existing_job(
         self, inputs_manifest: InputsManifest
     ) -> Union[JobRecord, None]:
-
         matches = []
 
-        for store_id, archive in self._kiara.job_registry.job_archives.items():
+        ignore_internal = True
+        if ignore_internal:
+            module = self._kiara.module_registry.create_module(inputs_manifest)
+            if module.characteristics.is_internal:
+                return None
 
+        for store_id, archive in self._kiara.job_registry.job_archives.items():
             match = archive.retrieve_record_for_job_hash(
                 job_hash=inputs_manifest.job_hash
             )
@@ -141,17 +120,14 @@ class DataHashJobMatcher(JobMatcher):
             )
 
         elif len(matches) == 1:
-
             job_record = matches[0]
             job_record._is_stored = True
 
             return job_record
 
-        inputs_data_cid = inputs_manifest.calculate_inputs_data_cid(
+        inputs_data_cid, contains_invalid = inputs_manifest.calculate_inputs_data_cid(
             data_registry=self._kiara.data_registry
         )
-        if not inputs_data_cid:
-            return None
 
         inputs_data_hash = str(inputs_data_cid)
 
@@ -178,10 +154,9 @@ class DataHashJobMatcher(JobMatcher):
 
 class JobRegistry(object):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: Kiara = kiara
 
-        self._job_matcher_cache: Dict[JobCacheStrategy, JobMatcher] = {}
+        self._job_matcher_cache: Dict[JobCacheStrategy, ExistingJobMatcher] = {}
 
         self._active_jobs: bidict[str, uuid.UUID] = bidict()
         self._failed_jobs: Dict[str, uuid.UUID] = {}
@@ -195,6 +170,8 @@ class JobRegistry(object):
 
         self._event_callback = self._kiara.event_registry.add_producer(self)
 
+        self._env_cache: Dict[str, Dict[str, RuntimeEnvironment]] = {}
+
         # default_archive = FileSystemJobStore.create_from_kiara_context(self._kiara)
         # self.register_job_archive(default_archive, store_alias=DEFAULT_STORE_MARKER)
 
@@ -202,7 +179,8 @@ class JobRegistry(object):
         # self.register_job_archive(default_file_store, store_alias="default_data_store")  # type: ignore
 
     @property
-    def job_matcher(self) -> JobMatcher:
+    def job_matcher(self) -> ExistingJobMatcher:
+        from kiara.context.runtime_config import JobCacheStrategy
 
         strategy = self._kiara.runtime_config.job_cache
         if is_develop():
@@ -217,11 +195,11 @@ class JobRegistry(object):
         job_matcher = self._job_matcher_cache.get(strategy, None)
         if job_matcher is None:
             if strategy == JobCacheStrategy.no_cache:
-                job_matcher = NoneJobMatcher(kiara=self._kiara)
+                job_matcher = NoneExistingJobMatcher(kiara=self._kiara)
             elif strategy == JobCacheStrategy.value_id:
-                job_matcher = ValueIdJobMatcher(kiara=self._kiara)
+                job_matcher = ValueIdExistingJobMatcher(kiara=self._kiara)
             elif strategy == JobCacheStrategy.data_hash:
-                job_matcher = DataHashJobMatcher(kiara=self._kiara)
+                job_matcher = DataHashExistingJobMatcher(kiara=self._kiara)
             else:
                 raise Exception(f"Job cache strategy not implemented: {strategy}")
             self._job_matcher_cache[strategy] = job_matcher
@@ -229,19 +207,20 @@ class JobRegistry(object):
         return job_matcher
 
     def suppoerted_event_types(self) -> Iterable[Type[KiaraEvent]]:
-
         return [JobArchiveAddedEvent, JobRecordPreStoreEvent, JobRecordStoredEvent]
 
-    def register_job_archive(self, archive: JobArchive, alias: Union[str, None] = None):
+    def register_job_archive(self, archive: JobArchive) -> str:
+        alias = archive.archive_name
 
-        if alias is None:
-            alias = str(archive.archive_id)
+        if not alias:
+            raise Exception("Invalid job archive alias: can't be empty.")
 
         if alias in self._job_archives.keys():
             raise Exception(
                 f"Can't register job store, store id already registered: {alias}."
             )
 
+        archive.register_archive(self._kiara)
         self._job_archives[alias] = archive
 
         is_store = False
@@ -251,7 +230,7 @@ class JobRegistry(object):
             if self._default_job_store is None:
                 self._default_job_store = alias
 
-        event = JobArchiveAddedEvent.construct(
+        event = JobArchiveAddedEvent(
             kiara_id=self._kiara.id,
             job_archive_id=archive.archive_id,
             job_archive_alias=alias,
@@ -260,21 +239,34 @@ class JobRegistry(object):
         )
         self._event_callback(event)
 
+        return alias
+
     @property
     def default_job_store(self) -> str:
-
         if self._default_job_store is None:
             raise Exception("No default job store set (yet).")
         return self._default_job_store  # type: ignore
 
-    def get_archive(self, store_id: Union[str, None] = None) -> JobArchive:
-
-        if store_id is None:
-            store_id = self.default_job_store
-            if store_id is None:
+    def get_archive(self, store_id: Union[str, None, uuid.UUID] = None) -> JobArchive:
+        if store_id in [
+            None,
+            "",
+            DEFAULT_DATA_STORE_MARKER,
+            DEFAULT_JOB_STORE_MARKER,
+            DEFAULT_STORE_MARKER,
+        ]:
+            if self.default_job_store is None:
                 raise Exception("Can't retrieve deafult job archive, none set (yet).")
+            _store_id: str = self.default_job_store
 
-        return self._job_archives[store_id]
+        elif not isinstance(store_id, str):
+            raise NotImplementedError(
+                "Can't retrieve job archive by (uu)id or other type (yet)."
+            )
+        else:
+            _store_id = store_id
+
+        return self._job_archives[_store_id]
 
     @property
     def job_archives(self) -> Mapping[str, JobArchive]:
@@ -286,7 +278,6 @@ class JobRegistry(object):
         old_status: Union[JobStatus, None],
         new_status: JobStatus,
     ):
-
         # print(f"JOB STATUS CHANGED: {job_id} - {old_status} - {new_status.value}")
         if job_id in self._active_jobs.values() and new_status is JobStatus.FAILED:
             job_hash = self._active_jobs.inverse.pop(job_id)
@@ -299,14 +290,30 @@ class JobRegistry(object):
             self._finished_jobs[job_hash] = job_id
             self._archived_records[job_id] = job_record
 
-    def store_job_record(self, job_id: uuid.UUID):
+    def _persist_environment(self, env_type: str, env_hash: str):
+        cached = self._env_cache.get(env_type, {}).get(env_hash, None)
+        if cached is not None:
+            return
 
-        if job_id not in self._archived_records.keys():
-            raise Exception(
-                f"Can't store job with id '{job_id}': no job record with that id exists."
+        environment = self._kiara.metadata_registry.retrieve_environment_item(env_hash)
+
+        if not environment:
+            raise KiaraException(
+                f"Can't persist job environment for with hash '{env_hash}': no such environment registered."
             )
 
-        job_record = self._archived_records[job_id]
+        self._kiara.metadata_registry.register_metadata_item(
+            key=ENVIRONMENT_MARKER_KEY, item=environment
+        )
+        self._env_cache.setdefault(env_type, {})[env_hash] = environment
+
+    def store_job_record(self, job_id: uuid.UUID, store: Union[str, None] = None):
+        # TODO: allow to store job record to external store
+
+        job_record = self.get_job_record(job_id=job_id)
+
+        for env_type, env_hash in job_record.environment_hashes.items():
+            self._persist_environment(env_type, env_hash)
 
         if job_record._is_stored:
             logger.debug(
@@ -314,42 +321,32 @@ class JobRegistry(object):
             )
             return
 
-        store: JobStore = self.get_archive()  # type: ignore
+        store: JobStore = self.get_archive(store)  # type: ignore
         if not isinstance(store, JobStore):
             raise Exception("Can't store job record to archive: not writable.")
 
-        # if job_record.job_id in self._finished_jobs.values():
-        #     logger.debug(
-        #         "ignore.store.job_record",
-        #         reason="already stored in store",
-        #         job_id=str(job_id),
-        #     )
-        #     return
-
         logger.debug(
             "store.job_record",
             job_hash=job_record.job_hash,
             module_type=job_record.module_type,
         )
 
-        pre_store_event = JobRecordPreStoreEvent.construct(
+        pre_store_event = JobRecordPreStoreEvent(
             kiara_id=self._kiara.id, job_record=job_record
         )
         self._event_callback(pre_store_event)
 
         store.store_job_record(job_record)
 
-        stored_event = JobRecordStoredEvent.construct(
+        stored_event = JobRecordStoredEvent(
             kiara_id=self._kiara.id, job_record=job_record
         )
         self._event_callback(stored_event)
 
     def get_job_record_in_session(self, job_id: uuid.UUID) -> JobRecord:
-
         return self._processor.get_job_record(job_id)
 
-    def get_job_record(self, job_id: uuid.UUID) -> Union[JobRecord, None]:
-
+    def get_job_record(self, job_id: uuid.UUID) -> JobRecord:
         if job_id in self._archived_records.keys():
             return self._archived_records[job_id]
 
@@ -359,40 +356,103 @@ class JobRegistry(object):
         except Exception:
             pass
 
-        job = self._processor.get_job(job_id=job_id)
-        if job is not None:
-            if job.status == JobStatus.FAILED:
-                return None
+        # try:
+        #     job = self._processor.get_job(job_id=job_id)
+        #     if job is not None:
+        #         if job.status == JobStatus.FAILED:
+        #             return None
+        # except Exception:
+        #     pass
 
-        raise NotImplementedError()
+        all_job_records = self.retrieve_all_job_records()
+        for r in all_job_records.values():
+            if r.job_id == job_id:
+                return r
 
-    def retrieve_all_job_records(self) -> Mapping[str, JobRecord]:
+        # raise a FailedJobException if the job is in the failed jobs list
+        self.get_active_job(job_id=job_id)
 
-        all_records: Dict[str, JobRecord] = {}
+        # this should never happen
+        raise KiaraException("Can't find job record with id: {job_id}")
+
+    def find_job_records(self, matcher: JobMatcher) -> Mapping[uuid.UUID, JobRecord]:
+        all_records: List[JobRecord] = []
         for archive in self.job_archives.values():
-            all_record_ids = archive.retrieve_all_job_hashes()
-            if all_record_ids is None:
-                return {}
+            _job_records = archive.retrieve_matching_job_records(matcher=matcher)
+            all_records.extend(_job_records)
+
+        # TODO: check for duplicates and mismatching datetimes
+        all_jobs_sorted = {
+            job.job_id: job
+            for job in sorted(
+                all_records,
+                key=lambda item: item.job_submitted,
+                reverse=True,
+            )
+        }
+
+        return all_jobs_sorted
+
+    def retrieve_all_job_record_ids(self) -> List[uuid.UUID]:
+        """Retrieve a list of all available job record ids, sorted from latest to earliest."""
+
+        all_records: Dict[uuid.UUID, datetime] = {}
+        for archive in self.job_archives.values():
+            all_record_ids = archive.retrieve_all_job_ids()
+            # TODO: check for duplicates and mismatching datetimes
+            all_records.update(all_record_ids)
+
+        all_ids_sorted = [
+            uuid
+            for uuid, _ in sorted(
+                all_records.items(), key=lambda item: item[1], reverse=True
+            )
+        ]
+
+        return all_ids_sorted
+
+    def retrieve_all_job_records(self) -> Mapping[uuid.UUID, JobRecord]:
+        """Retrieves all job records from all job archives.
+
+        Returns:
+            a map of job-id/job-record pairs, sorted by job submission time, from latest to earliest
+        """
+
+        all_records: Dict[uuid.UUID, JobRecord] = {}
+        for archive in self.job_archives.values():
+            all_record_ids = archive.retrieve_all_job_ids().keys()
             for r in all_record_ids:
-                assert r not in all_records.keys()
-                job_record = archive.retrieve_record_for_job_hash(r)
+                if r in all_records.keys():
+                    continue
+
+                job_record = archive.retrieve_record_for_job_id(r)
                 assert job_record is not None
                 all_records[r] = job_record
 
-        return all_records
+        all_records_sorted = dict(
+            sorted(
+                all_records.items(),
+                key=lambda item: item[1].job_submitted,
+                reverse=True,
+            )
+        )
 
-    def find_matching_job_record(
+        return all_records_sorted
+
+    def find_job_record_for_manifest(
         self, inputs_manifest: InputsManifest
     ) -> Union[uuid.UUID, None]:
-        """Check if a job with same inputs manifest already ran some time before.
+        """
+        Check if a job with same inputs manifest already ran some time before.
 
         Arguments:
+        ---------
             inputs_manifest: the manifest incl. inputs
 
         Returns:
+        -------
             'None' if no such job exists, a (uuid) job-id if the job is currently running or has run in the past
         """
-
         log = logger.bind(module_type=inputs_manifest.module_type)
         if inputs_manifest.job_hash in self._active_jobs.keys():
             log.debug("job.use_running")
@@ -412,6 +472,7 @@ class JobRegistry(object):
             return None
 
         job_record = self.job_matcher.find_existing_job(inputs_manifest=inputs_manifest)
+
         if job_record is None:
             return None
 
@@ -423,11 +484,16 @@ class JobRegistry(object):
             job_hash=inputs_manifest.job_hash,
             module_type=inputs_manifest.module_type,
         )
+
         return job_record.job_id
 
     def prepare_job_config(
         self, manifest: Manifest, inputs: Mapping[str, Any]
     ) -> JobConfig:
+        """Prepare a JobConfig instance from the manifest and inputs.
+
+        This involves creating (and therefor validating) a module instance, as well as making sure the inputs are valid.
+        """
 
         module = self._kiara.module_registry.create_module(manifest=manifest)
 
@@ -442,49 +508,113 @@ class JobRegistry(object):
         manifest: Manifest,
         inputs: Mapping[str, Any],
         wait: bool = False,
-        job_metadata: Union[None, Any] = None,
     ) -> uuid.UUID:
+        """Prepare a job config, then execute it."""
 
         job_config = self.prepare_job_config(manifest=manifest, inputs=inputs)
-        return self.execute_job(job_config, wait=wait, job_metadata=job_metadata)
+        return self.execute_job(job_config, wait=wait)
 
     def execute_job(
-        self,
-        job_config: JobConfig,
-        wait: bool = False,
-        job_metadata: Union[None, Any] = None,
+        self, job_config: JobConfig, wait: bool = False, auto_save_result=False
     ) -> uuid.UUID:
+        """Execute the job specified by the job config.
 
-        log = logger.bind(
-            module_type=job_config.module_type,
-            module_config=job_config.module_config,
-            inputs={k: str(v) for k, v in job_config.inputs.items()},
-            job_hash=job_config.job_hash,
-        )
+        Arguments:
+            job_config: the job config
+            wait: whether to wait for the job to finish
+            auto_save_result: whether to automatically save the job's outputs to the data registry once the job finished successfully
+        """
+
+        # from kiara.models.metadata import CommentMetadata
+        # if "comment" not in job_metadata.keys():
+        #     raise KiaraException("You need to provide a 'comment' for the job.")
+        #
+        # comment = job_metadata.get("comment")
+        # if not isinstance(comment, str):
+        #     raise KiaraException("The 'comment' must be a string.")
+        #
+        # comment_metadata = CommentMetadata(comment=comment)
+        # self.context.metadata_registry.register_metadata_item(
+        #     key="comment", item=comment_metadata, force=False, store=None
+        # )
+
+        if job_config.module_type != "pipeline":
+            log = logger.bind(
+                module_type=job_config.module_type,
+                module_config=job_config.module_config,
+                inputs={k: str(v) for k, v in job_config.inputs.items()},
+                job_hash=job_config.job_hash,
+            )
+        else:
+            pipeline_name = job_config.module_config.get("pipeline_name", "n/a")
+            log = logger.bind(
+                module_type=job_config.module_type,
+                pipeline_name=pipeline_name,
+                inputs={k: str(v) for k, v in job_config.inputs.items()},
+                job_hash=job_config.job_hash,
+            )
+
+        stored_job = self.find_job_record_for_manifest(inputs_manifest=job_config)
+
+        is_pipeline_step = False if job_config.pipeline_metadata is None else True
+        if is_pipeline_step:
+            pipeline_step_id: Union[None, str] = job_config.pipeline_metadata.step_id  # type: ignore
+            pipeline_id: Union[None, uuid.UUID] = (
+                job_config.pipeline_metadata.pipeline_id  # type: ignore
+            )  # type: ignore
+        else:
+            pipeline_step_id = None
+            pipeline_id = None
 
-        stored_job = self.find_matching_job_record(inputs_manifest=job_config)
         if stored_job is not None:
             log.debug(
                 "job.use_cached",
                 job_id=str(stored_job),
                 module_type=job_config.module_type,
             )
+            if is_develop():
+                module = self._kiara.module_registry.create_module(manifest=job_config)
+                if is_pipeline_step:
+                    title = f"Using cached pipeline step: {pipeline_step_id}"
+                else:
+                    title = f"Using cached job for: {module.module_type_name}"
+
+                from kiara.utils.debug import create_module_preparation_table
+                from kiara.utils.develop import log_dev_message
+
+                stored_job_record = self.get_job_record(stored_job)
+                if stored_job_record is None:
+                    raise Exception(
+                        f"Can't retrieve job record for job with id '{stored_job}'."
+                    )
+
+                table = create_module_preparation_table(
+                    kiara=self._kiara,
+                    job_config=job_config,
+                    job_id=stored_job_record.job_id,
+                    module=module,
+                )
+                include = ["job_hash", "inputs_id_hash", "input_ids_hash", "outputs"]
+                table_job_record = stored_job_record.create_renderable(include=include)
+                panel = Group(table, table_job_record)
+                log_dev_message(panel, title=title)
+
+            # TODO: in this case, and if 'auto_save_result' is True, we should also verify the outputs are saved?
+
             return stored_job
 
-        if job_metadata is None:
-            job_metadata = {}
-
-        is_pipeline_step = job_metadata.get("is_pipeline_step", False)
         dbg_data = {
             "module_type": job_config.module_type,
             "is_pipeline_step": is_pipeline_step,
         }
         if is_pipeline_step:
-            dbg_data["step_id"] = job_metadata["step_id"]
+            dbg_data["step_id"] = pipeline_step_id
+            dbg_data["pipeline_id"] = str(pipeline_id)
+
         log.debug("job.execute", **dbg_data)
 
         job_id = self._processor.create_job(
-            job_config=job_config, job_metadata=job_metadata
+            job_config=job_config, auto_save_result=auto_save_result
         )
         self._active_jobs[job_config.job_hash] = job_id
 
@@ -500,7 +630,6 @@ class JobRegistry(object):
         return job_id
 
     def get_active_job(self, job_id: uuid.UUID) -> ActiveJob:
-
         if job_id in self._active_jobs.keys() or job_id in self._failed_jobs.keys():
             return self._processor.get_job(job_id)
         else:
@@ -523,7 +652,6 @@ class JobRegistry(object):
         return self._processor.get_job(job_id=job_id)
 
     def get_job_status(self, job_id: uuid.UUID) -> JobStatus:
-
         if job_id in self._archived_records.keys():
             return JobStatus.SUCCESS
         elif job_id in self._failed_jobs.values():
@@ -536,8 +664,7 @@ class JobRegistry(object):
         if not_finished:
             self._processor.wait_for(*not_finished)
 
-    def retrieve_result(self, job_id: uuid.UUID) -> ValueMap:
-
+    def retrieve_result(self, job_id: uuid.UUID) -> ValueMapReadOnly:
         if job_id not in self._archived_records.keys():
             self._processor.wait_for(job_id)
 
@@ -547,14 +674,14 @@ class JobRegistry(object):
             return results
         elif job_id in self._failed_jobs.values():
             j = self._processor.get_job(job_id=job_id)
-            raise FailedJobException(job=j, parent=j._exception)
+            exception = FailedJobException(job=j, parent=j._exception)
+            raise exception
         else:
             raise Exception(f"Could not find job with id: {job_id}")
 
     def execute_and_retrieve(
         self, manifest: Manifest, inputs: Mapping[str, Any]
     ) -> ValueMap:
-
         job_id = self.execute(manifest=manifest, inputs=inputs, wait=True)
         results = self.retrieve_result(job_id=job_id)
         return results
