@@ -8,12 +8,12 @@ import datetime
 import shutil
 import uuid
 from pathlib import Path
-from typing import Any, Dict, Iterable, Union
+from typing import Any, Dict, Generator, Iterable, Mapping, Union
 
 import orjson
 import structlog
 
-from kiara.models.module.jobs import JobRecord
+from kiara.models.module.jobs import JobMatcher, JobRecord
 from kiara.registries import ArchiveDetails, FileSystemArchiveConfig
 from kiara.registries.jobs import MANIFEST_SUB_PATH, JobArchive, JobStore
 from kiara.utils.windows import fix_windows_longpath
@@ -22,33 +22,58 @@ log = structlog.getLogger()
 
 
 class FileSystemJobArchive(JobArchive):
-
     _archive_type_name = "filesystem_job_archive"
     _config_cls = FileSystemArchiveConfig  # type: ignore
 
-    @classmethod
-    def is_writeable(cls) -> bool:
-        return False
-
     @classmethod
     def supported_item_types(cls) -> Iterable[str]:
         return ["job_record"]
 
-    def __init__(self, archive_id: uuid.UUID, config: FileSystemArchiveConfig):
-
-        super().__init__(archive_id=archive_id, config=config)
+    def __init__(
+        self,
+        archive_name: str,
+        archive_config: FileSystemArchiveConfig,
+        force_read_only: bool = False,
+    ):
+        super().__init__(
+            archive_name=archive_name,
+            archive_config=archive_config,
+            force_read_only=force_read_only,
+        )
         self._base_path: Union[Path, None] = None
 
     def get_archive_details(self) -> ArchiveDetails:
-
         size = sum(
             f.stat().st_size for f in self.job_store_path.glob("**/*") if f.is_file()
         )
-        return ArchiveDetails(size=size)
+        return ArchiveDetails(root={"size": size})
+
+    def _retrieve_archive_metadata(self) -> Mapping[str, Any]:
+        if not self.archive_metadata_path.is_file():
+            _archive_metadata = {}
+        else:
+            _archive_metadata = orjson.loads(self.archive_metadata_path.read_bytes())
+
+        archive_id = _archive_metadata.get("archive_id", None)
+        if not archive_id:
+            try:
+                _archive_id = uuid.UUID(
+                    self.job_store_path.name
+                )  # just to check if it's a valid uuid
+                _archive_metadata["archive_id"] = str(_archive_id)
+            except Exception:
+                raise Exception(
+                    f"Could not retrieve archive id for alias archive '{self.archive_name}'."
+                )
+
+        return _archive_metadata
+
+    @property
+    def archive_metadata_path(self) -> Path:
+        return self.job_store_path / "store_metadata.json"
 
     @property
     def job_store_path(self) -> Path:
-
         if self._base_path is not None:
             return self._base_path
 
@@ -58,26 +83,27 @@ class FileSystemJobArchive(JobArchive):
         return self._base_path
 
     def _delete_archive(self) -> None:
-
         shutil.rmtree(self.job_store_path)
 
     def retrieve_all_job_hashes(
         self,
         manifest_hash: Union[str, None] = None,
-        inputs_hash: Union[str, None] = None,
+        inputs_id_hash: Union[str, None] = None,
+        inputs_data_hash: Union[str, None] = None,
     ) -> Iterable[str]:
-
         base_path = self.job_store_path / MANIFEST_SUB_PATH
         if not manifest_hash:
-            if not inputs_hash:
+            if not inputs_id_hash:
                 records = base_path.glob("*/*/*.job_record")
             else:
-                records = base_path.glob(f"*/{inputs_hash}/*.job_record")
+                records = base_path.glob(f"*/{inputs_id_hash}/*.job_record")
         else:
-            if not inputs_hash:
+            if not inputs_id_hash:
                 records = base_path.glob(f"{manifest_hash}/*/*.job_record")
             else:
-                records = base_path.glob(f"{manifest_hash}/{inputs_hash}/*.job_record")
+                records = base_path.glob(
+                    f"{manifest_hash}/{inputs_id_hash}/*.job_record"
+                )
 
         result = []
         for record in records:
@@ -85,7 +111,6 @@ class FileSystemJobArchive(JobArchive):
         return result
 
     def _retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
-
         base_path = self.job_store_path / MANIFEST_SUB_PATH
         records = list(base_path.glob(f"*/*/{job_hash}.job_record"))
 
@@ -102,6 +127,17 @@ class FileSystemJobArchive(JobArchive):
         job_record._is_stored = True
         return job_record
 
+    def _retrieve_all_job_ids(self) -> Mapping[uuid.UUID, datetime.datetime]:
+        raise NotImplementedError()
+
+    def _retrieve_matching_job_records(
+        self, matcher: JobMatcher
+    ) -> Generator[JobRecord, None, None]:
+        raise NotImplementedError()
+
+    def _retrieve_record_for_job_id(self, job_id: uuid.UUID) -> Union[JobRecord, None]:
+        raise NotImplementedError()
+
     # def find_matching_job_record(
     #     self, inputs_manifest: InputsManifest
     # ) -> Optional[JobRecord]:
@@ -141,17 +177,12 @@ class FileSystemJobArchive(JobArchive):
 
 
 class FileSystemJobStore(FileSystemJobArchive, JobStore):
-
     _archive_type_name = "filesystem_job_store"
 
-    @classmethod
-    def is_writeable(cls) -> bool:
-        return False
-
     def store_job_record(self, job_record: JobRecord):
-
         manifest_cid = job_record.manifest_cid
-        inputs_hash = job_record.inputs_hash
+
+        input_ids_hash = job_record.input_ids_hash
 
         base_path = self.job_store_path / MANIFEST_SUB_PATH
         manifest_folder = base_path / str(manifest_cid)
@@ -162,7 +193,7 @@ class FileSystemJobStore(FileSystemJobArchive, JobStore):
         if not manifest_info_file.exists():
             manifest_info_file.write_text(job_record.manifest_data_as_json())
 
-        job_folder = manifest_folder / inputs_hash
+        job_folder = manifest_folder / input_ids_hash
         job_folder = fix_windows_longpath(job_folder)
         job_folder.mkdir(parents=True, exist_ok=True)
 
@@ -187,10 +218,9 @@ class FileSystemJobStore(FileSystemJobArchive, JobStore):
             )
             shutil.move(job_details_file.as_posix(), backup)
 
-        job_details_file.write_text(job_record.json())
+        job_details_file.write_text(job_record.model_dump_json())
 
         for output_name, output_v_id in job_record.outputs.items():
-
             outputs_file_name = (
                 job_folder / f"output__{output_name}__value_id__{output_v_id}.json"
             )
