@@ -23,9 +23,13 @@ from typing import (
 )
 
 import structlog
+from rich.console import Group, RenderableType
 from ruamel.yaml import YAML
 
-from kiara.exceptions import InvalidOperationException, NoSuchOperationException
+from kiara.exceptions import (
+    InvalidOperationException,
+    NoSuchOperationException,
+)
 from kiara.interfaces.python_api.models.info import (
     OperationTypeClassesInfo,
     OperationTypeInfo,
@@ -38,9 +42,12 @@ from kiara.models.module.operation import (
     PipelineOperationConfig,
 )
 from kiara.models.module.pipeline import PipelineConfig
+from kiara.models.module.pipeline.pipeline import Pipeline
 from kiara.models.python_class import KiaraModuleInstance
 from kiara.operations import OperationType
-from kiara.utils import log_exception, log_message
+from kiara.utils import is_develop, log_exception, log_message
+from kiara.utils.cli import terminal_print
+from kiara.utils.output import extract_renderable
 from kiara.utils.pipelines import find_pipeline_data_in_paths
 
 if TYPE_CHECKING:
@@ -59,12 +66,11 @@ class OperationRegistry(object):
         kiara: "Kiara",
         operation_type_classes: Union[Mapping[str, Type[OperationType]], None] = None,
     ):
-
         self._kiara: "Kiara" = kiara
 
-        self._operation_type_classes: Union[
-            Dict[str, Type["OperationType"]], None
-        ] = None
+        self._operation_type_classes: Union[Dict[str, Type["OperationType"]], None] = (
+            None
+        )
 
         if operation_type_classes is not None:
             self._operation_type_classes = dict(operation_type_classes)
@@ -82,11 +88,9 @@ class OperationRegistry(object):
 
     @property
     def is_initialized(self) -> bool:
-
         return self._operations is not None
 
     def get_module_map(self) -> Mapping[str, Mapping[str, Any]]:
-
         if not self.is_initialized:
             raise Exception(
                 "Can't retrieve module map: operations not initialized yet."
@@ -106,7 +110,6 @@ class OperationRegistry(object):
 
     @property
     def operation_types(self) -> Mapping[str, OperationType]:
-
         if self._operation_types is not None:
             return self._operation_types
 
@@ -125,7 +128,6 @@ class OperationRegistry(object):
         return self._operation_types
 
     def get_operation_type(self, op_type: Union[str, Type[OP_TYPE]]) -> OP_TYPE:
-
         if not isinstance(op_type, str):
             try:
                 op_type = op_type._operation_type_name  # type: ignore
@@ -142,7 +144,6 @@ class OperationRegistry(object):
         return self.operation_types[op_type]  # type: ignore
 
     def get_type_metadata(self, type_name: str) -> OperationTypeInfo:
-
         md = self._operation_type_metadata.get(type_name, None)
         if md is None:
             md = OperationTypeInfo.create_from_type_class(
@@ -154,7 +155,6 @@ class OperationRegistry(object):
     def get_context_metadata(
         self, alias: Union[str, None] = None, only_for_package: Union[str, None] = None
     ) -> OperationTypeClassesInfo:
-
         result = {}
         for type_name in self.operation_type_classes.keys():
             md = self.get_type_metadata(type_name=type_name)
@@ -164,13 +164,12 @@ class OperationRegistry(object):
             else:
                 result[type_name] = md
 
-        return OperationTypeClassesInfo.construct(group_alias=alias, item_infos=result)  # type: ignore
+        return OperationTypeClassesInfo(group_title=alias, item_infos=result)  # type: ignore
 
     @property
     def operation_type_classes(
         self,
     ) -> Mapping[str, Type["OperationType"]]:
-
         if self._operation_type_classes is not None:
             return self._operation_type_classes
 
@@ -189,7 +188,6 @@ class OperationRegistry(object):
 
     @property
     def operations(self) -> Mapping[str, Operation]:
-
         if self._operations is not None:
             return self._operations
 
@@ -203,7 +201,8 @@ class OperationRegistry(object):
 
         for data_type in self._kiara.data_type_classes.values():
             if hasattr(data_type, "retrieve_included_operations"):
-                for op in all_op_configs:
+                included_ops = data_type.retrieve_included_operations()
+                for op in included_ops:
                     if isinstance(op, Mapping):
                         op = ManifestOperationConfig(**op)
                     all_op_configs.add(op)
@@ -215,9 +214,7 @@ class OperationRegistry(object):
 
         # first iteration
         for op_config in all_op_configs:
-
             try:
-
                 if isinstance(op_config, PipelineOperationConfig):
                     for mt in op_config.required_module_types:
                         if mt not in self._kiara.module_type_names:
@@ -243,17 +240,15 @@ class OperationRegistry(object):
                 continue
 
             try:
-
                 module_type = op_config.retrieve_module_type(kiara=self._kiara)
                 if module_type not in self._kiara.module_type_names:
                     deferred_module_names.setdefault(module_type, []).append(op_config)
                 else:
                     module_config = op_config.retrieve_module_config(kiara=self._kiara)
 
-                    manifest = Manifest.construct(
+                    manifest = Manifest(
                         module_type=module_type, module_config=module_config
                     )
-
                     ops = self._create_operations(manifest=manifest, doc=op_config.doc)
 
                     for op_type_name, _op in ops.items():
@@ -287,7 +282,6 @@ class OperationRegistry(object):
 
         error_details = {}
         while deferred_module_names:
-
             deferred_length = len(deferred_module_names)
 
             remove_deferred_names = set()
@@ -299,9 +293,7 @@ class OperationRegistry(object):
 
                 for op_config in deferred_module_names[missing_op_id]:
                     try:
-
                         if isinstance(op_config, PipelineOperationConfig):
-
                             if all(
                                 mt in self._kiara.module_type_names
                                 or mt in operations.keys()
@@ -321,7 +313,7 @@ class OperationRegistry(object):
                                     kiara=self._kiara
                                 )
 
-                                manifest = Manifest.construct(
+                                manifest = Manifest(
                                     module_type="pipeline",
                                     module_config=module_config,
                                 )
@@ -353,11 +345,10 @@ class OperationRegistry(object):
                             # new_module_config = dict(base_config)
                             # new_module_config.update(module_config)
                             #
-                            # manifest = Manifest.construct(module_type=operation.module_type,
+                            # manifest = Manifest(module_type=operation.module_type,
                             #                       module_config=new_module_config)
 
                         for op_type_name, _op in ops.items():
-
                             if _op.operation_id in operations.keys():
                                 raise Exception(
                                     f"Duplicate operation id: {_op.operation_id}"
@@ -428,12 +419,13 @@ class OperationRegistry(object):
         return self._operations
 
     def register_pipelines(self, *paths: Union[str, Path]) -> Dict[str, Operation]:
-        """Register pipelines from one or more paths.
+        """
+        Register pipelines from one or more paths.
 
         Args:
+        ----
             *paths: one or more paths to load pipelines from.
         """
-
         pipeline_data = find_pipeline_data_in_paths(
             {k if isinstance(k, str) else k.as_posix(): {} for k in paths}
         )
@@ -451,7 +443,19 @@ class OperationRegistry(object):
         ops = {}
         for op_id, op_data in pipeline_data.items():
             # TODO: what to do with the additional data, like source and source type?
-            op = self.register_pipeline(data=op_data["data"], operation_id=op_id)
+            try:
+                op = self.register_pipeline(data=op_data["data"], operation_id=op_id)
+            except Exception as e:
+                log_message("invalid.pipeline", pipeline_id=op_id, reason=str(e))
+                if is_develop():
+                    renderables: List[RenderableType] = []
+                    renderables.append("")
+                    renderables.append(extract_renderable(e))
+                    renderables.append("")
+                    label = f"[red]Invalid Pipeline [/red][i]'{op_id}'[/i]"
+                    terminal_print(Group(*renderables), in_panel=label)
+                # log_exception(e)
+                continue
             ops[op.operation_id] = op
         return ops
 
@@ -460,7 +464,6 @@ class OperationRegistry(object):
         data: Union[Path, str, Mapping[str, Any]],
         operation_id: Union[str, None] = None,
     ) -> Operation:
-
         if isinstance(data, Path):
             if not data.is_file():
                 raise Exception(
@@ -471,7 +474,6 @@ class OperationRegistry(object):
                 data.as_posix(), kiara=self._kiara, pipeline_name=operation_id
             )
         elif isinstance(data, Mapping):
-
             pipeline_config = PipelineConfig.from_config(
                 pipeline_name=operation_id, data=data, kiara=self._kiara
             )
@@ -511,8 +513,8 @@ class OperationRegistry(object):
                 f"Can't register pipeline with id '{_operation_id}': operation id already in use."
             )
 
-        manifest = Manifest.construct(
-            module_type="pipeline", module_config=pipeline_config.dict()
+        manifest = Manifest(
+            module_type="pipeline", module_config=pipeline_config.model_dump()
         )
         module = self._kiara.module_registry.create_module(manifest)
 
@@ -537,13 +539,22 @@ class OperationRegistry(object):
             metadata=metadata,
             doc=pipeline_config.doc,
         )
+
+        pc: PipelineConfig = module.config
+        # make sure the pipeline can be created
+        Pipeline(structure=pc.structure, kiara=self._kiara)
+
         operation._module = module
         assert self._operations is not None
         self._operations[_operation_id] = operation
-        current_pipelines = self.operations_by_type.get("pipeline", [])
+        current_pipelines = self.operations_by_type.get("pipeline", None)
+        if not current_pipelines:
+            current_pipelines = []
+            self._operations_by_type["pipeline"] = current_pipelines  # type: ignore
+
         current_pipelines.append(_operation_id)  # type: ignore
         assert self._operations_by_type is not None
-        self._operations_by_type["pipeline"] = sorted(current_pipelines)
+        # self._operations_by_type["pipeline"] = sorted(current_pipelines)
 
         logger.debug("pipeline.registered", operation_id=_operation_id)
         return operation
@@ -554,7 +565,6 @@ class OperationRegistry(object):
         doc: Any,
         metadata: Union[Mapping[str, Any], None] = None,
     ) -> Dict[str, Operation]:
-
         module = self._kiara.module_registry.create_module(manifest)
         op_types = {}
 
@@ -562,7 +572,6 @@ class OperationRegistry(object):
             metadata = {}
 
         for op_name, op_type in self.operation_types.items():
-
             op_details = op_type.check_matching_operation(module=module)
             if not op_details:
                 continue
@@ -583,7 +592,6 @@ class OperationRegistry(object):
         return op_types
 
     def get_operation(self, operation_id: str) -> Operation:
-
         if operation_id not in self.operation_ids:
             if operation_id in self._invalid_operations.keys():
                 raise InvalidOperationException(self._invalid_operations[operation_id])
@@ -597,7 +605,6 @@ class OperationRegistry(object):
         return op
 
     def find_all_operation_types(self, operation_id: str) -> Set[str]:
-
         result = set()
         for op_type, ops in self.operations_by_type.items():
             if operation_id in ops:
@@ -607,13 +614,11 @@ class OperationRegistry(object):
 
     @property
     def operations_by_type(self) -> Mapping[str, Iterable[str]]:
-
         if self._operations_by_type is None:
             self.operations
         return self._operations_by_type  # type: ignore
 
     def find_operation_id(self, manifest: Manifest) -> Union[str, None]:
-
         for op in self.operations.values():
             if manifest.manifest_cid == op.manifest_cid:
                 return op.operation_id
