@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
+import functools
 
 #  Copyright (c) 2021, University of Luxembourg / DHARPA project
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
 import importlib
 import inspect
+import logging
 import os
 import sys
 from pkgutil import iter_modules
@@ -25,8 +26,7 @@ from typing import (
     Union,
 )
 
-from rich.markdown import Markdown
-from stevedore import ExtensionManager
+import structlog
 
 from kiara.utils import (
     _get_all_subclasses,
@@ -36,9 +36,10 @@ from kiara.utils import (
     log_exception,
     log_message,
 )
-from kiara.utils.develop import log_dev_message
 
 if TYPE_CHECKING:
+    from click import Command
+
     from kiara.data_types import DataType
     from kiara.models import KiaraModel
     from kiara.modules import KiaraModule
@@ -46,9 +47,6 @@ if TYPE_CHECKING:
     from kiara.registries import KiaraArchive
     from kiara.renderers import KiaraRenderer
 
-import logging
-
-import structlog
 
 logger = structlog.getLogger()
 
@@ -60,7 +58,6 @@ SUBCLASS_TYPE = TypeVar("SUBCLASS_TYPE")
 
 def _default_id_func(cls: Type) -> str:
     """Utility method to auto-generate a more or less nice looking id_or_alias for a class."""
-
     name = camel_case_to_snake_case(cls.__name__)
     path = cls.__module__
 
@@ -80,8 +77,7 @@ def _default_id_func(cls: Type) -> str:
 
 def _cls_name_id_func(cls: Type) -> str:
     """Utility method to auto-generate a more or less nice looking id_or_alias for a class."""
-
-    name = camel_case_to_snake_case(cls.__name__)
+    name: str = camel_case_to_snake_case(cls.__name__)
     return name
 
 
@@ -89,16 +85,18 @@ def find_subclasses_under(
     base_class: Type[SUBCLASS_TYPE],
     python_module: Union[str, ModuleType],
 ) -> List[Type[SUBCLASS_TYPE]]:
-    """Find all (non-abstract) subclasses of a base class that live under a module (recursively).
+    """
+    Find all (non-abstract) subclasses of a base class that live under a module (recursively).
 
     Arguments:
+    ---------
         base_class: the parent class
         python_module: the Python module to search
 
     Returns:
+    -------
         a list of all subclasses
     """
-
     # if hasattr(sys, "frozen"):
     #     raise NotImplementedError("Pyinstaller bundling not supported yet.")
 
@@ -116,7 +114,6 @@ def find_subclasses_under(
 
     result = []
     for sc in subclasses:
-
         if not sc.__module__.startswith(python_module.__name__):
             continue
 
@@ -135,9 +132,11 @@ def _process_subclass(
     ignore_abstract_classes: bool = True,
     ignore_modules_with_null_module_name: bool = True,
 ) -> Union[str, None]:
-    """Process subclasses of a base class that live under a module (recursively).
+    """
+    Process subclasses of a base class that live under a module (recursively).
 
     Arguments:
+    ---------
         base_class: the parent class
         python_module: the Python module to search
         ignore_abstract_classes: whether to include abstract classes in the result
@@ -148,11 +147,11 @@ def _process_subclass(
         ignore_modules_with_null_module_name: ignore modules that have their '_module_type_name' attribute set to 'None', this is mostly useful to filter out base classes
 
     Returns:
+    -------
         the type id
     """
     is_abstract = inspect.isabstract(sub_class)
     if ignore_abstract_classes and is_abstract:
-
         if sub_class.__dict__.get("_is_abstract", False):
             return None
 
@@ -169,21 +168,26 @@ def _process_subclass(
                 if not hasattr(sub_class, "process"):
                     missing.append("process")
 
-                name = f"{sub_class.__module__}.{sub_class.__name__}"
-                title = "Invalid kiara module"
-                if hasattr(sub_class, "_module_type_name"):
-                    name = f"**{name}** ( *{sub_class._module_type_name}* )"  # type: ignore
-                    title = f"{title} '[i]{sub_class._module_type_name}[/i]'"  # type: ignore
-                msg = f"Invalid kiara module: {name}\n\nMissing method(s):"
-                for m in missing:
-                    msg = f"{msg}\n- *{m}*"
+                if missing:
+                    name = f"{sub_class.__module__}.{sub_class.__name__}"
+                    title = "Invalid kiara module"
+                    if hasattr(sub_class, "_module_type_name"):
+                        name = f"**{name}** ( *{sub_class._module_type_name}* )"  # type: ignore
+                        title = f"{title} '[i]{sub_class._module_type_name}[/i]'"  # type: ignore
+                    msg = f"Invalid kiara module: {name}\n\nMissing method(s):"
+                    for m in missing:
+                        msg = f"{msg}\n- *{m}*"
 
-                log_dev_message(msg=Markdown(msg), title=title)
+                    from rich.markdown import Markdown
+
+                    from kiara.utils.develop import log_dev_message
+
+                    log_dev_message(msg=Markdown(msg), title=title)
 
         log_message(
             "ignore.subclass",
-            sub_class=sub_class,
-            base_class=base_class,
+            sub_class=f"{sub_class.__module__}.{sub_class.__name__}",
+            base_class=f"{base_class.__module__}.{base_class.__name__}",
             reason="subclass is abstract",
         )
         return None
@@ -192,15 +196,14 @@ def _process_subclass(
         type_id_func = _default_id_func
 
     if type_id_key:
-
         if hasattr(sub_class, type_id_key):
-            type_id = getattr(sub_class, type_id_key)
+            type_id: Union[str, None] = getattr(sub_class, type_id_key)
             if type_id is None and ignore_modules_with_null_module_name:
                 log_message(
                     "ignore.subclass",
-                    sub_class=sub_class,
-                    base_class=base_class,
-                    reason=f"'{ type_id_key }' subclass is set to 'None'",
+                    sub_class=f"{sub_class.__module__}.{sub_class.__name__}",
+                    base_class=f"{base_class.__module__}.{base_class.__name__}",
+                    reason=f"'{type_id_key}' subclass is set to 'None'",
                 )
                 return None
             if not type_id and not is_abstract:
@@ -237,9 +240,11 @@ def load_all_subclasses_for_entry_point(
     type_id_no_attach: bool = False,
     attach_python_metadata: Union[bool, str] = False,
 ) -> Dict[str, Type[SUBCLASS_TYPE]]:
-    """Find all subclasses of a base class via package entry points.
+    """
+    Find all subclasses of a base class via package entry points.
 
     Arguments:
+    ---------
         entry_point_name: the entry point name to query entries for
         base_class: the base class to look for
         ignore_abstract_classes: whether to include abstract classes in the result
@@ -248,7 +253,6 @@ def load_all_subclasses_for_entry_point(
         type_id_no_attach: in case you want to use the type_id_key to set the id, but don't want it attached to classes that don't have it, set this to true. In most cases, you won't need this option
         attach_python_metadata: whether to attach a [PythonClass][kiara.models.python_class.PythonClass] metadata model to the class. By default, '_python_class' is used as attribute name if this argument is 'True', If this argument is a string, that will be used as name instead.
     """
-
     log2 = logging.getLogger("stevedore")
     out_hdlr = logging.StreamHandler(sys.stdout)
     out_hdlr.setFormatter(
@@ -266,6 +270,8 @@ def load_all_subclasses_for_entry_point(
 
     log_message("events.loading.entry_points", entry_point_name=entry_point_name)
 
+    from stevedore import ExtensionManager
+
     mgr = ExtensionManager(
         namespace=entry_point_name,
         invoke_on_load=False,
@@ -361,11 +367,12 @@ def load_all_subclasses_for_entry_point(
 
 
 def find_all_kiara_modules() -> Dict[str, Type["KiaraModule"]]:
-    """Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
-
-    TODO
     """
+    Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
 
+    Todo:
+    ----
+    """
     from kiara.modules import KiaraModule
 
     modules = load_all_subclasses_for_entry_point(
@@ -378,10 +385,13 @@ def find_all_kiara_modules() -> Dict[str, Type["KiaraModule"]]:
     result = {}
     # need to test this, since I couldn't add an abstract method to the KiaraModule class itself (mypy complained because it is potentially overloaded)
     for k, cls in modules.items():
-
         if not hasattr(cls, "process"):
             if is_develop():
+                from rich.markdown import Markdown
+
                 msg = f"Invalid kiara module: **{cls.__module__}.{cls.__name__}**\n\nMissing method(s):\n- *process*"
+                from kiara.utils.develop import log_dev_message
+
                 log_dev_message(msg=Markdown(msg))
 
             # TODO: check signature of process method
@@ -398,11 +408,12 @@ def find_all_kiara_modules() -> Dict[str, Type["KiaraModule"]]:
 
 
 def find_all_kiara_model_classes() -> Dict[str, Type["KiaraModel"]]:
-    """Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
-
-    TODO
     """
+    Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
 
+    Todo:
+    ----
+    """
     from kiara.models import KiaraModel
 
     return load_all_subclasses_for_entry_point(
@@ -431,9 +442,9 @@ def find_all_kiara_model_classes() -> Dict[str, Type["KiaraModel"]]:
 #     )
 
 
+@functools.lru_cache(maxsize=1)
 def find_all_archive_types() -> Dict[str, Type["KiaraArchive"]]:
     """Find all [KiaraArchive][kiara.registries.KiaraArchive] subclasses via package entry points."""
-
     from kiara.registries import KiaraArchive
 
     return load_all_subclasses_for_entry_point(
@@ -446,11 +457,12 @@ def find_all_archive_types() -> Dict[str, Type["KiaraArchive"]]:
 
 
 def find_all_data_types() -> Dict[str, Type["DataType"]]:
-    """Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
-
-    TODO
     """
+    Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
 
+    Todo:
+    ----
+    """
     from kiara.data_types import DataType
 
     all_data_types = load_all_subclasses_for_entry_point(
@@ -470,7 +482,6 @@ def find_all_data_types() -> Dict[str, Type["DataType"]]:
 
 
 def find_all_operation_types() -> Dict[str, Type["OperationType"]]:
-
     from kiara.operations import OperationType
 
     result = load_all_subclasses_for_entry_point(
@@ -484,7 +495,6 @@ def find_all_operation_types() -> Dict[str, Type["OperationType"]]:
 def find_kiara_modules_under(
     module: Union[str, ModuleType],
 ) -> List[Type["KiaraModule"]]:
-
     from kiara.modules import KiaraModule
 
     return find_subclasses_under(
@@ -494,9 +504,8 @@ def find_kiara_modules_under(
 
 
 def find_kiara_model_classes_under(
-    module: Union[str, ModuleType]
+    module: Union[str, ModuleType],
 ) -> List[Type["KiaraModel"]]:
-
     from kiara.models import KiaraModel
 
     result = find_subclasses_under(
@@ -522,7 +531,6 @@ def find_kiara_model_classes_under(
 
 
 def find_data_types_under(module: Union[str, ModuleType]) -> List[Type["DataType"]]:
-
     from kiara.data_types import DataType
 
     return find_subclasses_under(
@@ -532,9 +540,8 @@ def find_data_types_under(module: Union[str, ModuleType]) -> List[Type["DataType
 
 
 def find_operations_under(
-    module: Union[str, ModuleType]
+    module: Union[str, ModuleType],
 ) -> List[Type["OperationType"]]:
-
     from kiara.operations import OperationType
 
     return find_subclasses_under(
@@ -544,9 +551,8 @@ def find_operations_under(
 
 
 def find_pipeline_base_path_for_module(
-    module: Union[str, ModuleType]
+    module: Union[str, ModuleType],
 ) -> Union[str, None]:
-
     # if hasattr(sys, "frozen"):
     #     raise NotImplementedError("Pyinstaller bundling not supported yet.")
 
@@ -567,7 +573,6 @@ def find_pipeline_base_path_for_module(
 def find_all_kiara_pipeline_paths(
     skip_errors: bool = False,
 ) -> Dict[str, Union[Dict[str, Any], None]]:
-
     import logging
 
     log2 = logging.getLogger("stevedore")
@@ -581,6 +586,8 @@ def find_all_kiara_pipeline_paths(
 
     log_message("events.loading.pipelines")
 
+    from stevedore import ExtensionManager
+
     mgr = ExtensionManager(
         namespace="kiara.pipelines", invoke_on_load=False, propagate_map_exceptions=True
     )
@@ -588,7 +595,6 @@ def find_all_kiara_pipeline_paths(
     paths: Dict[str, Union[Dict[str, Any], None]] = {}
     # TODO: make sure we load 'core' first?
     for plugin in mgr:
-
         name = plugin.name
         if (
             isinstance(plugin.plugin, tuple)
@@ -650,8 +656,7 @@ def find_all_kiara_pipeline_paths(
     return paths
 
 
-def find_all_cli_subcommands():
-
+def find_all_cli_subcommands() -> Iterable["Command"]:
     entry_point_name = "kiara.cli_subcommands"
     log2 = logging.getLogger("stevedore")
     out_hdlr = logging.StreamHandler(sys.stdout)
@@ -669,6 +674,7 @@ def find_all_cli_subcommands():
         log2.setLevel(logging.INFO)
 
     log_message("events.loading.entry_points", entry_point_name=entry_point_name)
+    from stevedore import ExtensionManager
 
     mgr = ExtensionManager(
         namespace=entry_point_name,
@@ -680,11 +686,12 @@ def find_all_cli_subcommands():
 
 
 def find_all_kiara_renderers() -> Dict[str, Type["KiaraRenderer"]]:
-    """Find all [KiaraComponent][kiara_plugin.streamilt.components.KiaraComponent] subclasses via package entry points.
-
-    TODO
     """
+    Find all [KiaraComponent][kiara_plugin.streamilt.components.KiaraComponent] subclasses via package entry points.
 
+    Todo:
+    ----
+    """
     from kiara.renderers import KiaraRenderer
 
     components = load_all_subclasses_for_entry_point(
@@ -701,7 +708,6 @@ def find_all_kiara_renderers() -> Dict[str, Type["KiaraRenderer"]]:
 def find_kiara_renderers_under(
     module: Union[str, ModuleType],
 ) -> List[Type["KiaraRenderer"]]:
-
     from kiara.renderers import KiaraRenderer
 
     return find_subclasses_under(
@@ -733,12 +739,10 @@ def find_kiara_renderers_under(
 #     # TODO: typecheck?
 #     return _callable_wrapper(func=func)  # type: ignore
 def _import_modules_recursively(module: "ModuleType"):
-
     if not hasattr(module, "__path__"):
         return
 
     for submodule in iter_modules(module.__path__):  # type: ignore
-
         try:
             submodule_mod = importlib.import_module(
                 f"{module.__name__}.{submodule.name}"
