@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+import logging
 import uuid
 from typing import TYPE_CHECKING, Any, List, Mapping
 
@@ -34,9 +35,8 @@ def create_module_preparation_table(
     job_config: JobConfig,
     job_id: uuid.UUID,
     module: "KiaraModule",
-    **render_config: Any
+    **render_config: Any,
 ) -> Table:
-
     dev_config = get_dev_config()
     table = Table(show_header=False, box=box.SIMPLE)
     table.add_column("key", style="i")
@@ -46,22 +46,27 @@ def create_module_preparation_table(
 
     module_details = dev_config.log.pre_run.module_info
     if module_details not in [DetailLevel.NONE.value, DetailLevel.NONE]:
+        pipeline_name = job_config.module_config.get("pipeline_name", None)
         if module_details in [DetailLevel.MINIMAL.value, DetailLevel.MINIMAL]:
             table.add_row("module", job_config.module_type)
+            if pipeline_name:
+                table.add_row("pipeline name", pipeline_name)
             doc = module.operation.doc
             table.add_row(
                 "module desc",
-                doc.description
+                doc.description,
                 # kiara.context_info.module_types.item_infos[
                 #     job_config.module_type
                 # ].documentation.description,
             )
         elif module_details in [DetailLevel.FULL.value, DetailLevel.FULL]:
             table.add_row("module", job_config.module_type)
+            if pipeline_name:
+                table.add_row("pipeline name", pipeline_name)
             doc = module.operation.doc
             table.add_row(
                 "module doc",
-                doc.full_doc
+                doc.full_doc,
                 # kiara.context_info.module_types.item_infos[
                 #     job_config.module_type
                 # ].documentation.full_doc,
@@ -92,15 +97,35 @@ def create_post_run_table(
     job: ActiveJob,
     module: "KiaraModule",
     job_config: JobConfig,
-    **render_config: Any
+    **render_config: Any,
 ) -> Table:
-
     dev_config = get_dev_config()
     table = Table(show_header=False, box=box.SIMPLE)
     table.add_column("key", style="i")
     table.add_column("value")
 
     table.add_row("job_id", str(job.job_id))
+    table.add_row("status", job.status.value)
+    if job.error:
+        table.add_row("error", job.error)
+    if job.job_log.log:
+        start_time = job.job_log.log[0].timestamp
+        last_time = start_time
+        log_table = Table(show_header=False, box=box.SIMPLE_HEAD)
+        log_table.add_column("log", style="i")
+        log_table.add_column("level")
+        log_table.add_column("timestamp")
+        for log in job.job_log.log:
+            log_time = log.timestamp
+            if log_time == start_time:
+                time_str = str(log_time)
+            else:
+                time_str = f"+ {log_time - last_time}"
+            log_level = logging.getLevelName(log.log_level).lower()
+            log_table.add_row(log.msg, log_level, time_str)
+            last_time = log_time
+        table.add_row("duration", f"{last_time - start_time}")
+        table.add_row("logs", log_table)
     module_details = dev_config.log.post_run.module_info
     if module_details not in [DetailLevel.NONE.value, DetailLevel.NONE]:
         if module_details in [DetailLevel.MINIMAL.value, DetailLevel.MINIMAL]:
@@ -119,11 +144,21 @@ def create_post_run_table(
                     module.module_type_name
                 ].documentation.full_doc,
             )
-            if module_config_is_empty(module.config.dict()):
+            if module_config_is_empty(module.config.model_dump()):
                 table.add_row("module_config", "-- no config --")
             else:
                 table.add_row("module_config", module.config)
 
+    if job_config.pipeline_metadata is not None:
+        pm_table = Table(show_header=False, box=box.SIMPLE)
+        pm_table.add_column("key")
+        pm_table.add_column("value")
+        pm_table.add_row("pipeline_id", str(job_config.pipeline_metadata.pipeline_id))
+        pm_table.add_row("step_id", job_config.pipeline_metadata.step_id)
+        table.add_row("pipeline_step_metadata", pm_table)
+    else:
+        table.add_row("pipeline_step_metadata", "-- not a pipeline step --")
+
     inputs_details = dev_config.log.post_run.inputs_info
     if inputs_details not in [DetailLevel.NONE.value, DetailLevel.NONE]:
         if inputs_details in [DetailLevel.MINIMAL, DetailLevel.MINIMAL.value]:
@@ -159,12 +194,10 @@ def create_post_run_table(
 
 
 def terminal_print_manifest(manifest: Manifest):
-
     terminal_print(manifest.create_renderable())
 
 
 def create_value_map_renderable(value_map: Mapping[str, Any], **render_config: Any):
-
     show_type = render_config.get("show_type", True)
 
     rc = dict(DEFAULT_VALUE_MAP_RENDER_CONFIG)
