@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from typing import TYPE_CHECKING, Any, Iterable, Mapping, Union
+from typing import TYPE_CHECKING, Any, Iterable, List, Mapping, Union
 
 from pydantic import BaseModel
 
@@ -16,7 +16,6 @@ def generate_html(
     add_type_column: bool = False,
 ) -> "Airium":
     """Create html representing this models data."""
-
     from airium import Airium  # type: ignore
 
     doc = Airium()
@@ -29,7 +28,6 @@ def generate_html(
     if isinstance(item, str):
         doc(item)
     elif isinstance(item, BaseModel):
-
         from kiara.models import KiaraModel
 
         if isinstance(item, KiaraModel):
@@ -48,12 +46,11 @@ def generate_html(
         props = model_cls.schema().get("properties", {})
 
         rows = []
-        for field_name, field in model_cls.__fields__.items():
-
+        for field_name, field in model_cls.model_fields.items():
             if exclude_fields and field_name in exclude_fields:
                 continue
 
-            row = [field_name]
+            row: List[Any] = [field_name]
 
             p = props.get(field_name, None)
             if add_type_column:
@@ -100,9 +97,8 @@ def generate_html(
                 with doc.tr():
                     doc.td(_t=k)
                     value_el = generate_html(v)
-                    doc.td(_t=value_el)
+                    doc.td(_t=value_el)  # type: ignore
     elif isinstance(item, Iterable):
-
         with doc.ul():
             for i in item:
                 with doc.li():
