@@ -28,7 +28,7 @@ from kiara.utils.output import (
 )
 
 if TYPE_CHECKING:
-    from kiara import ValueMap
+    from kiara.api import ValueMap
     from kiara.context import Kiara
 
 yaml = YAML(typ="safe")
@@ -37,14 +37,12 @@ yaml = YAML(typ="safe")
 def filter_operations(
     kiara: "Kiara", pkg_name: Union[str, None] = None, **operations: "Operation"
 ) -> OperationGroupInfo:
-
     result: Dict[str, OperationInfo] = {}
 
     # op_infos = kiara.operation_registry.get_context_metadata(only_for_package=pkg_name)
     modules = kiara.module_registry.get_context_metadata(only_for_package=pkg_name)
 
     for op_id, op in operations.items():
-
         if op.module.module_type_name != "pipeline":
             if op.module.module_type_name in modules.item_infos.keys():
                 result[op_id] = OperationInfo.create_from_operation(
@@ -72,7 +70,7 @@ def filter_operations(
         #         kiara=kiara, operation=op
         #     )
 
-    return OperationGroupInfo.construct(item_infos=result)  # type: ignore
+    return OperationGroupInfo(item_infos=result)  # type: ignore
 
 
 def create_operation(
@@ -80,7 +78,6 @@ def create_operation(
     operation_config: Union[None, Mapping[str, Any]] = None,
     kiara: Union[None, "Kiara"] = None,
 ) -> Operation:
-
     operation: Union[Operation, None]
 
     if kiara is None:
@@ -91,7 +88,6 @@ def create_operation(
     operation = None
 
     if module_or_operation in kiara.operation_registry.operation_ids:
-
         operation = kiara.operation_registry.get_operation(module_or_operation)
         if operation_config:
             if module_or_operation in kiara.module_type_names:
@@ -109,7 +105,6 @@ def create_operation(
         module_or_operation != "pipeline"
         and module_or_operation in kiara.module_type_names
     ):
-
         if operation_config is None:
             operation_config = {}
         manifest = Manifest(
@@ -136,7 +131,9 @@ def create_operation(
             execution_context=execution_context,
         )
 
-        manifest = kiara.create_manifest("pipeline", config=pipeline_config.dict())
+        manifest = kiara.create_manifest(
+            "pipeline", config=pipeline_config.model_dump()
+        )
         module = kiara.module_registry.create_module(manifest=manifest)
 
         operation = Operation.create_from_module(module, doc=pipeline_config.doc)
@@ -164,7 +161,6 @@ def create_operation(
             d = dict(data)
             pipeline_name = d.pop("pipeline_name", None)
             if pipeline_name is not None:
-
                 execution_context = ExecutionContext(
                     pipeline_dir=os.path.abspath(os.path.dirname(module_or_operation))
                 )
@@ -176,7 +172,7 @@ def create_operation(
                 )
 
                 manifest = kiara.create_manifest(
-                    "pipeline", config=pipeline_config.dict()
+                    "pipeline", config=pipeline_config.model_dump()
                 )
                 module = kiara.module_registry.create_module(manifest=manifest)
 
@@ -187,7 +183,6 @@ def create_operation(
                 raise Exception("Invalid pipeline config, missing 'pipeline_name' key.")
 
         if operation is None:
-
             if module_or_operation == "pipeline":
                 msg = "Can't assemble pipeline."
             else:
@@ -199,7 +194,6 @@ def create_operation(
             )
 
     if operation is None:
-
         merged = set(kiara.module_type_names)
         merged.update(kiara.operation_registry.operation_ids)
         raise NoSuchExecutionTargetException(
@@ -213,9 +207,9 @@ def create_operation(
 def create_operation_status_renderable(
     operation: Operation, inputs: Union["ValueMap", None], render_config: Any
 ) -> RenderableType:
-
     show_operation_name = render_config.get("show_operation_name", True)
     show_operation_doc = render_config.get("show_operation_doc", True)
+    show_only_description = render_config.get("show_only_description", True)
     show_inputs = render_config.get("show_inputs", False)
     show_outputs_schema = render_config.get("show_outputs_schema", False)
     show_headers = render_config.get("show_headers", True)
@@ -226,7 +220,10 @@ def create_operation_status_renderable(
         items.append(f"Operation: [bold]{operation.operation_id}[/bold]")
     if show_operation_doc and operation.doc.is_set:
         items.append("")
-        items.append(Markdown(operation.doc.full_doc, style="i"))
+        if show_only_description:
+            items.append(Markdown(operation.doc.description))
+        else:
+            items.append(Markdown(operation.doc.full_doc))
 
     if show_inputs:
         assert inputs is not None
