@@ -18,13 +18,13 @@ from kiara.defaults import (
     VALID_PIPELINE_FILE_EXTENSIONS,
 )
 from kiara.exceptions import InvalidOperationException, NoSuchOperationException
-from kiara.models.module.pipeline.value_refs import StepValueAddress
 from kiara.utils import log_exception
 from kiara.utils.files import get_data_from_file
 
 if typing.TYPE_CHECKING:
     from kiara.context import Kiara
     from kiara.models.module.pipeline import PipelineConfig
+    from kiara.models.module.pipeline.value_refs import StepValueAddress
     from kiara.modules.included_core_modules.pipeline import PipelineModule
 
 logger = structlog.get_logger()
@@ -34,6 +34,7 @@ def create_step_value_address(
     value_address_config: Union[str, Mapping[str, Any]],
     default_field_name: str,
 ) -> "StepValueAddress":
+    from kiara.models.module.pipeline.value_refs import StepValueAddress
 
     if isinstance(value_address_config, StepValueAddress):
         return value_address_config
@@ -41,7 +42,6 @@ def create_step_value_address(
     sub_value: Union[Mapping[str, Any], None] = None
 
     if isinstance(value_address_config, str):
-
         tokens = value_address_config.split(".")
         if len(tokens) == 1:
             step_id = value_address_config
@@ -57,7 +57,6 @@ def create_step_value_address(
             raise NotImplementedError()
 
     elif isinstance(value_address_config, Mapping):
-
         step_id = value_address_config["step_id"]
         output_name = value_address_config["value_name"]
         sub_value = value_address_config.get("sub_value", None)
@@ -80,7 +79,6 @@ def create_step_value_address(
 def ensure_step_value_addresses(
     link: Union[str, Mapping, Iterable], default_field_name: str
 ) -> List["StepValueAddress"]:
-
     if isinstance(link, (str, Mapping)):
         input_links: List[StepValueAddress] = [
             create_step_value_address(
@@ -106,15 +104,16 @@ def get_pipeline_details_from_path(
     module_type_name: Union[str, None] = None,
     base_module: Union[str, None] = None,
 ) -> Dict[str, Any]:
-    """Load a pipeline description, save it's content, and determine it the pipeline base name.
+    """
+    Load a pipeline description, save it's content, and determine it the pipeline base name.
 
     Arguments:
+    ---------
         path: the path to the pipeline file
         module_type_name: if specifies, overwrites any auto-detected or assigned pipeline name
         base_module: overrides the base module the assembled pipeline module will be located in the python hierarchy
 
     """
-
     if isinstance(path, str):
         path = Path(os.path.expanduser(path))
 
@@ -143,7 +142,6 @@ def get_pipeline_details_from_path(
 
 
 def check_doc_sidecar(path: Union[Path, str], data: Dict[str, Any]) -> Dict[str, Any]:
-
     if isinstance(path, str):
         path = Path(os.path.expanduser(path))
 
@@ -161,7 +159,8 @@ def check_doc_sidecar(path: Union[Path, str], data: Dict[str, Any]) -> Dict[str,
 def get_pipeline_config(
     pipeline: str, kiara: typing.Union["Kiara", None] = None
 ) -> "PipelineConfig":
-    """Extract a pipeline config from the item specified.
+    """
+    Extract a pipeline config from the item specified.
 
     The lookup of the 'pipeline' reference happens in this order (first match returns the result):
     - check whether there is an operation with that name that is a pipeline
@@ -175,7 +174,6 @@ def get_pipeline_config(
     Returns:
         a pipeline config object
     """
-
     if kiara is None:
         from kiara.context import Kiara
 
@@ -225,19 +223,20 @@ def get_pipeline_config(
 
 
 def find_pipeline_data_in_paths(
-    pipeline_paths: Dict[str, Union[Dict[str, Any], None]]
+    pipeline_paths: Dict[str, Union[Dict[str, Any], None]],
 ) -> Mapping[str, Mapping[str, Any]]:
-    """Find pipeline data in the provided paths.
+    """
+    Find pipeline data in the provided paths.
 
     The 'pipeline_paths' argument has a local path as key, and a mapping as value that contains optional metadata about the context for all the pipelines that are found under the path.
 
     Arguments:
+    ---------
         pipeline_paths: a mapping of pipeline names to paths
     """
-
     all_pipelines = []
 
-    for _path in pipeline_paths.keys():
+    for _path, _md in pipeline_paths.items():
         path = Path(_path)
         if not path.exists():
             logger.warning(
@@ -246,10 +245,12 @@ def find_pipeline_data_in_paths(
             continue
 
         elif path.is_dir():
-
             for root, dirnames, filenames in os.walk(path, topdown=True):
-
-                dirnames[:] = [d for d in dirnames if d not in DEFAULT_EXCLUDE_DIRS]
+                dirnames[:] = [
+                    d
+                    for d in dirnames
+                    if d not in DEFAULT_EXCLUDE_DIRS and not d.startswith(".")
+                ]
 
                 for filename in [
                     f
@@ -257,14 +258,11 @@ def find_pipeline_data_in_paths(
                     if os.path.isfile(os.path.join(root, f))
                     and any(f.endswith(ext) for ext in VALID_PIPELINE_FILE_EXTENSIONS)
                 ]:
-
                     full_path = os.path.join(root, filename)
                     try:
-
                         data = get_pipeline_details_from_path(path=full_path)
                         data = check_doc_sidecar(full_path, data)
                         existing_metadata = data.pop("metadata", {})
-                        _md = pipeline_paths[_path]
                         if _md is None:
                             md = {}
                         else:
@@ -284,7 +282,6 @@ def find_pipeline_data_in_paths(
             data = get_pipeline_details_from_path(path=path)
             data = check_doc_sidecar(path, data)
             existing_metadata = data.pop("metadata", {})
-            _md = pipeline_paths[_path]
             if _md is None:
                 md = {}
             else:
@@ -305,3 +302,32 @@ def find_pipeline_data_in_paths(
         pipelines[name] = pipeline
 
     return pipelines
+
+
+def extract_data_to_hash_from_pipeline_config(
+    pipeline_config: Mapping[str, Any],
+) -> Mapping[str, Any]:
+    if "steps" not in pipeline_config:
+        return pipeline_config
+
+    step_configs = []
+    # TODO: make sure order is unique so hashes are stable?
+    for step in pipeline_config["steps"]:
+        step_module_config = extract_data_to_hash_from_pipeline_config(
+            step["module_config"]
+        )
+        data = {
+            "module_type": step["module_type"],
+            "module_config": step_module_config,
+            "step_id": step["step_id"],
+        }
+        step_configs.append(data)
+
+    result = {
+        "steps": step_configs,
+        "constants": pipeline_config.get("constants", {}),
+        "defaults": pipeline_config.get("defaults", {}),
+        "input_aliases": pipeline_config.get("input_aliases", {}),
+        "output_aliases": pipeline_config.get("output_aliases", {}),
+    }
+    return result
